@@ -93,8 +93,7 @@ bool CUNIT::CanAttack(int otherUnit) const {
 	return false;
 }
 
-bool CUNIT::CanAttackMe(int otherUnit) const {
-	otherUnit = otherUnit;
+bool CUNIT::CanAttackMe(int /*otherUnit*/) const {
 	// TODO: the function above, in reverse
 	return true;
 }
@@ -656,3 +655,14 @@ bool CUNIT::SetMaxSpeed(float speed) const {
 
 	return true;
 }
+
+bool CUNIT::IsCommander(const UnitDef* udef)
+{
+	assert(udef != NULL);
+	auto search = udef->customParams.find("iscommander");
+	if (search != udef->customParams.end()) {
+		return search->second == "1";
+	}
+	return false;
+}
+
