@@ -46,7 +46,7 @@ function AttackerBehaviour:UnitBuilt(unit)
 	end
 end
 
-function AttackerBehaviour:UnitDamaged(unit,attacker)
+function AttackerBehaviour:UnitDamaged(unit,attacker,damage)
 	if unit.engineID == self.unit.engineID then
 		self.damaged = game:Frame()
 	end
@@ -187,4 +187,4 @@ function AttackerBehaviour:SetMoveState()
 			thisUnit:Internal():ExecuteCustomCommand(CMD_MOVE_STATE, floats)
 		end
 	end
-end
\ No newline at end of file
+end
