@@ -47,7 +47,7 @@ function CountHandler:InitializeNameCounts()
 	end
 end
 
-function CountHandler:UnitDamaged(unit, attacker)
+function CountHandler:UnitDamaged(unit, attacker,damage)
 	local aname = "nil"
 	if attacker then 
 		if attacker:Team() ~= game:GetTeamID() then
@@ -61,4 +61,4 @@ function CountHandler:UnitDead(unit)
 	if unit:Team() ~= game:GetTeamID() then
 		EchoDebug("enemy unit died")
 	end
-end
\ No newline at end of file
+end
