@@ -619,7 +619,7 @@ local function UpdateDebug()
 end
 
 --[[
-function TargetHandler:UnitDamaged(unit, attacker)
+function TargetHandler:UnitDamaged(unit, attacker,damage)
 	-- even if the attacker can't be seen, human players know what weapons look like
 	-- but attacker is nil if it's an enemy unit, so this is useless
 	if attacker ~= nil then
@@ -1181,4 +1181,4 @@ function TargetHandler:RaiderHere(raidbehaviour)
 		end
 	end
 	self.raiderCounted[raidbehaviour.id] = true -- reset with UpdateMap()
-end
\ No newline at end of file
+end
