@@ -123,33 +123,27 @@ function CheckForMapControl()
 		end
 		
 		lastSiegeCheckFrame = f
-		local Metal = game:GetResourceByName("Metal")
-		if Metal.reserves < 0.5 * Metal.capacity and ai.wreckCount > 0 then
-			ai.needToReclaim = true
-		else
-			ai.needToReclaim = false
-		end
+		ai.needToReclaim = ai.Metal.full < 0.5 and ai.wreckCount > 0
 		AAUnitPerTypeLimit = math.ceil(ai.turtlehandler:GetTotalPriority() / 4)
-		heavyPlasmaLimit = math.ceil(ai.combatCount / 7)
-		nukeLimit = math.ceil(ai.combatCount / 20)
-		tacticalNukeLimit = math.ceil(ai.combatCount / 12)
+		heavyPlasmaLimit = math.ceil(ai.combatCount / 10)
+		nukeLimit = math.ceil(ai.combatCount / 50)
+		tacticalNukeLimit = math.ceil(ai.combatCount / 40)
 
 		local attackCounter = ai.attackhandler:GetCounter()
 		local couldAttack = ai.couldAttack >= 1 or ai.couldBomb >= 1
 		local bombingTooExpensive = ai.bomberhandler:GetCounter() == maxBomberCounter
 		local attackTooExpensive = attackCounter == maxAttackCounter
-		local plentyOfCombatUnits = ai.combatCount > attackCounter * 0.9
 		local controlMetalSpots = ai.mexCount > #ai.mobNetworkMetals["air"][1] * 0.4
-		local needUpgrade = plentyOfCombatUnits or couldAttack or bombingTooExpensive or attackTooExpensive
-		local lotsOfMetal = Metal.income > 20 or controlMetalSpots
+		local needUpgrade = couldAttack or bombingTooExpensive or attackTooExpensive
+		local lotsOfMetal = ai.Metal.income > 25 or controlMetalSpots
 
 		EchoDebug(ai.totalEnemyThreat .. " " .. ai.totalEnemyImmobileThreat .. " " .. ai.totalEnemyMobileThreat)
 		-- build siege units if the enemy is turtling, if a lot of our attackers are getting destroyed, or if we control over 40% of the metal spots
 		needSiege = (ai.totalEnemyImmobileThreat > ai.totalEnemyMobileThreat * 3.5 and ai.totalEnemyImmobileThreat > 50000) or attackCounter >= siegeAttackCounter or controlMetalSpots
-		ai.needAdvanced = (Metal.income > 10 or controlMetalSpots) and ai.factories > 0 and (needUpgrade or lotsOfMetal)
+		ai.needAdvanced = (ai.Metal.income > 10 or controlMetalSpots) and ai.factories > 0 and (needUpgrade or lotsOfMetal)
 		ai.needExperimental = false
 		ai.needNukes = false
-		if Metal.income > 50 and ai.haveAdvFactory and needUpgrade and ai.enemyBasePosition then
+		if ai.Metal.income > 50 and ai.haveAdvFactory and needUpgrade and ai.enemyBasePosition then
 			if not ai.haveExpFactory then
 				for i, factory in pairs(ai.factoriesAtLevel[ai.maxFactoryLevel]) do
 					if ai.maphandler:MobilityNetworkHere("bot", factory.position) == ai.maphandler:MobilityNetworkHere("bot", ai.enemyBasePosition) then
@@ -161,10 +155,10 @@ function CheckForMapControl()
 			ai.needNukes = true
 		end
 		EchoDebug("need experimental? " .. tostring(ai.needExperimental) .. ", need nukes? " .. tostring(ai.needNukes) .. ", have advanced? " .. tostring(ai.haveAdvFactory) .. ", need upgrade? " .. tostring(needUpgrade) .. ", have enemy base position? " .. tostring(ai.enemyBasePosition))
-		EchoDebug("metal income: " .. Metal.income .. "  combat units: " .. ai.combatCount)
+		EchoDebug("metal income: " .. ai.Metal.income .. "  combat units: " .. ai.combatCount)
 		EchoDebug("have advanced? " .. tostring(ai.haveAdvFactory) .. " have experimental? " .. tostring(ai.haveExpFactory))
 		EchoDebug("need advanced? " .. tostring(ai.needAdvanced) .. "  need experimental? " .. tostring(ai.needExperimental))
-		EchoDebug("need advanced? " .. tostring(ai.needAdvanced) .. ", need upgrade? " .. tostring(needUpgrade) .. ", have attacked enough? " .. tostring(couldAttack) .. " (" .. ai.couldAttack .. "), have " .. ai.factories .. " factories, " .. math.floor(Metal.income) .. " metal income")
+		EchoDebug("need advanced? " .. tostring(ai.needAdvanced) .. ", need upgrade? " .. tostring(needUpgrade) .. ", have attacked enough? " .. tostring(couldAttack) .. " (" .. ai.couldAttack .. "), have " .. ai.factories .. " factories, " .. math.floor(ai.Metal.income) .. " metal income")
 	end
 end
 
@@ -248,8 +242,7 @@ end
 
 function BuildWindSolarIfNeeded()
 	-- check if we need power
-	local res = game:GetResourceByName("Energy")
-	if res.income < res.usage then
+	if ai.Energy.extra < 0 then
 		retVal = WindSolar
 		EchoDebug("BuildWindSolarIfNeeded: income "..res.income..", usage "..res.usage..", building more energy")
 	else
@@ -274,61 +267,62 @@ end
 
 -- build conversion or storage
 function DoSomethingForTheEconomy(self)
-	local Energy = game:GetResourceByName("Energy")
-	local extraE = Energy.income - Energy.usage
-	local Metal = game:GetResourceByName("Metal")
-	local extraM = Metal.income - Metal.usage
+	local highEnergy = ai.Energy.full > 0.9
+	local lowEnergy = ai.Energy.full < 0.1
+	local highMetal = ai.Metal.full > 0.9
+	local lowMetal = ai.Metal.full < 0.1
 	local isWater = unitTable[self.unit:Internal():Name()].needsWater
 	local unitName = DummyUnitName
 	-- maybe we need conversion?
-	if extraE > 60 and extraM < 0 and Energy.income > 300 then
+	if ai.Energy.extra > 80 and highEnergy and lowMetal and ai.Metal.extra < 0 and ai.Energy.income > 300 then
+		local converterLimit = math.min(math.floor(ai.Energy.income / 200), 4)
 		if isWater then
 			if ai.mySide == CORESideName then
-				unitName = BuildWithLimitedNumber("corfmkr", 8)
+				unitName = BuildWithLimitedNumber("corfmkr", converterLimit)
 			else
-				unitName = BuildWithLimitedNumber("armfmkr", 8)
+				unitName = BuildWithLimitedNumber("armfmkr", converterLimit)
 			end		
 		else
 			if ai.mySide == CORESideName then
-				unitName = BuildWithLimitedNumber("cormakr", 8)
+				unitName = BuildWithLimitedNumber("cormakr", converterLimit)
 			else
-				unitName = BuildWithLimitedNumber("armmakr", 8)
+				unitName = BuildWithLimitedNumber("armmakr", converterLimit)
 			end
 		end
 	end
 	-- maybe we need storage?
 	if unitName == DummyUnitName then
 		-- energy storage
-		if Energy.reserves >= 0.9 * Energy.capacity and extraE > 100 then
+		if ai.Energy.extra > 150 and highEnergy and not lowMetal then
 			if isWater then
 				if ai.mySide == CORESideName then
-					unitName = BuildWithLimitedNumber("coruwes", 3)
+					unitName = BuildWithLimitedNumber("coruwes", 2)
 				else
-					unitName = BuildWithLimitedNumber("armuwes", 3)
+					unitName = BuildWithLimitedNumber("armuwes", 2)
 				end	
 			else
 				if ai.mySide == CORESideName then
-					unitName = BuildWithLimitedNumber("corestor", 3)
+					unitName = BuildWithLimitedNumber("corestor", 2)
 				else
-					unitName = BuildWithLimitedNumber("armestor", 3)
+					unitName = BuildWithLimitedNumber("armestor", 2)
 				end
 			end
 		end
 	end
 	if unitName == DummyUnitName then
 		-- metal storage
-		if Metal.reserves >= 0.9 * Metal.capacity and extraM > 3 then
+		if ai.Metal.extra > 5 and highMetal and highEnergy then
 			if isWater then
 				if ai.mySide == CORESideName then
-					unitName = BuildWithLimitedNumber("coruwms", 3)
+					unitName = BuildWithLimitedNumber("coruwms", 2)
 				else
-					unitName = BuildWithLimitedNumber("armuwms", 3)
+					unitName = BuildWithLimitedNumber("armuwms", 2)
 				end	
 			else
 				if ai.mySide == CORESideName then
-					unitName = BuildWithLimitedNumber("cormstor", 3)
+					unitName = BuildWithLimitedNumber("cormstor", 2)
 				else
-					unitName = BuildWithLimitedNumber("armmstor", 3)
+					unitName = BuildWithLimitedNumber("armmstor", 2)
 				end
 			end
 		end
@@ -340,33 +334,36 @@ end
 
 -- build advanced conversion or storage
 function DoSomethingAdvancedForTheEconomy(self)
-	local Energy = game:GetResourceByName("Energy")
-	local extraE = Energy.income - Energy.usage
-	local Metal = game:GetResourceByName("Metal")
-	local extraM = Metal.income - Metal.usage
+	local highEnergy = ai.Energy.full > 0.9
+	local lowEnergy = ai.Energy.full < 0.1
+	local highMetal = ai.Metal.full > 0.9
+	local lowMetal = ai.Metal.full < 0.1
 	local unitName = self.unit:Internal():Name()
 	local isWater = unitTable[unitName].needsWater or seaplaneConList[unitName]
 	local unitName = DummyUnitName
 	-- maybe we need conversion?
-	if extraE > 600 and extraM < 0 and Energy.income > 2000 then
+	if ai.Energy.extra > 800 and highEnergy and lowMetal and ai.Metal.extra < 0 and ai.Energy.income > 2000 then
+		local converterLimit = math.floor(ai.Energy.income / 1000)
 		if isWater then
 			if ai.mySide == CORESideName then
-				unitName = BuildWithLimitedNumber("armuwmmm", Energy.income / 1000)
+				unitName = BuildWithLimitedNumber("armuwmmm", converterLimit)
 			else
-				unitName = BuildWithLimitedNumber("armuwmmm", Energy.income / 1000)
+				unitName = BuildWithLimitedNumber("armuwmmm", converterLimit)
 			end		
 		else
 			if ai.mySide == CORESideName then
-				unitName = BuildWithLimitedNumber("cormmkr", Energy.income / 1000)
+				unitName = BuildWithLimitedNumber("cormmkr", converterLimit)
 			else
-				unitName = BuildWithLimitedNumber("armmmkr", Energy.income / 1000)
+				unitName = BuildWithLimitedNumber("armmmkr", converterLimit)
 			end
 		end
 	end
+	-- building big storage is a waste
+	--[[
 	-- maybe we need storage?
 	if unitName == DummyUnitName then
 		-- energy storage
-		if Energy.reserves >= 0.9 * Energy.capacity and extraE > 1000 then
+		if ai.Energy.extra > 1500 and highEnergy and not lowMetal then
 			if ai.mySide == CORESideName then
 				unitName = BuildWithLimitedNumber("coruwadves", 1)
 			else
@@ -376,7 +373,7 @@ function DoSomethingAdvancedForTheEconomy(self)
 	end
 	if unitName == DummyUnitName then
 		-- metal storage
-		if Metal.reserves >= 0.9 * Metal.capacity and extraM > 10 then
+		if ai.Metal.extra > 25 and highMetal and highEnergy then
 			if ai.mySide == CORESideName then
 				unitName = BuildWithLimitedNumber("coruwadvms", 1)
 			else
@@ -384,6 +381,7 @@ function DoSomethingAdvancedForTheEconomy(self)
 			end	
 		end
 	end
+	]]--
 
 	return unitName
 end
@@ -411,7 +409,9 @@ end
 function BuildSiegeIfNeeded(unitName)
 	if unitName == DummyUnitName then return DummyUnitName end
 	if IsSiegeEquipmentNeeded() then
-		return BuildWithLimitedNumber(unitName, math.ceil((ai.battleCount + ai.breakthroughCount) * 0.3))
+		if ai.siegeCount < (ai.battleCount + ai.breakthroughCount) * 0.35 then
+			return unitName
+		end
 	end
 	return DummyUnitName
 end
@@ -428,6 +428,7 @@ function BuildBreakthroughIfNeeded(unitName)
 			return DummyUnitName
 		end
 	else
+		if ai.battleCount <= minBattleCount then return DummyUnitName end
 		local attackCounter = ai.attackhandler:GetCounter(mtype)
 		if attackCounter == maxAttackCounter then
 			return unitName
@@ -626,6 +627,13 @@ end
 function BuildRaiderIfNeeded(unitName)
 	if unitName == DummyUnitName or unitName == nil then return DummyUnitName end
 	local mtype = unitTable[unitName].mtype
+	if ai.factoriesAtLevel[3] ~= nil and ai.factoriesAtLevel[3] ~= {} then
+		-- if we have a level 2 factory, don't build raiders until we have some battle units
+		local attackCounter = ai.attackhandler:GetCounter(mtype)
+		if ai.battleCount + ai.breakthroughCount < attackCounter / 2 then
+			return DummyUnitName
+		end
+	end
 	local counter = ai.raidhandler:GetCounter(mtype)
 	if counter == minRaidCounter then return DummyUnitName end
 	if ai.raiderCount[mtype] == nil then
@@ -765,11 +773,15 @@ function BuildBattleIfNeeded(unitName)
 	local mtype = unitTable[unitName].mtype
 	local attackCounter = ai.attackhandler:GetCounter(mtype)
 	EchoDebug(mtype .. " " .. attackCounter .. " " .. maxAttackCounter)
-	if attackCounter == maxAttackCounter then return DummyUnitName end
+	if attackCounter == maxAttackCounter and ai.battleCount > minBattleCount then return DummyUnitName end
 	if mtype == "veh" and ai.mySide == CORESideName and (ai.factoriesAtLevel[1] == nil or ai.factoriesAtLevel[1] == {}) then
 		-- core only has a lvl1 vehicle raider, so this prevents getting stuck
 		return unitName
 	end
+	if ai.factoriesAtLevel[3] ~= nil and ai.factoriesAtLevel[3] ~= {} then
+		-- if we have a level 2 factory, don't wait to build raiders first
+		return unitName
+	end
 	local raidCounter = ai.raidhandler:GetCounter(mtype)
 	EchoDebug(mtype .. " " .. raidCounter .. " " .. maxRaidCounter)
 	if raidCounter == minRaidCounter then return unitName end
@@ -1193,47 +1205,6 @@ local function ConHover()
 	end
 end
 
--- how many of our own unitName there are in a radius around a position
-function CountOwnUnitsInRadius(unitName, pos, radius, maxCount)
-	local ownUnits = game:GetFriendlies()
-	local unitCount = 0
-	-- optimisation: there is always 0 null units on map
-	if unitName == DummyUnitName then
-		return 0
-	end
-	for _, u in pairs(ownUnits) do
-		if u:Name() == unitName then
-			local upos = u:GetPosition()
-			if Distance(pos, upos) < radius then
-				unitCount = unitCount + 1
-			end
-			-- optimisation: if the limit is already exceeded, don't count further
-			if unitCount >= maxCount then
-				break
-			end
-		end
-	end
-	return unitCount
-end
-
-local function CheckAreaLimit(unitName, builder, unitLimit, range)
-	-- this is special case, it means the unit will not be built anyway
-	if unitName == DummyUnitName then
-		return unitName
-	end
-	local pos = builder:GetPosition()
-	if range == nil then range = AreaCheckRange end
-	-- now check how many of the wanted unit is nearby
-	local NumberOfUnits = CountOwnUnitsInRadius(unitName, pos, range, unitLimit)
-	local AllowBuilding = NumberOfUnits < unitLimit
-	EchoDebug(""..unitName.." wanted, with range limit of "..unitLimit..", with "..NumberOfUnits.." already there. The check is: "..tostring(AllowBuilding))
-	if AllowBuilding then
-		return unitName
-	else
-		return DummyUnitName
-	end
-end
-
 local function GroundDefenseIfNeeded(unitName, builder)
 	if not ai.needGroundDefense then
 		return DummyUnitName
@@ -1795,20 +1766,6 @@ local function CheckMySideIfNeeded()
 	end
 end
 
-local function CheckForOwnUnit(name)
-	local ownUnits = game:GetFriendlies()
-	for _, u in pairs(ownUnits) do
-		local un = u:Name()
-		if un == name then
-			local ut = u:Team()
-			if ut == ownTeamID then
-				return true
-			end
-		end
-	end
-	return false
-end
-
 local function BuildAppropriateFactory()
 	return FactoryUnitName
 end
@@ -2012,22 +1969,22 @@ local anyConShip = {
 }
 
 local anyAdvConUnit = {
+	BuildAppropriateFactory,
+	BuildFusion,
+	BuildNukeIfNeeded,
+	BuildAdvancedRadar,
+	BuildHeavyPlasma,
+	BuildAntinuke,
 	BuildLvl2PopUp,
 	BuildHeavyAA,
-	BuildAntinuke,
 	BuildLvl2Plasma,
 	BuildTachyon,
-	BuildHeavyPlasma,
-	BuildTacticalNuke,
-	BuildFusion,
-	BuildAdvancedRadar,
-	BuildLvl2Jammer,
-	BuildAppropriateFactory,
-	BuildNukeIfNeeded,
+	-- BuildTacticalNuke,
 	BuildExtraHeavyAA,
+	BuildLvl2Jammer,
 	BuildMohoGeo,
 	BuildMohoMex,
-	DoSomethingAdvancedForTheEconomy,
+	-- DoSomethingAdvancedForTheEconomy,
 }
 
 local anyConSeaplane = {
@@ -2037,7 +1994,7 @@ local anyConSeaplane = {
 	BuildAdvancedSonar,
 	BuildHeavyTorpedo,
 	BuildAppropriateFactory,
-	DoSomethingAdvancedForTheEconomy,
+	-- DoSomethingAdvancedForTheEconomy,
 }
 
 local anyAdvConSub = {
@@ -2046,7 +2003,7 @@ local anyAdvConSub = {
 	BuildUWFusion,
 	BuildAdvancedSonar,
 	BuildHeavyTorpedo,
-	DoSomethingAdvancedForTheEconomy,
+	-- DoSomethingAdvancedForTheEconomy,
 }
 
 local anyNavalEngineer = {
