@@ -886,10 +886,6 @@ function gadgetHandler:GotChatMsg(msg, player)
     end
   end
 
-  if (IsSyncedCode()) then
-    SendToUnsynced(player, msg)
-  end
-
   return false
 end
 
@@ -1161,6 +1157,16 @@ function gadgetHandler:AllowDirectUnitControl(unitID, unitDefID, unitTeam,
 end
 
 
+function gadgetHandler:AllowBuilderHoldFire(unitID, unitDefID, action)
+  for _,g in ipairs(self.AllowBuilderHoldFire) do
+    if (not AllowBuilderHoldFire(unitID, unitDefID, action)) then
+      return false
+    end
+  end
+  return true
+end
+
+
 function gadgetHandler:MoveCtrlNotify(unitID, unitDefID, unitTeam, data)
   local state = false
   for _,g in ipairs(self.MoveCtrlNotifyList) do
@@ -1187,13 +1193,22 @@ end
 
 
 function gadgetHandler:AllowWeaponTargetCheck(attackerID, attackerWeaponNum, attackerWeaponDefID)
+	local ignore = true
 	for _, g in ipairs(self.AllowWeaponTargetCheckList) do
-		if (not g:AllowWeaponTargetCheck(attackerID, attackerWeaponNum, attackerWeaponDefID)) then
-			return false
+		local allowCheck, ignoreCheck = g:AllowWeaponTargetCheck(attackerID, attackerWeaponNum, attackerWeaponDefID)
+		if not ignoreCheck then
+			ignore = false
+			if not allowCheck then
+				return 0
+			end
 		end
 	end
-
-	return true
+	
+	if ignore then
+		return -1
+	else
+		return 1
+	end
 end
 
 function gadgetHandler:AllowWeaponTarget(attackerID, targetID, attackerWeaponNum, attackerWeaponDefID, defPriority)
@@ -1339,6 +1354,17 @@ function gadgetHandler:UnitDamaged(
   end
 end
 
+function gadgetHandler:UnitStunned(
+  unitID,
+  unitDefID,
+  unitTeam,
+  stunned
+)
+  for _,g in ipairs(self.UnitStunnedList) do
+    g:UnitStunned(unitID, unitDefID, unitTeam, stunned)
+  end
+end
+
 
 function gadgetHandler:UnitTaken(unitID, unitDefID, unitTeam, newTeam)
   for _,g in ipairs(self.UnitTakenList) do
