@@ -17,9 +17,9 @@
 CAIInterfaceLibrary::CAIInterfaceLibrary(const CAIInterfaceLibraryInfo& _info)
 		: interfaceId(-1)
 		, initialized(false)
+		, sAIInterfaceLibrary({ NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL})
 		, info(_info)
 {
-	sAIInterfaceLibrary = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
 
 	libFilePath = FindLibFile();
 
