@@ -77,34 +77,36 @@ void CEngineOutHandler::HandleAIException(const char* description) {
 /////////////////////////////
 
 
-CEngineOutHandler* CEngineOutHandler::singleton = NULL;
+static CEngineOutHandler* singleton = NULL;
+static unsigned int numInstances = 0;
 
 CEngineOutHandler* CEngineOutHandler::GetInstance() {
-	static unsigned int numInstances = 0;
-
-	if (singleton == NULL) {
-		numInstances += 1;
-		singleton = new CEngineOutHandler();
-	}
-
 	// if more than one instance, some code called eoh->func()
 	// and created another after Destroy was already executed
 	// from ~CGame --> usually dtors
 	assert(numInstances == 1);
 	return singleton;
 }
+
+void CEngineOutHandler::Create() {
+	if (singleton == NULL) {
+		singleton = new CEngineOutHandler();
+		numInstances += 1;
+	}
+}
+
 void CEngineOutHandler::Destroy() {
 	if (singleton != NULL) {
 		singleton->PreDestroy();
 
-		CEngineOutHandler* tmp = singleton;
-		singleton = NULL;
-		delete tmp;
-
+		SafeDelete(singleton);
 		IAILibraryManager::Destroy();
+
+		numInstances -= 1;
 	}
 }
 
+
 CEngineOutHandler::~CEngineOutHandler() {
 	// id_skirmishAI should be empty already, but this can not hurt
 	for (id_ai_t::iterator ai = id_skirmishAI.begin(); ai != id_skirmishAI.end(); ++ai) {
@@ -565,18 +567,18 @@ void CEngineOutHandler::CreateSkirmishAI(const size_t skirmishAIId) {
 				(unpauseAfterAIInit ?
 				 " The game is auto-unpaused as soon as the AI is ready." :
 				 ""));
-		net->Send(CBaseNetProtocol::Get().SendPause(gu->myPlayerNum, true));
+		clientNet->Send(CBaseNetProtocol::Get().SendPause(gu->myPlayerNum, true));
 	}*/
 
 	const SkirmishAIData* aiData = skirmishAIHandler.GetSkirmishAI(skirmishAIId);
 
 	if (aiData->status != SKIRMAISTATE_RELOADING) {
-		net->Send(CBaseNetProtocol::Get().SendAIStateChanged(gu->myPlayerNum, skirmishAIId, SKIRMAISTATE_INITIALIZING));
+		clientNet->Send(CBaseNetProtocol::Get().SendAIStateChanged(gu->myPlayerNum, skirmishAIId, SKIRMAISTATE_INITIALIZING));
 	}
 
 	if (aiData->isLuaAI) {
 		// currently, we need doing nothing for Lua AIs
-		net->Send(CBaseNetProtocol::Get().SendAIStateChanged(gu->myPlayerNum, skirmishAIId, SKIRMAISTATE_ALIVE));
+		clientNet->Send(CBaseNetProtocol::Get().SendAIStateChanged(gu->myPlayerNum, skirmishAIId, SKIRMAISTATE_ALIVE));
 	} else {
 		CSkirmishAIWrapper* aiWrapper = NULL;
 		try {
@@ -609,7 +611,7 @@ void CEngineOutHandler::CreateSkirmishAI(const size_t skirmishAIId) {
 				u = uNext;
 			}
 
-			net->Send(CBaseNetProtocol::Get().SendAIStateChanged(gu->myPlayerNum, skirmishAIId, SKIRMAISTATE_ALIVE));
+			clientNet->Send(CBaseNetProtocol::Get().SendAIStateChanged(gu->myPlayerNum, skirmishAIId, SKIRMAISTATE_ALIVE));
 		}
 	}
 }
@@ -654,7 +656,7 @@ void CEngineOutHandler::DestroySkirmishAI(const size_t skirmishAIId) {
 		delete aiWrapper;
 		aiWrapper = NULL;
 
-		net->Send(CBaseNetProtocol::Get().SendAIStateChanged(gu->myPlayerNum, skirmishAIId, SKIRMAISTATE_DEAD));
+		clientNet->Send(CBaseNetProtocol::Get().SendAIStateChanged(gu->myPlayerNum, skirmishAIId, SKIRMAISTATE_DEAD));
 	} CATCH_AI_EXCEPTION;
 }
 
