@@ -29,6 +29,8 @@ class CEngineOutHandler : public CObject {
 
 public:
 	static CEngineOutHandler* GetInstance();
+
+	static void Create();
 	static void Destroy();
 
 	void PostLoad();
@@ -139,9 +141,6 @@ public:
 			throw;												\
 		}
 
-private:
-	static CEngineOutHandler* singleton;
-
 private:
 	typedef std::vector<unsigned char> ids_t;
 	typedef std::map<unsigned char, CSkirmishAIWrapper*> id_ai_t;
