@@ -4,6 +4,7 @@
 #include "OverviewController.h"
 
 #include "Map/Ground.h"
+#include "Map/ReadMap.h"
 #include "Game/UI/MiniMap.h"
 #include "Game/UI/MouseHandler.h"
 #include "Sim/Misc/GlobalConstants.h"
@@ -14,8 +15,8 @@ COverviewController::COverviewController()
 	enabled = false;
 	minimizeMinimap = false;
 
-	pos.x = gs->mapx * 0.5f * SQUARE_SIZE;
-	pos.z = gs->mapy * 0.5f * SQUARE_SIZE;
+	pos.x = mapDims.mapx * 0.5f * SQUARE_SIZE;
+	pos.z = mapDims.mapy * 0.5f * SQUARE_SIZE;
 	const float height = std::max(pos.x / globalRendering->aspectRatio, pos.z);
 	pos.y = CGround::GetHeightAboveWater(pos.x, pos.z, false) + (2.5f * height);
 
@@ -59,7 +60,7 @@ float3 COverviewController::SwitchFrom() const
 	return rpos;
 }
 
-void COverviewController::SwitchTo(bool showText)
+void COverviewController::SwitchTo(const int oldCam, const bool showText)
 {
 	if (showText) {
 		LOG("Switching to Overview style camera");
