@@ -6,7 +6,6 @@
 #include "GameSetup.h"
 #include "Game/GlobalUnsynced.h"
 #include "Lua/LuaUI.h"
-#include "Lua/LuaRules.h"
 #include "Map/Ground.h"
 #include "Map/MapDamage.h"
 #include "Map/ReadMap.h"
@@ -38,11 +37,10 @@
 #include "Sim/Weapons/Weapon.h"
 #include "System/EventHandler.h"
 #include "System/myMath.h"
-#include "System/Sound/SoundChannels.h"
+#include "System/Sound/ISoundChannels.h"
 #include "System/Sync/SyncTracer.h"
 
 #define NUM_WAITING_DAMAGE_LISTS 128
-#define PLAY_SOUNDS 1
 
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
@@ -118,11 +116,11 @@ void CGameHelper::DoExplosionDamage(
 	const float3& volPos = vol->GetWorldSpacePos(unit, lapPos);
 
 	// linear damage falloff with distance
-	const float expDist = vol->GetPointSurfaceDistance(unit, lap, expPos);
+	const float expDist = (expRadius != 0.0f) ? vol->GetPointSurfaceDistance(unit, lap, expPos) : 0.0f;
 	const float expRim = expDist * expEdgeEffect;
 
 	// return early if (distance > radius)
-	if (expDist >= expRadius)
+	if (expDist > expRadius)
 		return;
 
 	// expEdgeEffect should be in [0, 1], so expRadius >= expDist >= expDist*expEdgeEffect
@@ -130,7 +128,7 @@ void CGameHelper::DoExplosionDamage(
 
 	// expMod will also be in [0, 1], no negatives
 	// TODO: damage attenuation for underwater units from surface explosions?
-	const float expDistanceMod = (expRadius - expDist) / (expRadius + 0.01f - expRim);
+	const float expDistanceMod = (expRadius + 0.001f - expDist) / (expRadius + 0.001f - expRim);
 	const float modImpulseScale = CalcImpulseScale(damages, expDistanceMod);
 
 	// NOTE: if an explosion occurs right underneath a
@@ -157,6 +155,7 @@ void CGameHelper::DoExplosionDamage(
 
 void CGameHelper::DoExplosionDamage(
 	CFeature* feature,
+	CUnit* owner,
 	const float3& expPos,
 	const float expRadius,
 	const float expEdgeEffect,
@@ -169,28 +168,27 @@ void CGameHelper::DoExplosionDamage(
 	const CollisionVolume* vol = feature->GetCollisionVolume(NULL);
 	const float3& volPos = vol->GetWorldSpacePos(feature, ZeroVector);
 
-	const float expDist = vol->GetPointSurfaceDistance(feature, NULL, expPos);
+	const float expDist = (expRadius != 0.0f) ? vol->GetPointSurfaceDistance(feature, NULL, expPos) : 0.0f;
 	const float expRim = expDist * expEdgeEffect;
 
-	if (expDist >= expRadius)
+	if (expDist > expRadius)
 		return;
 
 	assert(expRadius >= expRim);
 
-	const float expDistanceMod = (expRadius - expDist) / (expRadius + 0.01f - expRim);
+	const float expDistanceMod = (expRadius + 0.001f - expDist) / (expRadius + 0.001f - expRim);
 	const float modImpulseScale = CalcImpulseScale(damages, expDistanceMod);
 
 	const float3 impulseDir = (volPos - expPos).SafeNormalize();
 	const float3 expImpulse = impulseDir * modImpulseScale;
 
-	feature->DoDamage(damages * expDistanceMod, expImpulse, NULL, weaponDefID, projectileID);
+	feature->DoDamage(damages * expDistanceMod, expImpulse, owner, weaponDefID, projectileID);
 }
 
 
 
 void CGameHelper::DamageObjectsInExplosionRadius(
 	const ExplosionParams& params,
-	const float3& expPos,
 	const float expRad,
 	const int weaponDefID
 ) {
@@ -205,7 +203,7 @@ void CGameHelper::DamageObjectsInExplosionRadius(
 	const unsigned int oldNumUnits = *(cache.GetNumUnitsPtr());
 	const unsigned int oldNumFeatures = *(cache.GetNumFeaturesPtr());
 
-	quadField->GetUnitsAndFeaturesColVol(expPos, expRad, units, features, cache.GetNumUnitsPtr(), cache.GetNumFeaturesPtr());
+	quadField->GetUnitsAndFeaturesColVol(params.pos, expRad, units, features, cache.GetNumUnitsPtr(), cache.GetNumFeaturesPtr());
 
 	const unsigned int newNumUnits = *(cache.GetNumUnitsPtr());
 	const unsigned int newNumFeatures = *(cache.GetNumFeaturesPtr());
@@ -217,20 +215,18 @@ void CGameHelper::DamageObjectsInExplosionRadius(
 	//   not keep track of end-markers --> certain objects
 	//   would not be damaged AT ALL (!)
 	for (unsigned int n = oldNumUnits; n < newNumUnits; n++) {
-		DoExplosionDamage(units[n], params.owner, expPos, expRad, params.explosionSpeed, params.edgeEffectiveness, params.ignoreOwner, params.damages, weaponDefID, params.projectileID);
+		DoExplosionDamage(units[n], params.owner, params.pos, expRad, params.explosionSpeed, params.edgeEffectiveness, params.ignoreOwner, params.damages, weaponDefID, params.projectileID);
 	}
 
 	// damage all features within the explosion radius
 	for (unsigned int n = oldNumFeatures; n < newNumFeatures; n++) {
-		DoExplosionDamage(features[n], expPos, expRad, params.edgeEffectiveness, params.damages, weaponDefID, params.projectileID);
+		DoExplosionDamage(features[n], params.owner, params.pos, expRad, params.edgeEffectiveness, params.damages, weaponDefID, params.projectileID);
 	}
 
 	cache.Reset(oldNumUnits, oldNumFeatures);
 }
 
 void CGameHelper::Explosion(const ExplosionParams& params) {
-	const float3 expDir = params.dir;
-	const float3 expPos = params.pos;
 	const DamageArray& damages = params.damages;
 
 	// if weaponDef is NULL, this is a piece-explosion
@@ -244,15 +240,15 @@ void CGameHelper::Explosion(const ExplosionParams& params) {
 	const float craterAOE = std::max(1.0f, params.craterAreaOfEffect);
 	const float damageAOE = std::max(1.0f, params.damageAreaOfEffect);
 
-	const float realHeight = ground->GetHeightReal(expPos.x, expPos.z);
-	const float altitude = expPos.y - realHeight;
+	const float realHeight = CGround::GetHeightReal(params.pos);
+	const float altitude = (params.pos).y - realHeight;
 
 	// NOTE: event triggers before damage is applied to objects
-	const bool noGfx = eventHandler.Explosion(weaponDefID, params.projectileID, expPos, params.owner);
+	const bool noGfx = eventHandler.Explosion(weaponDefID, params.projectileID, params.pos, params.owner);
 
 	if (luaUI != NULL) {
 		if (weaponDef != NULL && weaponDef->cameraShake > 0.0f) {
-			luaUI->ShockFront(expPos, weaponDef->cameraShake, damageAOE);
+			luaUI->ShockFront(params.pos, weaponDef->cameraShake, damageAOE);
 		}
 	}
 
@@ -261,8 +257,8 @@ void CGameHelper::Explosion(const ExplosionParams& params) {
 			DoExplosionDamage(
 				params.hitUnit,
 				params.owner,
-				expPos,
-				damageAOE,
+				params.pos,
+				0.0f,
 				params.explosionSpeed,
 				params.edgeEffectiveness,
 				params.ignoreOwner,
@@ -275,8 +271,9 @@ void CGameHelper::Explosion(const ExplosionParams& params) {
 		if (params.hitFeature != NULL) {
 			DoExplosionDamage(
 				params.hitFeature,
-				expPos,
-				damageAOE,
+				params.owner,
+				params.pos,
+				0.0f,
 				params.edgeEffectiveness,
 				params.damages,
 				weaponDefID,
@@ -284,7 +281,7 @@ void CGameHelper::Explosion(const ExplosionParams& params) {
 			);
 		}
 	} else {
-		DamageObjectsInExplosionRadius(params, expPos, damageAOE, weaponDefID);
+		DamageObjectsInExplosionRadius(params, damageAOE, weaponDefID);
 
 		// deform the map if the explosion was above-ground
 		// (but had large enough radius to touch the ground)
@@ -296,7 +293,7 @@ void CGameHelper::Explosion(const ExplosionParams& params) {
 				const float craterStrength = (damageDepth + damages.craterBoost) * damages.craterMult;
 				const float craterRadius = craterAOE - altitude;
 
-				mapDamage->Explosion(expPos, craterStrength, craterRadius);
+				mapDamage->Explosion(params.pos, craterStrength, craterRadius);
 			}
 		}
 	}
@@ -304,8 +301,8 @@ void CGameHelper::Explosion(const ExplosionParams& params) {
 	if (!noGfx) {
 		explGenHandler->GenExplosion(
 			explosionID,
-			expPos,
-			expDir,
+			params.pos,
+			params.dir,
 			damages.GetDefaultDamage(),
 			damageAOE,
 			params.gfxMod,
@@ -314,24 +311,22 @@ void CGameHelper::Explosion(const ExplosionParams& params) {
 		);
 	}
 
-	CExplosionEvent explosionEvent(expPos, damages.GetDefaultDamage(), damageAOE, weaponDef);
+	CExplosionEvent explosionEvent(params.pos, damages.GetDefaultDamage(), damageAOE, weaponDef);
 	CExplosionCreator::FireExplosionEvent(explosionEvent);
 
-	#if (PLAY_SOUNDS == 1)
 	if (weaponDef != NULL) {
 		const GuiSoundSet& soundSet = weaponDef->hitSound;
 
-		const unsigned int soundFlags = CCustomExplosionGenerator::GetFlagsFromHeight(expPos.y, altitude);
+		const unsigned int soundFlags = CCustomExplosionGenerator::GetFlagsFromHeight(params.pos.y, realHeight);
 		const unsigned int soundMask = CCustomExplosionGenerator::SPW_WATER | CCustomExplosionGenerator::SPW_UNDERWATER;
 
 		const int soundNum = ((soundFlags & soundMask) != 0);
 		const int soundID = soundSet.getID(soundNum);
 
 		if (soundID > 0) {
-			Channels::Battle.PlaySample(soundID, expPos, soundSet.getVolume(soundNum));
+			Channels::Battle->PlaySample(soundID, params.pos, soundSet.getVolume(soundNum));
 		}
 	}
-	#endif
 }
 
 
@@ -359,25 +354,20 @@ void CGameHelper::Explosion(const ExplosionParams& params) {
 template<typename TFilter, typename TQuery>
 static inline void QueryUnits(TFilter filter, TQuery& query)
 {
-	GML_RECMUTEX_LOCK(qnum); // QueryUnits
-
-	const vector<int> &quads = quadField->GetQuads(query.pos, query.radius);
-
+	const auto& quads = quadField->GetQuads(query.pos, query.radius);
 	const int tempNum = gs->tempNum++;
 
-	for (vector<int>::const_iterator qi = quads.begin(); qi != quads.end(); ++qi) {
-		const CQuadField::Quad& quad = quadField->GetQuad(*qi);
-		for (int t = 0; t < teamHandler->ActiveAllyTeams(); ++t) {
-			if (!filter.Team(t)) {
-				continue;
-			}
-			std::list<CUnit*>::const_iterator ui;
-			const std::list<CUnit*>& allyTeamUnits = quad.teamUnits[t];
-			for (ui = allyTeamUnits.begin(); ui != allyTeamUnits.end(); ++ui) {
-				if ((*ui)->tempNum != tempNum) {
-					(*ui)->tempNum = tempNum;
-					if (filter.Unit(*ui)) {
-						query.AddUnit(*ui);
+	for (int t = 0; t < teamHandler->ActiveAllyTeams(); ++t) { //FIXME
+		if (!filter.Team(t)) {
+			continue;
+		}
+		for (const int qi: quads) {
+			const std::vector<CUnit*>& allyTeamUnits = quadField->GetQuad(qi).teamUnits[t];
+			for (CUnit* u: allyTeamUnits) {
+				if (u->tempNum != tempNum) {
+					u->tempNum = tempNum;
+					if (filter.Unit(u)) {
+						query.AddUnit(u);
 					}
 				}
 			}
@@ -484,7 +474,7 @@ namespace {
 			}
 		};
 
-	}; // end of namespace Filter
+	} // end of namespace Filter
 
 
 	namespace Query {
@@ -632,77 +622,62 @@ namespace {
 			}
 		};
 
-	}; // end of namespace Query
-}; // end of namespace
+	} // end of namespace Query
+} // end of namespace
 
 // Use this instead of unit->tempNum here, because it requires a mutex lock that will deadlock if luaRules is invoked simultaneously.
 // Not the cleanest solution, but faster than e.g. a std::set, and this function is called quite frequently.
 static int tempTargetUnits[MAX_UNITS] = {0};
 static int targetTempNum = 2;
 
-void CGameHelper::GenerateWeaponTargets(const CWeapon* weapon, const CUnit* lastTargetUnit, std::multimap<float, CUnit*>& targets)
+void CGameHelper::GenerateWeaponTargets(const CWeapon* weapon, const CUnit* avoidUnit, std::multimap<float, CUnit*>& targets)
 {
-	const CUnit* attacker = weapon->owner;
+	const CUnit* owner    = weapon->owner;
 	const float radius    = weapon->range;
-	const float3& pos     = attacker->pos;
-	const float heightMod = weapon->heightMod;
-	const float aHeight   = weapon->weaponPos.y;
+	const float3& pos     = owner->pos;
+	const float aHeight   = weapon->aimFromPos.y;
+	const CUnit* lastAttacker = ((owner->lastAttackFrame + 200) <= gs->frameNum) ? owner->lastAttacker : nullptr;
 
 	const WeaponDef* weaponDef = weapon->weaponDef;
+	const float heightMod = weaponDef->heightmod;
 
 	// how much damage the weapon deals over 1 second
 	const float secDamage = weaponDef->damages.GetDefaultDamage() * weapon->salvoSize / weapon->reloadTime * GAME_SPEED;
 	const bool paralyzer  = (weaponDef->damages.paralyzeDamageTime != 0);
 
-	const std::vector<int>& quads = quadField->GetQuads(pos, radius + (aHeight - std::max(0.0f, readMap->GetInitMinHeight())) * heightMod);
-
+	const auto& quads = quadField->GetQuads(pos, radius + (aHeight - std::max(0.0f, readMap->GetInitMinHeight())) * heightMod);
 	const int tempNum = targetTempNum++;
 
-	typedef std::vector<int>::const_iterator VectorIt;
-	typedef std::list<CUnit*>::const_iterator ListIt;
+	for (int t = 0; t < teamHandler->ActiveAllyTeams(); ++t) {
+		if (teamHandler->Ally(owner->allyteam, t)) {
+			continue;
+		}
+		for (const int qi: quads) {
+			const std::vector<CUnit*>& allyTeamUnits = quadField->GetQuad(qi).teamUnits[t];
 
-	for (VectorIt qi = quads.begin(); qi != quads.end(); ++qi) {
-		for (int t = 0; t < teamHandler->ActiveAllyTeams(); ++t) {
-			if (teamHandler->Ally(attacker->allyteam, t)) {
-				continue;
-			}
-
-			const std::list<CUnit*>& allyTeamUnits = quadField->GetQuad(*qi).teamUnits[t];
-
-			for (ListIt ui = allyTeamUnits.begin(); ui != allyTeamUnits.end(); ++ui) {
-				CUnit* targetUnit = *ui;
+			for (CUnit* targetUnit: allyTeamUnits) {
 				float targetPriority = 1.0f;
 
-				if (!(targetUnit->category & weapon->onlyTargetCategory)) {
-					continue;
-				}
-				if (targetUnit->GetTransporter() != NULL) {
-					if (!modInfo.targetableTransportedUnits)
-						continue;
-					// the transportee might be "hidden" below terrain, in which case we can't target it
-					if (targetUnit->pos.y < ground->GetHeightReal(targetUnit->pos.x, targetUnit->pos.z))
-						continue;
-				}
 				if (tempTargetUnits[targetUnit->id] == tempNum) {
 					continue;
 				}
-
 				tempTargetUnits[targetUnit->id] = tempNum;
 
-				if (targetUnit->IsUnderWater() && !weaponDef->waterweapon) {
+				if (!weapon->TestTarget(float3(), SWeaponTarget(targetUnit))) {
 					continue;
 				}
-				if (targetUnit->isDead) {
-					continue;
+
+				if (targetUnit == avoidUnit) {
+					targetPriority *= 10.0f;
 				}
 
 				float3 targPos;
-				const unsigned short targetLOSState = targetUnit->losStatus[attacker->allyteam];
+				const unsigned short targetLOSState = targetUnit->losStatus[owner->allyteam];
 
 				if (targetLOSState & LOS_INLOS) {
 					targPos = targetUnit->aimPos;
 				} else if (targetLOSState & LOS_INRADAR) {
-					targPos = targetUnit->aimPos + (targetUnit->posErrorVector * radarHandler->GetAllyTeamRadarErrorSize(attacker->allyteam));
+					targPos = weapon->GetUnitPositionWithError(targetUnit);
 					targetPriority *= 10.0f;
 				} else {
 					continue;
@@ -710,7 +685,7 @@ void CGameHelper::GenerateWeaponTargets(const CWeapon* weapon, const CUnit* last
 
 				const float modRange = radius + (aHeight - targPos.y) * heightMod;
 
-				if ((pos - targPos).SqLength2D() > modRange * modRange) {
+				if (pos.SqDistance2D(targPos) > modRange * modRange) {
 					continue;
 				}
 
@@ -723,10 +698,6 @@ void CGameHelper::GenerateWeaponTargets(const CWeapon* weapon, const CUnit* last
 				if (targetLOSState & LOS_INLOS) {
 					targetPriority *= (secDamage + targetUnit->health);
 
-					if (targetUnit == lastTargetUnit) {
-						targetPriority *= weapon->avoidTarget ? 10.0f : 0.4f;
-					}
-
 					if (paralyzer && targetUnit->paralyzeDamage > (modInfo.paralyzeOnMaxHealth? targetUnit->maxHealth: targetUnit->health)) {
 						targetPriority *= 4.0f;
 					}
@@ -747,12 +718,13 @@ void CGameHelper::GenerateWeaponTargets(const CWeapon* weapon, const CUnit* last
 					if (targetUnit->IsCrashing()) {
 						targetPriority *= 1000.0f;
 					}
+					if (targetUnit == lastAttacker) {
+						targetPriority *= 0.5f;
+					}
 				}
 
-				if (luaRules != NULL) {
-					if (!luaRules->AllowWeaponTarget(attacker->id, targetUnit->id, weapon->weaponNum, weaponDef->id, &targetPriority)) {
-						continue;
-					}
+				if (!eventHandler.AllowWeaponTarget(owner->id, targetUnit->id, weapon->weaponNum, weaponDef->id, &targetPriority)) {
+					continue;
 				}
 
 				targets.insert(std::pair<float, CUnit*>(targetPriority, targetUnit));
@@ -762,7 +734,7 @@ void CGameHelper::GenerateWeaponTargets(const CWeapon* weapon, const CUnit* last
 
 #ifdef TRACE_SYNC
 	{
-		tracefile << "[GenerateWeaponTargets] attackerID, attackRadius: " << attacker->id << ", " << radius << " ";
+		tracefile << "[GenerateWeaponTargets] ownerID, attackRadius: " << owner->id << ", " << radius << " ";
 
 		for (std::multimap<float, CUnit*>::const_iterator ti = targets.begin(); ti != targets.end(); ++ti)
 			tracefile << "\tpriority: " << (ti->first) <<  ", targetID: " << (ti->second)->id <<  " ";
@@ -870,15 +842,17 @@ float3 CGameHelper::Pos2BuildPos(const BuildInfo& buildInfo, bool synced)
 {
 	float3 pos;
 
+	static const int HALFMAP_SQ = SQUARE_SIZE * 2;
+
 	if (buildInfo.GetXSize() & 2)
-		pos.x = math::floor((buildInfo.pos.x              ) / (SQUARE_SIZE * 2)) * SQUARE_SIZE * 2 + SQUARE_SIZE;
+		pos.x = math::floor((buildInfo.pos.x              ) / (HALFMAP_SQ)) * HALFMAP_SQ + SQUARE_SIZE;
 	else
-		pos.x = math::floor((buildInfo.pos.x + SQUARE_SIZE) / (SQUARE_SIZE * 2)) * SQUARE_SIZE * 2;
+		pos.x = math::floor((buildInfo.pos.x + SQUARE_SIZE) / (HALFMAP_SQ)) * HALFMAP_SQ;
 
 	if (buildInfo.GetZSize() & 2)
-		pos.z = math::floor((buildInfo.pos.z              ) / (SQUARE_SIZE * 2)) * SQUARE_SIZE * 2 + SQUARE_SIZE;
+		pos.z = math::floor((buildInfo.pos.z              ) / (HALFMAP_SQ)) * HALFMAP_SQ + SQUARE_SIZE;
 	else
-		pos.z = math::floor((buildInfo.pos.z + SQUARE_SIZE) / (SQUARE_SIZE * 2)) * SQUARE_SIZE * 2;
+		pos.z = math::floor((buildInfo.pos.z + SQUARE_SIZE) / (HALFMAP_SQ)) * HALFMAP_SQ;
 
 	pos.y = CGameHelper::GetBuildHeight(pos, buildInfo.def, synced);
 	return pos;
@@ -945,14 +919,14 @@ float3 CGameHelper::ClosestBuildSite(int team, const UnitDef* unitDef, float3 po
 			bool good = true;
 
 			int z2Min = std::max(       0, zs - (zsize    ) / 2 - minDist);
-			int z2Max = std::min(gs->mapy, zs + (zsize + 1) / 2 + minDist);
+			int z2Max = std::min(mapDims.mapy, zs + (zsize + 1) / 2 + minDist);
 			int x2Min = std::max(       0, xs - (xsize    ) / 2 - minDist);
-			int x2Max = std::min(gs->mapx, xs + (xsize + 1) / 2 + minDist);
+			int x2Max = std::min(mapDims.mapx, xs + (xsize + 1) / 2 + minDist);
 
 			// check for nearby blocking features
 			for (int z2 = z2Min; z2 < z2Max; ++z2) {
 				for (int x2 = x2Min; x2 < x2Max; ++x2) {
-					CSolidObject* solObj = groundBlockingObjectMap->GroundBlockedUnsafe(z2 * gs->mapx + x2);
+					CSolidObject* solObj = groundBlockingObjectMap->GroundBlockedUnsafe(z2 * mapDims.mapx + x2);
 
 					if (solObj && solObj->immobile && !dynamic_cast<CFeature*>(solObj)) {
 						good = false;
@@ -963,22 +937,20 @@ float3 CGameHelper::ClosestBuildSite(int team, const UnitDef* unitDef, float3 po
 
 			if (good) {
 				z2Min = std::max(       0, zs - (zsize    ) / 2 - minDist - 2);
-				z2Max = std::min(gs->mapy, zs + (zsize + 1) / 2 + minDist + 2);
+				z2Max = std::min(mapDims.mapy, zs + (zsize + 1) / 2 + minDist + 2);
 				x2Min = std::max(       0, xs - (xsize    ) / 2 - minDist - 2);
-				x2Max = std::min(gs->mapx, xs + (xsize + 1) / 2 + minDist + 2);
+				x2Max = std::min(mapDims.mapx, xs + (xsize + 1) / 2 + minDist + 2);
 
 				// check for nearby factories with open yards
 				for (int z2 = z2Min; z2 < z2Max; ++z2) {
 					for (int x2 = x2Min; x2 < x2Max; ++x2) {
-						CSolidObject* solObj = groundBlockingObjectMap->GroundBlockedUnsafe(z2 * gs->mapx + x2);
+						CSolidObject* solObj = groundBlockingObjectMap->GroundBlockedUnsafe(z2 * mapDims.mapx + x2);
 
 						if (solObj == NULL)
 							continue;
 						if (!solObj->immobile)
 							continue;
-						if (dynamic_cast<CFactory*>(solObj) == NULL)
-							continue;
-						if (!static_cast<CFactory*>(solObj)->opening)
+						if (!solObj->yardOpen)
 							continue;
 
 						good = false;
@@ -1004,7 +976,7 @@ float CGameHelper::GetBuildHeight(const float3& pos, const UnitDef* unitdef, boo
 	// (so we do not care about maxHeightDif constraints either)
 	// TODO: maybe respect waterline if <pos> is in water
 	if (!unitdef->IsImmobileUnit())
-		return (std::max(pos.y, ground->GetHeightReal(pos.x, pos.z, synced)));
+		return (std::max(pos.y, CGround::GetHeightReal(pos.x, pos.z, synced)));
 
 	const float* orgHeightMap = readMap->GetOriginalHeightMapSynced();
 	const float* curHeightMap = readMap->GetCornerHeightMapSynced();
@@ -1031,15 +1003,15 @@ float CGameHelper::GetBuildHeight(const float3& pos, const UnitDef* unitdef, boo
 	const int px = (pos.x - (xsize * (SQUARE_SIZE >> 1))) / SQUARE_SIZE;
 	const int pz = (pos.z - (zsize * (SQUARE_SIZE >> 1))) / SQUARE_SIZE;
 	// top-left and bottom-right footprint corner (clamped)
-	const int x1 = std::min(gs->mapx, std::max(0, px));
-	const int z1 = std::min(gs->mapy, std::max(0, pz));
-	const int x2 = std::max(0, std::min(gs->mapx, x1 + xsize));
-	const int z2 = std::max(0, std::min(gs->mapy, z1 + zsize));
+	const int x1 = std::min(mapDims.mapx, std::max(0, px));
+	const int z1 = std::min(mapDims.mapy, std::max(0, pz));
+	const int x2 = std::max(0, std::min(mapDims.mapx, x1 + xsize));
+	const int z2 = std::max(0, std::min(mapDims.mapy, z1 + zsize));
 
 	for (int x = x1; x <= x2; x++) {
 		for (int z = z1; z <= z2; z++) {
-			const float sqOrgHgt = orgHeightMap[z * gs->mapxp1 + x];
-			const float sqCurHgt = curHeightMap[z * gs->mapxp1 + x];
+			const float sqOrgHgt = orgHeightMap[z * mapDims.mapxp1 + x];
+			const float sqCurHgt = curHeightMap[z * mapDims.mapxp1 + x];
 			const float sqMinHgt = std::min(sqCurHgt, sqOrgHgt);
 			const float sqMaxHgt = std::max(sqCurHgt, sqOrgHgt);
 
@@ -1089,9 +1061,8 @@ CGameHelper::BuildSquareStatus CGameHelper::TestUnitBuildSquare(
 
 	const int x1 = int(pos.x / SQUARE_SIZE) - (xsize >> 1), x2 = x1 + xsize;
 	const int z1 = int(pos.z / SQUARE_SIZE) - (zsize >> 1), z2 = z1 + zsize;
-
-	const int2 xrange = int2(x1, x2);
-	const int2 zrange = int2(z1, z2);
+	int2 xrange = int2(x1, x2);
+	int2 zrange = int2(z1, z2);
 
 	const MoveDef* moveDef = (buildInfo.def->pathType != -1U) ? moveDefHandler->GetMoveDefByPathType(buildInfo.def->pathType) : NULL;
 	/*const S3DModel* model =*/ buildInfo.def->LoadModel();
@@ -1130,14 +1101,15 @@ CGameHelper::BuildSquareStatus CGameHelper::TestUnitBuildSquare(
 
 		for (int z = z1; z < z2; z++) {
 			for (int x = x1; x < x2; x++) {
-				BuildSquareStatus tbs = TestBuildSquare(float3(x * SQUARE_SIZE, buildHeight, z * SQUARE_SIZE), xrange, zrange, buildInfo.def, moveDef, feature, gu->myAllyTeam, synced);
+				const float3 bpos(x * SQUARE_SIZE, buildHeight, z * SQUARE_SIZE);
+				BuildSquareStatus tbs = (bpos.IsInBounds()) ? TestBuildSquare(bpos, xrange, zrange, buildInfo.def, moveDef, feature, gu->myAllyTeam, synced) : BUILDSQUARE_BLOCKED;
 
 				if (tbs != BUILDSQUARE_BLOCKED) {
 					// test if build-position overlaps a queued command
 					for (std::vector<Command>::const_iterator ci = commands->begin(); ci != commands->end(); ++ci) {
 						BuildInfo bc(*ci);
-						if (std::max(bc.pos.x - x * SQUARE_SIZE - SQUARE_SIZE, x * SQUARE_SIZE - bc.pos.x) * 2 < bc.GetXSize() * SQUARE_SIZE &&
-							std::max(bc.pos.z - z * SQUARE_SIZE - SQUARE_SIZE, z * SQUARE_SIZE - bc.pos.z) * 2 < bc.GetZSize() * SQUARE_SIZE) {
+						if (std::max(bc.pos.x - bpos.x - SQUARE_SIZE, bpos.x - bc.pos.x) * 2 < bc.GetXSize() * SQUARE_SIZE &&
+							std::max(bc.pos.z - bpos.z - SQUARE_SIZE, bpos.z - bc.pos.z) * 2 < bc.GetZSize() * SQUARE_SIZE) {
 							tbs = BUILDSQUARE_BLOCKED;
 							break;
 						}
@@ -1146,14 +1118,14 @@ CGameHelper::BuildSquareStatus CGameHelper::TestUnitBuildSquare(
 
 				switch (tbs) {
 					case BUILDSQUARE_OPEN:
-						canbuildpos->push_back(float3(x * SQUARE_SIZE, buildHeight, z * SQUARE_SIZE));
+						canbuildpos->push_back(bpos);
 						break;
 					case BUILDSQUARE_RECLAIMABLE:
 					case BUILDSQUARE_OCCUPIED:
-						featurepos->push_back(float3(x * SQUARE_SIZE, buildHeight, z * SQUARE_SIZE));
+						featurepos->push_back(bpos);
 						break;
 					case BUILDSQUARE_BLOCKED:
-						nobuildpos->push_back(float3(x * SQUARE_SIZE, buildHeight, z * SQUARE_SIZE));
+						nobuildpos->push_back(bpos);
 						break;
 				}
 
@@ -1161,6 +1133,10 @@ CGameHelper::BuildSquareStatus CGameHelper::TestUnitBuildSquare(
 			}
 		}
 	} else {
+		// out of map?
+		if (unsigned(x1) > mapDims.mapx || unsigned(x2) > mapDims.mapx || unsigned(z1) > mapDims.mapy || unsigned(z2) > mapDims.mapy)
+			return BUILDSQUARE_BLOCKED;
+
 		// this can be called in either context
 		for (int z = z1; z < z2; z++) {
 			for (int x = x1; x < x2; x++) {
@@ -1186,17 +1162,31 @@ CGameHelper::BuildSquareStatus CGameHelper::TestBuildSquare(
 	int allyteam,
 	bool synced
 ) {
-	if (!pos.IsInMap())
+	assert(pos.IsInBounds());
+
+	const int sqx = unsigned(pos.x) / SQUARE_SIZE;
+	const int sqz = unsigned(pos.z) / SQUARE_SIZE;
+	const float groundHeight = CGround::GetApproximateHeightUnsafe(sqx, sqz, synced);
+
+	if (!unitDef->CheckTerrainConstraints(moveDef, groundHeight))
 		return BUILDSQUARE_BLOCKED;
 
-	const int yardxpos = int(pos.x + (SQUARE_SIZE >> 1)) / SQUARE_SIZE;
-	const int yardypos = int(pos.z + (SQUARE_SIZE >> 1)) / SQUARE_SIZE;
-
-	const float groundHeight = ground->GetHeightReal(pos.x, pos.z, synced);
+	// check maxHeightDif constraint (structures only)
+	//
+	// if we are capable of floating, only test local
+	// height difference IF terrain is above sea-level
+	if (unitDef->IsImmobileUnit()) {
+		if (!unitDef->floatOnWater || groundHeight > 0.0f) {
+			if (std::abs(pos.y - groundHeight) > unitDef->maxHeightDif) {
+				return BUILDSQUARE_BLOCKED;
+			}
+		}
+	}
 
 	BuildSquareStatus ret = BUILDSQUARE_OPEN;
+	const int yardxpos = unsigned(pos.x + (SQUARE_SIZE >> 1)) / SQUARE_SIZE;
+	const int yardypos = unsigned(pos.z + (SQUARE_SIZE >> 1)) / SQUARE_SIZE;
 	CSolidObject* so = groundBlockingObjectMap->GroundBlocked(yardxpos, yardypos);
-
 	if (so != NULL) {
 		CFeature* f = dynamic_cast<CFeature*>(so);
 		CUnit* u = dynamic_cast<CUnit*>(so);
@@ -1214,11 +1204,14 @@ CGameHelper::BuildSquareStatus CGameHelper::TestBuildSquare(
 					feature = f;
 				}
 			}
-		} else if (u == NULL || (allyteam < 0) || (u->losStatus[allyteam] & LOS_INLOS)) {
-			if (so->immobile) {
-				ret = BUILDSQUARE_BLOCKED;
-			} else {
-				ret = BUILDSQUARE_OCCUPIED;
+		} else {
+			assert(u);
+			if ((allyteam < 0) || (u->losStatus[allyteam] & LOS_INLOS)) {
+				if (so->immobile) {
+					ret = BUILDSQUARE_BLOCKED;
+				} else {
+					ret = BUILDSQUARE_OCCUPIED;
+				}
 			}
 		}
 
@@ -1254,33 +1247,6 @@ CGameHelper::BuildSquareStatus CGameHelper::TestBuildSquare(
 		}
 	}
 
-	// if we are capable of floating, only test local
-	// height difference IF terrain is above sea-level
-	if (!unitDef->floatOnWater || groundHeight > 0.0f) {
-		const float* orgHeightMap = readMap->GetOriginalHeightMapSynced();
-		const float* curHeightMap = readMap->GetCornerHeightMapSynced();
-
-		#ifdef USE_UNSYNCED_HEIGHTMAP
-		if (!synced) {
-			orgHeightMap = readMap->GetCornerHeightMapUnsynced();
-			curHeightMap = readMap->GetCornerHeightMapUnsynced();
-		}
-		#endif
-
-		const int sqx = pos.x / SQUARE_SIZE;
-		const int sqz = pos.z / SQUARE_SIZE;
-
-		const float orgHgt = orgHeightMap[sqz * gs->mapxp1 + sqx];
-		const float curHgt = curHeightMap[sqz * gs->mapxp1 + sqx];
-		const float difHgt = unitDef->maxHeightDif;
-
-		if (pos.y > std::max(orgHgt + difHgt, curHgt + difHgt)) { return BUILDSQUARE_BLOCKED; }
-		if (pos.y < std::min(orgHgt - difHgt, curHgt - difHgt)) { return BUILDSQUARE_BLOCKED; }
-	}
-
-	if (!unitDef->CheckTerrainConstraints(moveDef, groundHeight))
-		ret = BUILDSQUARE_BLOCKED;
-
 	return ret;
 }
 
@@ -1293,8 +1259,6 @@ CGameHelper::BuildSquareStatus CGameHelper::TestBuildSquare(
 Command CGameHelper::GetBuildCommand(const float3& pos, const float3& dir) {
 	float3 tempF1 = pos;
 
-	GML_STDMUTEX_LOCK(cai); // GetBuildCommand
-
 	CCommandQueue::iterator ci;
 
 	const std::list<CUnit*>& units = unitHandler->activeUnits;
