@@ -117,12 +117,6 @@ std::string GetAdditional()
 	return additional;
 }
 
-const std::string& GetBuildTime()
-{
-	static const std::string buildTime = __DATE__ " " __TIME__;
-	return buildTime;
-}
-
 #define QUOTEME_(x) #x
 #define QUOTEME(x) QUOTEME_(x)
 
@@ -178,6 +172,15 @@ bool IsHeadless()
 #endif
 }
 
+bool IsUnitsync()
+{
+#ifdef UNITSYNC
+	return true;
+#else
+	return false;
+#endif
+}
+
 const std::string& Get()
 {
 	static const std::string base = IsRelease()
