@@ -72,9 +72,6 @@ namespace SpringVersion
 	/// additional information (compiler flags, VCS revision etc.)
 	extern std::string GetAdditional();
 
-	/// time of build
-	extern const std::string& GetBuildTime();
-
 	/// build options
 	extern const std::string& GetBuildEnvironment();
 
@@ -86,6 +83,9 @@ namespace SpringVersion
 
 	/// Returns true if this build is a "HEADLESS" build
 	extern bool IsHeadless();
+	
+	/// Returns true if this build is a "UNITSYNC" build
+	extern bool IsUnitsync();
 
 	/**
 	 * The basic part of a spring version.
