@@ -11,13 +11,16 @@
 
 class ILoadSaveHandler;
 class GameData;
+class CGameSetup;
 class ClientSetup;
+
+
 namespace netcode {
 	class RawPacket;
 }
 
 /**
- * @brief This controlls the game start
+ * @brief This controls the game start
  *
  * Before a game starts, this class does everything that needs to be done before.
  * It basically goes like this:
@@ -26,7 +29,7 @@ namespace netcode {
  * 2. Start the server with this settings
  * 3. continue with "For clients"
  *
- * ForClients:
+ * For Clients:
  * 1. Connect to the server
  * 2. Receive GameData from server
  * 3. Start the CGame with the information provided by server
@@ -34,7 +37,7 @@ namespace netcode {
 class CPreGame : public CGameController
 {
 public:
-	CPreGame(boost::shared_ptr<const ClientSetup> setup);
+	CPreGame(boost::shared_ptr<ClientSetup> setup);
 	virtual ~CPreGame();
 
 	void LoadSetupscript(const std::string& script);
@@ -46,7 +49,9 @@ public:
 	bool Update();
 
 private:
+	void AddGameSetupArchivesToVFS(const CGameSetup* setup, bool mapOnly);
 	void StartServer(const std::string& setupscript);
+	void StartServerForDemo(const std::string& demoName);
 
 	/// reads out map, mod and script from demos (with or without a gameSetupScript)
 	void ReadDataFromDemo(const std::string& demoName);
@@ -59,10 +64,11 @@ private:
 	/**
 	@brief GameData we received from server
 
-	We won't start until we received this
+	We won't start until we received this (NULL until GameDataReceived)
 	*/
-	boost::shared_ptr<const GameData> gameData;
-	boost::shared_ptr<const ClientSetup> settings;
+	boost::shared_ptr<GameData> gameData;
+	boost::shared_ptr<ClientSetup> clientSetup;
+
 	std::string modArchive;
 	ILoadSaveHandler* savefile;
 
