@@ -69,9 +69,9 @@ public:
 			"Allows/Disallows spectators to draw on the map") {}
 
 	bool Execute(const SyncedAction& action) const {
-		bool disabled;
-		InverseOrSetBool(disabled, action.GetArgs());
-		inMapDrawer->SetSpecMapDrawingAllowed(!disabled);
+		bool allowSpecMapDrawing = inMapDrawer->GetSpecMapDrawingAllowed();
+		InverseOrSetBool(allowSpecMapDrawing, action.GetArgs(), true);
+		inMapDrawer->SetSpecMapDrawingAllowed(allowSpecMapDrawing);
 		return true;
 	}
 };
@@ -471,9 +471,11 @@ public:
 			int targetFrame;
 			buf >> targetFrame;
 			game->StartSkip(targetFrame);
+			LOG("Skipping to frame %i", targetFrame);
 		}
 		else if (action.GetArgs() == "end") {
 			game->EndSkip();
+			LOG("Skip finished");
 		} else {
 			LOG_L(L_WARNING, "/%s: wrong syntax", GetCommand().c_str());
 		}
@@ -515,7 +517,6 @@ void SyncedGameCommands::AddDefaultActionExecutors() {
 SyncedGameCommands* SyncedGameCommands::singleton = NULL;
 
 void SyncedGameCommands::CreateInstance() {
-
 	if (singleton == NULL) {
 		singleton = new SyncedGameCommands();
 	} else {
@@ -524,12 +525,8 @@ void SyncedGameCommands::CreateInstance() {
 }
 
 void SyncedGameCommands::DestroyInstance() {
-
 	if (singleton != NULL) {
-		// SafeDelete
-		SyncedGameCommands* tmp = singleton;
-		singleton = NULL;
-		delete tmp;
+		SafeDelete(singleton);
 	} else {
 		// this might happen during shutdown after an unclean init
 		LOG_L(L_WARNING, "SyncedGameCommands singleton was not initialized or is already destroyed");
