@@ -52,7 +52,7 @@ inline static bool TestConeHelper(
 		if (!ret) { ret = ((cv->GetPointSurfaceDistance(static_cast<const CFeature*>(obj), NULL, expPos3D) - coneSize) <= 0.0f); }
 	}
 
-	if (globalRendering->drawdebugtraceray && Threading::IsSimThread()) {
+	if (globalRendering->drawdebugtraceray) {
 		#define go geometricObjects
 
 		if (ret) {
@@ -90,7 +90,9 @@ inline static bool TestTrajectoryConeHelper(
 	// firing-cone is centered along dir2D with radius
 	// <x * spread + baseSize> (usually baseSize != 0
 	// so weapons with spread = 0 will test against a
-	// cylinder, not an infinitely thin line)
+	// cylinder, not an infinitely thin line as safety
+	// measure against friendly-fire damage in tightly
+	// packed unit groups)
 	//
 	// return true iff the world-space point <x, f(x)>
 	// lies on or inside the object's collision volume
@@ -127,10 +129,10 @@ inline static bool TestTrajectoryConeHelper(
 		if (!ret) { ret = ((cv->GetPointSurfaceDistance(static_cast<const CFeature*>(obj), NULL, expPos3D) - coneSize) <= 0.0f); }
 	}
 
-	if (globalRendering->drawdebugtraceray && Threading::IsSimThread()) {
+
+	if (globalRendering->drawdebugtraceray) {
 		// FIXME? seems to under-estimate gravity near edge of range
-		// (Cannon and MissileLauncher both subtract 30 elmos from it
-		// in HaveFreeLineOfFire???)
+		// (place objects along trajectory of a cannon to visualize)
 		#define go geometricObjects
 
 		if (ret) {
@@ -169,6 +171,7 @@ float TraceRay(
 	const bool ignoreFeatures = ((avoidFlags & Collision::NOFEATURES  ) != 0);
 	const bool ignoreNeutrals = ((avoidFlags & Collision::NONEUTRALS  ) != 0);
 	const bool ignoreGround   = ((avoidFlags & Collision::NOGROUND    ) != 0);
+	const bool ignoreCloaked  = ((avoidFlags & Collision::NOCLOAKED   ) != 0);
 
 	const bool ignoreUnits = ignoreEnemies && ignoreAllies && ignoreNeutrals;
 
@@ -179,14 +182,9 @@ float TraceRay(
 		return -1.0f;
 
 	if (!ignoreFeatures || !ignoreUnits) {
-		GML_RECMUTEX_LOCK(quad); // TraceRay
-
 		CollisionQuery cq;
 
-		int* begQuad = NULL;
-		int* endQuad = NULL;
-
-		quadField->GetQuadsOnRay(start, dir, length, begQuad, endQuad);
+		const auto& quads = quadField->GetQuadsOnRay(start, dir, length);
 
 		// locally point somewhere non-NULL; we cannot pass hitColQuery
 		// to DetectHit directly because each call resets it internally
@@ -195,12 +193,10 @@ float TraceRay(
 
 		// feature intersection
 		if (!ignoreFeatures) {
-			for (int* quadPtr = begQuad; quadPtr != endQuad; ++quadPtr) {
-				const CQuadField::Quad& quad = quadField->GetQuad(*quadPtr);
-
-				for (std::list<CFeature*>::const_iterator ui = quad.features.begin(); ui != quad.features.end(); ++ui) {
-					CFeature* f = *ui;
+			for (const int quadIdx: quads) {
+				const CQuadField::Quad& quad = quadField->GetQuad(quadIdx);
 
+				for (CFeature* f: quad.features) {
 					// NOTE:
 					//     if f is non-blocking, ProjectileHandler will not test
 					//     for collisions with projectiles so we can skip it here
@@ -223,21 +219,21 @@ float TraceRay(
 
 		// unit intersection
 		if (!ignoreUnits) {
-			for (int* quadPtr = begQuad; quadPtr != endQuad; ++quadPtr) {
-				const CQuadField::Quad& quad = quadField->GetQuad(*quadPtr);
-
-				for (std::list<CUnit*>::const_iterator ui = quad.units.begin(); ui != quad.units.end(); ++ui) {
-					CUnit* u = *ui;
+			for (const int quadIdx: quads) {
+				const CQuadField::Quad& quad = quadField->GetQuad(quadIdx);
 
+				for (CUnit* u: quad.units) {
 					if (u == owner)
 						continue;
 					if (!u->HasCollidableStateBit(CSolidObject::CSTATE_BIT_QUADMAPRAYS))
 						continue;
 					if (ignoreAllies && u->allyteam == owner->allyteam)
 						continue;
+					if (ignoreEnemies && u->allyteam != owner->allyteam)
+						continue;
 					if (ignoreNeutrals && u->IsNeutral())
 						continue;
-					if (ignoreEnemies && u->allyteam != owner->allyteam)
+					if (ignoreCloaked && u->IsCloaked())
 						continue;
 
 					if (CCollisionHandler::DetectHit(u, start, start + dir * length, &cq, true)) {
@@ -252,14 +248,16 @@ float TraceRay(
 					}
 				}
 			}
-			if (hitUnit)
+
+			if (hitUnit != NULL) {
 				hitFeature = NULL;
+			}
 		}
 	}
 
 	if (!ignoreGround) {
 		// ground intersection
-		const float groundLength = ground->LineGroundCol(start, start + dir * length);
+		const float groundLength = CGround::LineGroundCol(start, start + dir * length);
 
 		if (length > groundLength && groundLength > 0.0f) {
 			length = groundLength;
@@ -291,7 +289,7 @@ float GuiTraceRay(
 
 	// ground and water-plane intersection
 	const float guiRayLength = length;
-	const float groundRayLength = ground->LineGroundCol(start, start + dir * guiRayLength, false);
+	const float groundRayLength = CGround::LineGroundCol(start, start + dir * guiRayLength, false);
 	const float waterRayLength = math::floorf(math::fabs(start.y / std::min(dir.y, -0.00001f)));
 
 	float minRayLength = groundRayLength;
@@ -307,25 +305,14 @@ float GuiTraceRay(
 	if (groundOnly)
 		return minRayLength;
 
-	GML_RECMUTEX_LOCK(quad); // GuiTraceRay
-
-	int* begQuad = NULL;
-	int* endQuad = NULL;
-
-	quadField->GetQuadsOnRay(start, dir, length, begQuad, endQuad);
-
-	std::list<CUnit*>::const_iterator ui;
-	std::list<CFeature*>::const_iterator fi;
-
 	CollisionQuery cq;
 
-	for (int* quadPtr = begQuad; quadPtr != endQuad; ++quadPtr) {
-		const CQuadField::Quad& quad = quadField->GetQuad(*quadPtr);
+	const auto& quads = quadField->GetQuadsOnRay(start, dir, length);
+	for (const int quadIdx: quads) {
+		const CQuadField::Quad& quad = quadField->GetQuad(quadIdx);
 
 		// Unit Intersection
-		for (ui = quad.units.begin(); ui != quad.units.end(); ++ui) {
-			CUnit* unit = *ui;
-
+		for (CUnit* unit: quad.units) {
 			const bool unitIsEnemy = !teamHandler->Ally(unit->allyteam, gu->myAllyTeam);
 			const bool unitOnRadar = (useRadar && radarHandler->InRadar(unit, gu->myAllyTeam));
 			const bool unitInSight = (unit->losStatus[gu->myAllyTeam] & (LOS_INLOS | LOS_CONTRADAR));
@@ -373,9 +360,7 @@ float GuiTraceRay(
 		}
 
 		// Feature Intersection
-		for (fi = quad.features.begin(); fi != quad.features.end(); ++fi) {
-			CFeature* f = *fi;
-
+		for (CFeature* f: quad.features) {
 			if (!gu->spectatingFullView && !f->IsInLosForAllyTeam(gu->myAllyTeam))
 				continue;
 			// test this bit only in synced traces, rely on noSelect here
@@ -423,28 +408,19 @@ bool TestCone(
 	int avoidFlags,
 	CUnit* owner)
 {
-	GML_RECMUTEX_LOCK(quad); // TestCone
-
-	int* begQuad = NULL;
-	int* endQuad = NULL;
-
-	if (quadField->GetQuadsOnRay(from, dir, length, begQuad, endQuad) == 0)
+	const auto& quads = quadField->GetQuadsOnRay(from, dir, length);
+	if (quads.empty())
 		return true;
 
 	const bool ignoreAllies   = ((avoidFlags & Collision::NOFRIENDLIES) != 0);
 	const bool ignoreNeutrals = ((avoidFlags & Collision::NONEUTRALS  ) != 0);
 	const bool ignoreFeatures = ((avoidFlags & Collision::NOFEATURES  ) != 0);
 
-	for (int* quadPtr = begQuad; quadPtr != endQuad; ++quadPtr) {
-		const CQuadField::Quad& quad = quadField->GetQuad(*quadPtr);
+	for (const int quadIdx: quads) {
+		const CQuadField::Quad& quad = quadField->GetQuad(quadIdx);
 
 		if (!ignoreAllies) {
-			const std::list<CUnit*>& units = quad.teamUnits[allyteam];
-			      std::list<CUnit*>::const_iterator unitsIt;
-
-			for (unitsIt = units.begin(); unitsIt != units.end(); ++unitsIt) {
-				const CUnit* u = *unitsIt;
-
+			for (const CUnit* u: quad.teamUnits[allyteam]) {
 				if (u == owner)
 					continue;
 				if (!u->HasCollidableStateBit(CSolidObject::CSTATE_BIT_QUADMAPRAYS))
@@ -456,18 +432,13 @@ bool TestCone(
 		}
 
 		if (!ignoreNeutrals) {
-			const std::list<CUnit*>& units = quad.units;
-			      std::list<CUnit*>::const_iterator unitsIt;
-
-			for (unitsIt = units.begin(); unitsIt != units.end(); ++unitsIt) {
-				const CUnit* u = *unitsIt;
-
+			for (const CUnit* u: quad.units) {
+				if (!u->IsNeutral())
+					continue;
 				if (u == owner)
 					continue;
 				if (!u->HasCollidableStateBit(CSolidObject::CSTATE_BIT_QUADMAPRAYS))
 					continue;
-				if (!u->IsNeutral())
-					continue;
 
 				if (TestConeHelper(from, dir, length, spread, u))
 					return true;
@@ -475,12 +446,7 @@ bool TestCone(
 		}
 
 		if (!ignoreFeatures) {
-			const std::list<CFeature*>& features = quad.features;
-			      std::list<CFeature*>::const_iterator featuresIt;
-
-			for (featuresIt = features.begin(); featuresIt != features.end(); ++featuresIt) {
-				const CFeature* f = *featuresIt;
-
+			for (const CFeature* f: quad.features) {
 				if (!f->HasCollidableStateBit(CSolidObject::CSTATE_BIT_QUADMAPRAYS))
 					continue;
 
@@ -506,37 +472,26 @@ bool TestTrajectoryCone(
 	int avoidFlags,
 	CUnit* owner)
 {
-	GML_RECMUTEX_LOCK(quad); // TestTrajectoryCone
-
-	int* begQuad = NULL;
-	int* endQuad = NULL;
-
-	if (quadField->GetQuadsOnRay(from, dir, length, begQuad, endQuad) == 0)
+	const auto& quads = quadField->GetQuadsOnRay(from, dir, length);
+	if (quads.empty())
 		return true;
 
 	const bool ignoreAllies   = ((avoidFlags & Collision::NOFRIENDLIES) != 0);
 	const bool ignoreNeutrals = ((avoidFlags & Collision::NONEUTRALS  ) != 0);
 	const bool ignoreFeatures = ((avoidFlags & Collision::NOFEATURES  ) != 0);
 
-	const float safetyRadii[2] = {2.0f, 0.5f};
-
-	for (int* quadPtr = begQuad; quadPtr != endQuad; ++quadPtr) {
-		const CQuadField::Quad& quad = quadField->GetQuad(*quadPtr);
+	for (const int quadIdx: quads) {
+		const CQuadField::Quad& quad = quadField->GetQuad(quadIdx);
 
 		// friendly units in this quad
 		if (!ignoreAllies) {
-			const std::list<CUnit*>& units = quad.teamUnits[allyteam];
-			      std::list<CUnit*>::const_iterator unitsIt;
-
-			for (unitsIt = units.begin(); unitsIt != units.end(); ++unitsIt) {
-				const CUnit* u = *unitsIt;
-
+			for (const CUnit* u: quad.teamUnits[allyteam]) {
 				if (u == owner)
 					continue;
 				if (!u->HasCollidableStateBit(CSolidObject::CSTATE_BIT_QUADMAPRAYS))
 					continue;
 
-				if (TestTrajectoryConeHelper(from, dir, length, linear, quadratic, spread, safetyRadii[u->immobile], u)) {
+				if (TestTrajectoryConeHelper(from, dir, length, linear, quadratic, spread, 0.0f, u)) {
 					return true;
 				}
 			}
@@ -544,32 +499,22 @@ bool TestTrajectoryCone(
 
 		// neutral units in this quad
 		if (!ignoreNeutrals) {
-			const std::list<CUnit*>& units = quad.units;
-			      std::list<CUnit*>::const_iterator unitsIt;
-
-			for (unitsIt = units.begin(); unitsIt != units.end(); ++unitsIt) {
-				const CUnit* u = *unitsIt;
-
-				if (u == owner)
-					continue;
+			for (const CUnit* u: quad.units) {
 				if (!u->IsNeutral())
 					continue;
+				if (u == owner)
+					continue;
 				if (!u->HasCollidableStateBit(CSolidObject::CSTATE_BIT_QUADMAPRAYS))
 					continue;
 
-				if (TestTrajectoryConeHelper(from, dir, length, linear, quadratic, spread, safetyRadii[u->immobile], u))
+				if (TestTrajectoryConeHelper(from, dir, length, linear, quadratic, spread, 0.0f, u))
 					return true;
 			}
 		}
 
 		// features in this quad
 		if (!ignoreFeatures) {
-			const std::list<CFeature*>& features = quad.features;
-			      std::list<CFeature*>::const_iterator featuresIt;
-
-			for (featuresIt = features.begin(); featuresIt != features.end(); ++featuresIt) {
-				const CFeature* f = *featuresIt;
-
+			for (const CFeature* f: quad.features) {
 				if (!f->HasCollidableStateBit(CSolidObject::CSTATE_BIT_QUADMAPRAYS))
 					continue;
 
