@@ -300,6 +300,18 @@ void CHwWinCursor::buildIco(unsigned char* dst, ImageData &image)
 	fclose(pFile); */
 }
 
+
+static inline int GetBestCursorSize(const int minSize)
+{
+	auto stdSizes = {/*16,*/ 32, 48, 64, 96, 128};
+	for (const int s: stdSizes)
+		if (s >= minSize)
+			return s;
+
+	return next_power_of_2(minSize);
+}
+
+
 void CHwWinCursor::Finish()
 {
 	if (frames.empty())
@@ -308,8 +320,7 @@ void CHwWinCursor::Finish()
 	hotx = (hotSpot==CMouseCursor::TopLeft) ? 0 : (short)xmaxsize/2;
 	hoty = (hotSpot==CMouseCursor::TopLeft) ? 0 : (short)ymaxsize/2;
 
-	//note: windows only except 16x16,32x32,64x64,etc. (and some more not 2^n ones)
-	int squaresize =  next_power_of_2( std::max(xmaxsize,ymaxsize) );
+	int squaresize = GetBestCursorSize( std::max(xmaxsize,ymaxsize) );
 
 	//resize images
 	for (std::vector<ImageData>::iterator it=icons.begin(); it<icons.end(); ++it)
