@@ -170,7 +170,7 @@ bool CKeySet::Parse(const std::string& token, bool showerror)
 // CTimedKeyChain
 //
 
-void CTimedKeyChain::push_back(const int key, const spring_time t)
+void CTimedKeyChain::push_back(const int key, const spring_time t, const bool isRepeat)
 {
 	assert(keyBindings);
 
@@ -182,6 +182,10 @@ void CTimedKeyChain::push_back(const int key, const spring_time t)
 
 	CKeySet ks(key, false);
 
+	// append repeating keystrokes only wjen they differ from the last
+	if (isRepeat && (!empty() && ks == back()))
+		return;
+
 	// When you want to press c,alt+b you will press the c(down),c(up),alt(down),b(down)
 	// -> the chain will be: c,Alt+alt,Alt+b
 	// this should now match "c,alt+b", so we need to get rid of the redundant Alt+alt .
