@@ -49,65 +49,40 @@ static int CustomParamsTable(lua_State* L, const void* data);
 
 bool LuaFeatureDefs::PushEntries(lua_State* L)
 {
-	if (paramMap.empty()) {
-		InitParamMap();
-	}
+	InitParamMap();
 
-	const map<string, const FeatureDef*>& featureDefs = featureHandler->GetFeatureDefs();
-	map<string, const FeatureDef*>::const_iterator fdIt;
-	for (fdIt = featureDefs.begin(); fdIt != featureDefs.end(); ++fdIt) {
-		const FeatureDef* fd = fdIt->second;
-		if (fd == NULL) {
+	typedef int (*IndxFuncType)(lua_State*);
+	typedef int (*IterFuncType)(lua_State*);
+	typedef std::map<std::string, const FeatureDef*> ObjectDefMapType;
+
+	const ObjectDefMapType& defsMap = featureHandler->GetFeatureDefs();
+
+	const std::array<const LuaHashString, 3> indxOpers = {
+		LuaHashString("__index"),
+		LuaHashString("__newindex"),
+		LuaHashString("__metatable")
+	};
+	const std::array<const LuaHashString, 2> iterOpers = {
+		LuaHashString("pairs"),
+		LuaHashString("next")
+	};
+
+	const std::array<const IndxFuncType, 3> indxFuncs = {FeatureDefIndex, FeatureDefNewIndex, FeatureDefMetatable};
+	const std::array<const IterFuncType, 2> iterFuncs = {Pairs, Next};
+
+	for (auto it = defsMap.cbegin(); it != defsMap.cend(); ++it) {
+		const auto def = it->second; // ObjectDefMapType::mapped_type
+
+		if (def == NULL)
 			continue;
-		}
-		lua_pushnumber(L, fd->id);
-		lua_newtable(L); { // the proxy table
 
-			lua_newtable(L); { // the metatable
-
-				HSTR_PUSH(L, "__index");
-				lua_pushlightuserdata(L, (void*)fd);
-				lua_pushcclosure(L, FeatureDefIndex, 1);
-				lua_rawset(L, -3); // closure
-
-				HSTR_PUSH(L, "__newindex");
-				lua_pushlightuserdata(L, (void*)fd);
-				lua_pushcclosure(L, FeatureDefNewIndex, 1);
-				lua_rawset(L, -3);
-
-				HSTR_PUSH(L, "__metatable");
-				lua_pushlightuserdata(L, (void*)fd);
-				lua_pushcclosure(L, FeatureDefMetatable, 1);
-				lua_rawset(L, -3);
-			}
-
-			lua_setmetatable(L, -2);
-		}
-
-		HSTR_PUSH(L, "pairs");
-		lua_pushcfunction(L, Pairs);
-		lua_rawset(L, -3);
-
-		HSTR_PUSH(L, "next");
-		lua_pushcfunction(L, Next);
-		lua_rawset(L, -3);
-
-		lua_rawset(L, -3); // proxy table into FeatureDefs
+		PushObjectDefProxyTable(L, indxOpers, iterOpers, indxFuncs, iterFuncs, it->second);
 	}
 
 	return true;
 }
 
 
-bool LuaFeatureDefs::IsDefaultParam(const string& word)
-{
-	if (paramMap.empty()) {
-	  InitParamMap();
-	}
-	return (paramMap.find(word) != paramMap.end());
-}
-
-
 /******************************************************************************/
 
 /*static void PushFeatureDef(lua_State* L, const FeatureDef* fd, int index)
@@ -415,6 +390,9 @@ static int ColVolTable(lua_State* L, const void* data) {
 		case CollisionVolume::COLVOL_TYPE_BOX:
 			HSTR_PUSH_STRING(L, "type", "box");
 			break;
+		case CollisionVolume::COLVOL_TYPE_SPHERE:
+			HSTR_PUSH_STRING(L, "type", "sphere");
+			break;
 	}
 
 	LuaPushNamedNumber(L, "scaleX", cv->GetScales().x);
@@ -461,6 +439,8 @@ TYPE_MODEL_FUNC(Maxz, maxs.z)
 
 static bool InitParamMap()
 {
+	paramMap.clear();
+
 	paramMap["next"]  = DataElement(READONLY_TYPE);
 	paramMap["pairs"] = DataElement(READONLY_TYPE);
 
