@@ -1,6 +1,7 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include <map>
+#include <set>
 #include <boost/assign/list_of.hpp>
 
 #include "LuaOpenGLUtils.h"
@@ -18,6 +19,8 @@
 #include "Rendering/UnitDrawer.h"
 #include "Rendering/GL/GeometryBuffer.h"
 #include "Rendering/Env/CubeMapHandler.h"
+#include "Rendering/Map/InfoTexture/IInfoTextureHandler.h"
+#include "Rendering/Map/InfoTexture/InfoTexture.h"
 #include "Rendering/Models/3DModel.h"
 #include "Rendering/Textures/NamedTextures.h"
 #include "Rendering/Textures/3DOTextureHandler.h"
@@ -32,6 +35,12 @@
 #include "System/Log/ILog.h"
 
 
+
+/******************************************************************************/
+/******************************************************************************/
+
+static std::set<std::string> infoTexNames;
+
 /******************************************************************************/
 /******************************************************************************/
 
@@ -206,6 +215,8 @@ bool LuaOpenGLUtils::ParseTextureImage(lua_State* L, LuaMatTexture& texUnit, con
 	}
 
 	if (image[0] == LuaTextures::prefix) {
+		if (L == nullptr)
+			return false;
 		// dynamic texture
 		LuaTextures& textures = CLuaHandle::GetActiveTextures(L);
 		const LuaTextures::Texture* texInfo = textures.GetInfo(image);
@@ -284,6 +295,17 @@ bool LuaOpenGLUtils::ParseTextureImage(lua_State* L, LuaMatTexture& texUnit, con
 			texUnit.type = LuaMatTexture::LUATEX_MINIMAP;
 		}
 
+		else if (image.find("$info:") == 0) {
+			const std::string infoTexName = image.substr(6);
+			CInfoTexture* itex = infoTextureHandler->GetInfoTexture(infoTexName);
+			if (!itex)
+				return false;
+
+			infoTexNames.insert(infoTexName);
+			texUnit.type = LuaMatTexture::LUATEX_INFOTEX;
+			texUnit.data = &*infoTexNames.find(infoTexName);
+		}
+
 		else if (image == "$info"        || image == "$extra"       ) { texUnit.type = LuaMatTexture::LUATEX_INFOTEX_ACTIVE; }
 		else if (image == "$info_losmap" || image == "$extra_losmap") { texUnit.type = LuaMatTexture::LUATEX_INFOTEX_LOSMAP; }
 		else if (image == "$info_mtlmap" || image == "$extra_mtlmap") { texUnit.type = LuaMatTexture::LUATEX_INFOTEX_MTLMAP; }
@@ -354,14 +376,14 @@ GLuint LuaMatTexture::GetTextureID() const
 			auto stex = reinterpret_cast<const CS3OTextureHandler::S3oTex*>(data);
 			texID = stex->tex2;
 		} break;
-		case LUATEX_3DOTEXTURE: {
+		case LUATEX_3DOTEXTURE: if (texturehandler3DO != nullptr) {
 			if (*reinterpret_cast<const int*>(&data) == 1) {
 				texID = texturehandler3DO->GetAtlasTex1ID();
 			} else {
 				texID = texturehandler3DO->GetAtlasTex2ID();
 			}
 		} break;
-		case LUATEX_UNITBUILDPIC: {
+		case LUATEX_UNITBUILDPIC: if (unitDefHandler != nullptr) {
 			auto ud = reinterpret_cast<const UnitDef*>(data);
 			texID = unitDefHandler->GetUnitDefImage(ud);
 		} break;
@@ -369,18 +391,17 @@ GLuint LuaMatTexture::GetTextureID() const
 			auto ud = reinterpret_cast<const UnitDef*>(data);
 			texID = ud->iconType->GetTextureID();
 		} break;
-		case LUATEX_SHADOWMAP: {
+		case LUATEX_SHADOWMAP: if (shadowHandler != nullptr) {
 			texID = shadowHandler->shadowTexture;
 		} break;
-		case LUATEX_REFLECTION: {
+		case LUATEX_REFLECTION: if (cubeMapHandler != nullptr) {
 			texID = cubeMapHandler->GetEnvReflectionTextureID();
 		} break;
-		case LUATEX_SPECULAR: {
+		case LUATEX_SPECULAR: if (cubeMapHandler != nullptr) {
 			texID = cubeMapHandler->GetSpecularTextureID();
 		} break;
-		case LUATEX_HEIGHTMAP: {
-			if (heightMapTexture)
-				texID = heightMapTexture->GetTextureID();
+		case LUATEX_HEIGHTMAP: if (heightMapTexture) {
+			texID = heightMapTexture->GetTextureID();
 		} break;
 		case LUATEX_SHADING: {
 			texID = (readMap != NULL)? readMap->GetShadingTexture(): 0;
@@ -394,15 +415,34 @@ GLuint LuaMatTexture::GetTextureID() const
 		case LUATEX_FONTSMALL: {
 			texID = smallFont->GetTexture();
 		} break;
-		case LUATEX_MINIMAP:
+		case LUATEX_MINIMAP: {
 			texID = (readMap != NULL)? readMap->GetMiniMapTexture(): 0;
-		break;
+		} break;
 
-		case LUATEX_INFOTEX_ACTIVE: { texID = (readMap != NULL)? groundDrawer->GetActiveInfoTexture()                         : 0; } break;
-		case LUATEX_INFOTEX_LOSMAP: { texID = (readMap != NULL)? groundDrawer->GetInfoTexture(CBaseGroundDrawer::drawLos     ): 0; } break;
-		case LUATEX_INFOTEX_MTLMAP: { texID = (readMap != NULL)? groundDrawer->GetInfoTexture(CBaseGroundDrawer::drawMetal   ): 0; } break;
-		case LUATEX_INFOTEX_HGTMAP: { texID = (readMap != NULL)? groundDrawer->GetInfoTexture(CBaseGroundDrawer::drawHeight  ): 0; } break;
-		case LUATEX_INFOTEX_BLKMAP: { texID = (readMap != NULL)? groundDrawer->GetInfoTexture(CBaseGroundDrawer::drawPathTrav): 0; } break;
+		case LUATEX_INFOTEX: if (infoTextureHandler != nullptr) {
+			CInfoTexture* itex = infoTextureHandler->GetInfoTexture(*static_cast<const std::string*>(data));
+			texID = (itex) ? itex->GetTexture() : 0;
+		} break;
+
+		case LUATEX_INFOTEX_ACTIVE: if (infoTextureHandler != nullptr) {
+			texID = infoTextureHandler->GetCurrentInfoTexture();
+		} break;
+		case LUATEX_INFOTEX_LOSMAP: if (infoTextureHandler != nullptr) {
+			CInfoTexture* itex = infoTextureHandler->GetInfoTexture("los");
+			texID = (itex) ? itex->GetTexture() : 0;
+		} break;
+		case LUATEX_INFOTEX_MTLMAP: if (infoTextureHandler != nullptr) {
+			CInfoTexture* itex = infoTextureHandler->GetInfoTexture("metal");
+			texID = (itex) ? itex->GetTexture() : 0;
+		} break;
+		case LUATEX_INFOTEX_HGTMAP: if (infoTextureHandler != nullptr) {
+			CInfoTexture* itex = infoTextureHandler->GetInfoTexture("height");
+			texID = (itex) ? itex->GetTexture() : 0;
+		} break;
+		case LUATEX_INFOTEX_BLKMAP: if (infoTextureHandler != nullptr) {
+			CInfoTexture* itex = infoTextureHandler->GetInfoTexture("path");
+			texID = (itex) ? itex->GetTexture() : 0;
+		} break;
 
 		case LUATEX_MAP_GBUFFER_NORMTEX: { texID = gdGeomBuff->GetBufferTexture(GL::GeometryBuffer::ATTACHMENT_NORMTEX); } break;
 		case LUATEX_MAP_GBUFFER_DIFFTEX: { texID = gdGeomBuff->GetBufferTexture(GL::GeometryBuffer::ATTACHMENT_DIFFTEX); } break;
@@ -460,6 +500,7 @@ GLuint LuaMatTexture::GetTextureTarget() const
 		case LUATEX_FONTSMALL:
 		case LUATEX_MINIMAP:
 
+		case LUATEX_INFOTEX:
 		case LUATEX_INFOTEX_ACTIVE:
 		case LUATEX_INFOTEX_LOSMAP:
 		case LUATEX_INFOTEX_MTLMAP:
@@ -555,9 +596,10 @@ int2 LuaMatTexture::GetSize() const
 			const auto stex = reinterpret_cast<const CS3OTextureHandler::S3oTex*>(data);
 			return int2(stex->tex2SizeX, stex->tex2SizeY);
 		} break;
-		case LUATEX_3DOTEXTURE:
+		case LUATEX_3DOTEXTURE: if (texturehandler3DO != nullptr) {
 			return int2(texturehandler3DO->GetAtlasTexSizeX(), texturehandler3DO->GetAtlasTexSizeY());
-		case LUATEX_UNITBUILDPIC: {
+		} break;
+		case LUATEX_UNITBUILDPIC: if (unitDefHandler != nullptr) {
 			const auto ud = reinterpret_cast<const UnitDef*>(data);
 			unitDefHandler->GetUnitDefImage(ud); // forced existance
 			return int2(ud->buildPic->imageSizeX, ud->buildPic->imageSizeY);
@@ -566,35 +608,61 @@ int2 LuaMatTexture::GetSize() const
 			const auto ud = reinterpret_cast<const UnitDef*>(data);
 			return int2(ud->iconType->GetSizeX(), ud->iconType->GetSizeY());
 		} break;
-		case LUATEX_SHADOWMAP:
+		case LUATEX_SHADOWMAP: if (shadowHandler != nullptr) {
 			return sqint2(shadowHandler->shadowMapSize);
-		case LUATEX_REFLECTION:
+		} break;
+		case LUATEX_REFLECTION: if (cubeMapHandler != nullptr) {
 			return sqint2(cubeMapHandler->GetReflectionTextureSize());
-		case LUATEX_SPECULAR:
+		} break;
+		case LUATEX_SPECULAR: if (cubeMapHandler != nullptr) {
 			return sqint2(cubeMapHandler->GetSpecularTextureSize());
-		case LUATEX_SHADING:
-			return int2(gs->pwr2mapx, gs->pwr2mapy);
+		} break;
+		case LUATEX_SHADING: if (gs != nullptr) {
+			return int2(mapDims.pwr2mapx, mapDims.pwr2mapy);
+		} break;
 		case LUATEX_GRASS:
 			return int2(1024, 1024);
 		case LUATEX_FONT:
 			return int2(font->GetTextureWidth(), font->GetTextureHeight());
 		case LUATEX_FONTSMALL:
 			return int2(smallFont->GetTextureWidth(), smallFont->GetTextureHeight());
-		case LUATEX_MINIMAP:
-			if (readMap != NULL) {
-				return readMap->GetMiniMapTextureSize();
-			}
-		break;
+		case LUATEX_MINIMAP: if (readMap != NULL) {
+			return readMap->GetMiniMapTextureSize();
+		} break;
 
-		case LUATEX_INFOTEX_ACTIVE:
-		case LUATEX_INFOTEX_LOSMAP:
-		case LUATEX_INFOTEX_MTLMAP:
-		case LUATEX_INFOTEX_HGTMAP:
-		case LUATEX_INFOTEX_BLKMAP: {
-			if (readMap != NULL) {
-				return (readMap->GetGroundDrawer()->GetInfoTexSize());
+		case LUATEX_INFOTEX: if (infoTextureHandler != nullptr) {
+			CInfoTexture* itex = infoTextureHandler->GetInfoTexture(*static_cast<const std::string*>(data));
+			if (itex) {
+				return itex->GetTexSize();
 			}
-		}
+		} break;
+		case LUATEX_INFOTEX_ACTIVE: if (infoTextureHandler != nullptr) {
+			return infoTextureHandler->GetCurrentInfoTextureSize();
+		} break;
+		case LUATEX_INFOTEX_LOSMAP: if (infoTextureHandler != nullptr) {
+			CInfoTexture* itex = infoTextureHandler->GetInfoTexture("los");
+			if (itex) {
+				return itex->GetTexSize();
+			}
+		} break;
+		case LUATEX_INFOTEX_MTLMAP: if (infoTextureHandler != nullptr) {
+			CInfoTexture* itex = infoTextureHandler->GetInfoTexture("metal");
+			if (itex) {
+				return itex->GetTexSize();
+			}
+		} break;
+		case LUATEX_INFOTEX_HGTMAP: if (infoTextureHandler != nullptr) {
+			CInfoTexture* itex = infoTextureHandler->GetInfoTexture("height");
+			if (itex) {
+				return itex->GetTexSize();
+			}
+		} break;
+		case LUATEX_INFOTEX_BLKMAP: if (infoTextureHandler != nullptr) {
+			CInfoTexture* itex = infoTextureHandler->GetInfoTexture("path");
+			if (itex) {
+				return itex->GetTexSize();
+			}
+		} break;
 
 		case LUATEX_MAP_GBUFFER_NORMTEX:
 		case LUATEX_MAP_GBUFFER_DIFFTEX:
@@ -605,7 +673,7 @@ int2 LuaMatTexture::GetSize() const
 			if (readMap != NULL) {
 				return (readMap->GetGroundDrawer()->GetGeometryBuffer()->GetWantedSize(readMap->GetGroundDrawer()->DrawDeferred()));
 			}
-		}
+		} break;
 
 		case LUATEX_MODEL_GBUFFER_NORMTEX:
 		case LUATEX_MODEL_GBUFFER_DIFFTEX:
@@ -616,12 +684,13 @@ int2 LuaMatTexture::GetSize() const
 			if (unitDrawer != NULL) {
 				return (unitDrawer->GetGeometryBuffer()->GetWantedSize(unitDrawer->DrawDeferred()));
 			}
-		}
+		} break;
 
-		case LUATEX_HEIGHTMAP:
+		case LUATEX_HEIGHTMAP: {
 			if (heightMapTexture != NULL) {
 				return int2(heightMapTexture->GetSizeX(), heightMapTexture->GetSizeY());
 			}
+		} break;
 
 		case LUATEX_NONE:
 		default: break;
@@ -685,6 +754,7 @@ void LuaMatTexture::Print(const string& indent) const
 		STRING_CASE(typeName, LUATEX_FONTSMALL);
 		STRING_CASE(typeName, LUATEX_MINIMAP);
 
+		STRING_CASE(typeName, LUATEX_INFOTEX);
 		STRING_CASE(typeName, LUATEX_INFOTEX_ACTIVE);
 		STRING_CASE(typeName, LUATEX_INFOTEX_LOSMAP);
 		STRING_CASE(typeName, LUATEX_INFOTEX_MTLMAP);
