@@ -545,7 +545,7 @@ static int SetMoveTypeData(lua_State* L, AMoveType* moveType, const char* caller
 	int numAssignedValues = 0;
 
 	if (moveType == NULL) {
-		luaL_error(L, "[%s] unit %d has incompatible movetype for %s", __FUNCTION__, caller, lua_tonumber(L, 1));
+		luaL_error(L, "[%s] unit %d has incompatible movetype for %s", __FUNCTION__, lua_tonumber(L, 1), caller);
 		return numAssignedValues;
 	}
 
@@ -603,27 +603,28 @@ int LuaSyncedMoveCtrl::SetGroundMoveTypeData(lua_State* L)
 int LuaSyncedMoveCtrl::SetMoveDef(lua_State* L)
 {
 	CUnit* unit = ParseUnit(L, __FUNCTION__, 1);
-	MoveDef* moveDef = NULL;
+	MoveDef* moveDef = nullptr;
 
-	if (unit == NULL) {
+	if (unit == nullptr) {
 		lua_pushboolean(L, false);
 		return 1;
 	}
-	if (unit->moveDef == NULL) {
+	if (unit->moveDef == nullptr) {
 		// aircraft or structure, not supported
 		lua_pushboolean(L, false);
 		return 1;
 	}
 
 	// MoveType instance must already have been assigned
-	assert(unit->moveType != NULL);
+	assert(unit->moveType != nullptr);
 
+	// parse a MoveDef by number *or* by string (mutually exclusive)
 	if (lua_isnumber(L, 2))
 		moveDef = moveDefHandler->GetMoveDefByPathType(Clamp(luaL_checkint(L, 2), 0, int(moveDefHandler->GetNumMoveDefs()) - 1));
 	if (lua_isstring(L, 2))
 		moveDef = moveDefHandler->GetMoveDefByName(lua_tostring(L, 2));
 
-	if (moveDef == NULL) {
+	if (moveDef == nullptr) {
 		lua_pushboolean(L, false);
 		return 1;
 	}
@@ -631,6 +632,7 @@ int LuaSyncedMoveCtrl::SetMoveDef(lua_State* L)
 	if (moveDef->udRefCount == 0) {
 		// pathfinders contain optimizations that
 		// make unreferenced movedef's non-usable
+		LOG_L(L_ERROR, "SetMoveDef: Tried to use an unreferenced (:=disabled) MoveDef!");
 		lua_pushboolean(L, false);
 		return 1;
 	}
