@@ -81,64 +81,40 @@ static int CategorySetFromString(lua_State* L, const void* data);
 
 bool LuaUnitDefs::PushEntries(lua_State* L)
 {
-	if (paramMap.empty()) {
-	  InitParamMap();
-	}
+	InitParamMap();
 
-	const map<string, int>& udMap = unitDefHandler->unitDefIDsByName;
-	map<string, int>::const_iterator udIt;
-	for (udIt = udMap.begin(); udIt != udMap.end(); ++udIt) {
-	  const UnitDef* ud = unitDefHandler->GetUnitDefByID(udIt->second);
-		if (ud == NULL) {
-	  	continue;
-		}
-		lua_pushnumber(L, ud->id);
-		lua_newtable(L); { // the proxy table
+	typedef int (*IndxFuncType)(lua_State*);
+	typedef int (*IterFuncType)(lua_State*);
+	typedef std::map<std::string, int> ObjectDefMapType;
 
-			lua_newtable(L); { // the metatable
+	const ObjectDefMapType& defsMap = unitDefHandler->unitDefIDsByName;
 
-				HSTR_PUSH(L, "__index");
-				lua_pushlightuserdata(L, (void*)ud);
-				lua_pushcclosure(L, UnitDefIndex, 1);
-				lua_rawset(L, -3); // closure
+	const std::array<const LuaHashString, 3> indxOpers = {
+		LuaHashString("__index"),
+		LuaHashString("__newindex"),
+		LuaHashString("__metatable")
+	};
+	const std::array<const LuaHashString, 2> iterOpers = {
+		LuaHashString("pairs"),
+		LuaHashString("next")
+	};
 
-				HSTR_PUSH(L, "__newindex");
-				lua_pushlightuserdata(L, (void*)ud);
-				lua_pushcclosure(L, UnitDefNewIndex, 1);
-				lua_rawset(L, -3);
+	const std::array<const IndxFuncType, 3> indxFuncs = {UnitDefIndex, UnitDefNewIndex, UnitDefMetatable};
+	const std::array<const IterFuncType, 2> iterFuncs = {Pairs, Next};
 
-				HSTR_PUSH(L, "__metatable");
-				lua_pushlightuserdata(L, (void*)ud);
-				lua_pushcclosure(L, UnitDefMetatable, 1);
-				lua_rawset(L, -3);
-			}
+	for (auto it = defsMap.cbegin(); it != defsMap.cend(); ++it) {
+		const auto def = unitDefHandler->GetUnitDefByID(it->second);
 
-			lua_setmetatable(L, -2);
-		}
+		if (def == NULL)
+			continue;
 
-		HSTR_PUSH(L, "pairs");
-		lua_pushcfunction(L, Pairs);
-		lua_rawset(L, -3);
-
-		HSTR_PUSH(L, "next");
-		lua_pushcfunction(L, Next);
-		lua_rawset(L, -3);
-
-		lua_rawset(L, -3); // proxy table into UnitDefs
+		PushObjectDefProxyTable(L, indxOpers, iterOpers, indxFuncs, iterFuncs, def);
 	}
 
 	return true;
 }
 
 
-bool LuaUnitDefs::IsDefaultParam(const string& word)
-{
-	if (paramMap.empty()) {
-	  InitParamMap();
-	}
-	return (paramMap.find(word) != paramMap.end());
-}
-
 
 /******************************************************************************/
 
@@ -569,6 +545,9 @@ static int ColVolTable(lua_State* L, const void* data) {
 		case CollisionVolume::COLVOL_TYPE_BOX:
 			HSTR_PUSH_STRING(L, "type", "box");
 			break;
+		case CollisionVolume::COLVOL_TYPE_SPHERE:
+			HSTR_PUSH_STRING(L, "type", "sphere");
+			break;
 	}
 
 	LuaPushNamedNumber(L, "scaleX", cv->GetScales().x);
@@ -660,6 +639,8 @@ static int ReturnNil(lua_State* L, const void* data) {
 
 static bool InitParamMap()
 {
+	paramMap.clear();
+
 	paramMap["next"]  = DataElement(READONLY_TYPE);
 	paramMap["pairs"] = DataElement(READONLY_TYPE);
 
@@ -779,6 +760,7 @@ ADD_BOOL("canAttackWater",  canAttackWater); // CUSTOM
 	ADD_INT("selfDCountdown", ud.selfDCountdown);
 
 	ADD_FLOAT("speed",    ud.speed);
+	ADD_FLOAT("rSpeed",    ud.rSpeed);
 	ADD_FLOAT("turnRate", ud.turnRate);
 	ADD_BOOL("turnInPlace", ud.turnInPlace);
 	ADD_FLOAT("turnInPlaceSpeedLimit", ud.turnInPlaceSpeedLimit);
