@@ -16,12 +16,7 @@ class LuaUnsyncedCtrl {
 	public:
 		static bool PushEntries(lua_State* L);
 
-		static void DrawUnitCommandQueues();
-		static void ClearUnitCommandQueues();
-
-	private:
-
-	private:
+	public:
 		static int Echo(lua_State* L);
 		static int Log(lua_State* L);
 		static int SendMessage(lua_State* L);
@@ -95,7 +90,9 @@ class LuaUnsyncedCtrl {
 
 		static int CreateDir(lua_State* L);
 
+		static int Reload(lua_State* L);
 		static int Restart(lua_State* L);
+
 		static int SetWMIcon(lua_State* L);
 		static int SetWMCaption(lua_State* L);
 
@@ -108,7 +105,6 @@ class LuaUnsyncedCtrl {
 		static int LoadCtrlPanelConfig(lua_State* L);
 		static int ForceLayoutUpdate(lua_State* L);
 
-		static int UpdateInfoTexture(lua_State* L);
 		static int SetLosViewColors(lua_State* L);
 
 		static int WarpMouse(lua_State* L);
