@@ -188,38 +188,29 @@ bool SelectMenu::Draw()
 
 void SelectMenu::Single()
 {
-	static bool once = false;
-	if (selw->userMod == SelectionWidget::NoModSelect)
-	{
+	if (selw->userMod == SelectionWidget::NoModSelect) {
 		selw->ShowModList();
-	}
-	else if (selw->userMap == SelectionWidget::NoMapSelect)
-	{
+	} else if (selw->userMap == SelectionWidget::NoMapSelect) {
 		selw->ShowMapList();
-	}
-	else if (selw->userScript == SelectionWidget::NoScriptSelect)
-	{
+	} else if (selw->userScript == SelectionWidget::NoScriptSelect) {
 		selw->ShowScriptList();
 	}
-	else if (!once) // in case of double-click
-	{
+	else if (pregame == NULL) {
+		// in case of double-click
 		if (selw->userScript == SelectionWidget::SandboxAI) {
 			selw->userScript.clear();
 		}
-		once = true;
 
 		pregame = new CPreGame(clientSetup);
 		pregame->LoadSetupscript(StartScriptGen::CreateDefaultSetup(selw->userMap, selw->userMod, selw->userScript, clientSetup->myPlayerName));
-		return agui::gui->RmElement(this);
-		//delete this;
+		return (agui::gui->RmElement(this));
 	}
 }
 
 void SelectMenu::Quit()
 {
 	gu->globalQuit = true;
-	return agui::gui->RmElement(this);
-	//delete this;
+	return (agui::gui->RmElement(this));
 }
 
 void SelectMenu::ShowConnectWindow(bool show)
@@ -301,11 +292,12 @@ void SelectMenu::CleanWindow() {
 void SelectMenu::DirectConnect(const std::string& addr)
 {
 	configHandler->SetString("address", addr);
+
 	clientSetup->hostIP = addr;
 	clientSetup->isHost = false;
+
 	pregame = new CPreGame(clientSetup);
-	return agui::gui->RmElement(this);
-	//delete this;
+	return (agui::gui->RmElement(this));
 }
 
 bool SelectMenu::HandleEventSelf(const SDL_Event& ev)
@@ -313,7 +305,7 @@ bool SelectMenu::HandleEventSelf(const SDL_Event& ev)
 	switch (ev.type) {
 		case SDL_KEYDOWN: {
 			if (ev.key.keysym.sym == SDLK_ESCAPE) {
-				LOG("User exited");
+				LOG("[SelectMenu] user exited");
 				Quit();
 			} else if (ev.key.keysym.sym == SDLK_RETURN) {
 				Single();
