@@ -163,7 +163,7 @@ void CAdvWater::Draw(bool useBlending)
 	glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, waterFP);
 	glEnable(GL_FRAGMENT_PROGRAM_ARB);
 
-	float3 forward = camera->forward;
+	float3 forward = camera->GetDir();
 	forward.y = 0;
 	forward.ANormalize();
 
@@ -308,8 +308,9 @@ void CAdvWater::UpdateWater(CGame* game)
 	char realCam[sizeof(CCamera)];
 	new (realCam) CCamera(*camera); // anti-crash workaround for multithreading
 
-	camera->forward.y *= -1.0f;
+	camera->SetDir(camera->GetDir() * float3(1.0f, -1.0f, 1.0f));
 	camera->SetPos(camera->GetPos() * float3(1.0f, -1.0f, 1.0f));
+	camera->SetRotZ(-camera->GetRot().z);
 	camera->Update();
 
 	reflectFBO.Bind();
@@ -352,4 +353,5 @@ void CAdvWater::UpdateWater(CGame* game)
 	camera->Update();
 	glPopAttrib();
 	glPopAttrib();
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 }
