@@ -7,7 +7,7 @@
 #include "System/Log/ILog.h"
 
 
-CONFIG(int, GroundDecals).defaultValue(1).minimumValue(0).description("Controls whether ground decals underneath buildings and ground scars from explosions will be rendered. Values >1 define how long such decals will stay.");
+CONFIG(int, GroundDecals).defaultValue(1).headlessValue(0).minimumValue(0).description("Controls whether ground decals underneath buildings and ground scars from explosions will be rendered. Values >1 define how long such decals will stay.");
 
 bool IGroundDecalDrawer::drawDecals = 0;
 unsigned int IGroundDecalDrawer::decalLevel = 0;
@@ -32,7 +32,7 @@ IGroundDecalDrawer* IGroundDecalDrawer::GetInstance()
 		} catch(const opengl_error& ex) {
 			LOG_L(L_ERROR, "IGroundDecalDrawer loading failed: %s", ex.what());
 #endif
-			delete singleton;
+			SafeDelete(singleton);
 			singleton = new CGroundDecalHandler();
 			LOG_L(L_INFO, "Loaded DecalsDrawer: %s", "Legacy");
 
@@ -47,6 +47,5 @@ IGroundDecalDrawer* IGroundDecalDrawer::GetInstance()
 
 void IGroundDecalDrawer::FreeInstance()
 {
-	delete singleton;
-	singleton = NULL;
+	SafeDelete(singleton);
 }
