@@ -202,19 +202,23 @@ static void CopyShaderState_Uniforms(GLuint newProgID, GLuint oldProgID, std::un
 			oldUniformState = &it->second;
 		} else {
 			// Uniform not found in state tracker, try to read it from old shader object
-			//oldUniformState = &(uniformStates->emplace(hash, name).first->second);
-			oldUniformState = &(uniformStates->insert(std::pair<size_t, Shader::UniformState>(hash, Shader::UniformState(name))).first->second);
+			oldUniformState = &(uniformStates->emplace(hash, name).first->second);
 		}
 		oldUniformState->SetLocation(newLoc);
 
 		// Check if we got data we can use to initialize the uniform
 		if (oldUniformState->IsUninit()) {
+			if (oldProgID == 0) {
+				oldLoc = -1;
+				continue;
+			}
 			oldLoc = glGetUniformLocation(oldProgID, &name[0]);
 
 			// No old data found, so we cannot initialize the uniform
 			if (oldLoc < 0)
 				continue;
-			//FIXME read data from old shader save data _in new uniformState_?
+
+			//FIXME read data from old shader & save data in _new_ uniformState?
 		}
 
 		// Initialize the uniform with previous data
@@ -257,11 +261,12 @@ static void CopyShaderState_Uniforms(GLuint newProgID, GLuint oldProgID, std::un
 			HANDLE_MATTYPE(FLOAT_MAT, 3, fv, GLfloat)
 			HANDLE_MATTYPE(FLOAT_MAT, 4, fv, GLfloat)
 
-			/*case ATOMIC: {
-				GLint binding;
+			case ATOMIC: {
+				assert(false);
+				/*GLint binding;
 				glGetActiveAtomicCounterBufferiv(oldProgID, i, GL_ATOMIC_COUNTER_BUFFER_BINDING, &binding);
-				glUniform1f(newLoc, 1, binding);
-			} break;*/
+				glUniform1f(newLoc, 1, binding);*/
+			} break;
 
 			default:
 				LOG_L(L_WARNING, "Unknown GLSL uniform \"%s\" has unknown vartype \"%X\"", name.c_str(), type);
@@ -425,10 +430,13 @@ namespace Shader {
 	{
 	#if !defined(HEADLESS)
 		CopyShaderState_Uniforms(newProgID, oldProgID, uniformStates);
-		CopyShaderState_UniformBlocks(newProgID, oldProgID);
-		CopyShaderState_Attributes(newProgID, oldProgID);
-		CopyShaderState_TransformFeedback(newProgID, oldProgID);
-		CopyShaderState_Geometry(newProgID, oldProgID);
+
+		if (oldProgID != 0) {
+			CopyShaderState_UniformBlocks(newProgID, oldProgID);
+			CopyShaderState_Attributes(newProgID, oldProgID);
+			CopyShaderState_TransformFeedback(newProgID, oldProgID);
+			CopyShaderState_Geometry(newProgID, oldProgID);
+		}
 	#endif
 	}
 }
