@@ -1,21 +1,31 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include "Rendering/Shaders/Shader.h"
+#include "Rendering/Shaders/ShaderHandler.h"
+#include "Rendering/Shaders/LuaShaderContainer.h"
 #include "Rendering/Shaders/GLSLCopyState.h"
 #include "Rendering/GL/myGL.h"
 #include "Rendering/GlobalRendering.h"
+#include "Lua/LuaMaterial.h"
+
 #include "System/Util.h"
 #include "System/FileSystem/FileHandler.h"
+#include "System/Sync/HsiehHash.h"
 #include "System/Log/ILog.h"
+
+#include "System/Config/ConfigHandler.h"
+
 #include <algorithm>
 #ifdef DEBUG
 	#include <string.h> // strncmp
 #endif
-#ifdef HEADLESS
+#ifndef GL_INVALID_INDEX
 	#define GL_INVALID_INDEX -1
 #endif
 
 
+/*****************************************************************/
+
 #define LOG_SECTION_SHADER "Shader"
 LOG_REGISTER_SECTION_GLOBAL(LOG_SECTION_SHADER)
 
@@ -25,10 +35,12 @@ LOG_REGISTER_SECTION_GLOBAL(LOG_SECTION_SHADER)
 #endif
 #define LOG_SECTION_CURRENT LOG_SECTION_SHADER
 
+/*****************************************************************/
 
-static std::unordered_map<size_t, GLuint> glslShadersCache;
+CONFIG(bool, UseShaderCache).defaultValue(true).description("If already compiled shaders should be shared via a cache, reducing compiles of already compiled shaders.");
 
 
+/*****************************************************************/
 
 static bool glslIsValid(GLuint obj)
 {
@@ -73,6 +85,9 @@ static std::string glslGetLog(GLuint obj)
 
 static std::string GetShaderSource(const std::string& fileName)
 {
+	if (fileName.find("void main()") != std::string::npos)
+		return fileName;
+
 	std::string soPath = "shaders/" + fileName;
 	std::string soSource = "";
 
@@ -101,7 +116,7 @@ static bool ExtractGlslVersion(std::string* src, std::string* version)
 	return false;
 }
 
-
+/*****************************************************************/
 
 
 namespace Shader {
@@ -112,6 +127,30 @@ namespace Shader {
 	NullProgramObject* nullProgramObject = &nullProgramObject_;
 
 
+	/*****************************************************************/
+
+	unsigned int IShaderObject::GetHash() const {
+		unsigned int hash = 127;
+		hash = HsiehHash((const void*)curShaderSrc.data(), curShaderSrc.size(), hash);
+		hash = HsiehHash((const void*)modDefStrs.data(), modDefStrs.size(), hash);
+		hash = HsiehHash((const void*)rawDefStrs.data(), rawDefStrs.size(), hash);
+		return hash;
+	}
+
+
+	bool IShaderObject::ReloadFromDisk()
+	{
+		std::string newsrc = GetShaderSource(srcFile);
+		if (newsrc != curShaderSrc) {
+			curShaderSrc = std::move(newsrc);
+			return true;
+		}
+		return false;
+	}
+
+
+	/*****************************************************************/
+
 	ARBShaderObject::ARBShaderObject(
 		unsigned int shType,
 		const std::string& shSrc,
@@ -122,18 +161,14 @@ namespace Shader {
 		glGenProgramsARB(1, &objID);
 	}
 
-	void ARBShaderObject::Compile(bool reloadFromDisk) {
+	void ARBShaderObject::Compile() {
 		glEnable(type);
 
-		if (reloadFromDisk)
-			curShaderSrc = GetShaderSource(srcFile);
-
 		glBindProgramARB(type, objID);
 		glProgramStringARB(type, GL_PROGRAM_FORMAT_ASCII_ARB, curShaderSrc.size(), curShaderSrc.c_str());
 
 		int errorPos = -1;
 		int isNative =  0;
-
 		glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB, &errorPos);
 		glGetProgramivARB(type, GL_PROGRAM_UNDER_NATIVE_LIMITS_ARB, &isNative);
 
@@ -151,6 +186,7 @@ namespace Shader {
 	}
 
 
+	/*****************************************************************/
 
 	GLSLShaderObject::GLSLShaderObject(
 		unsigned int shType,
@@ -161,10 +197,14 @@ namespace Shader {
 		assert(globalRendering->haveGLSL); // non-debug check is done in ShaderHandler
 	}
 
-	void GLSLShaderObject::Compile(bool reloadFromDisk) {
-		if (reloadFromDisk)
-			curShaderSrc = GetShaderSource(srcFile);
+	GLSLShaderObject::CompiledShaderObjectUniquePtr GLSLShaderObject::CompileShaderObject()
+	{
+		CompiledShaderObjectUniquePtr res(new CompiledShaderObject(), [](CompiledShaderObject* so) {
+			glDeleteShader(so->id);
+			so->id = 0;
+		});
 
+		assert(!curShaderSrc.empty());
 		std::string sourceStr = curShaderSrc;
 		std::string defFlags  = rawDefStrs + "\n" + modDefStrs;
 		std::string versionStr;
@@ -176,7 +216,6 @@ namespace Shader {
 		if (!versionStr.empty()) EnsureEndsWith(&versionStr, "\n");
 		if (!defFlags.empty())   EnsureEndsWith(&defFlags,   "\n");
 
-		// NOTE: many definition flags are not set until after shader is compiled
 		std::vector<const GLchar*> sources = {
 			"// SHADER VERSION\n",
 			versionStr.c_str(),
@@ -187,29 +226,27 @@ namespace Shader {
 			sourceStr.c_str()
 		};
 
-		if (objID == 0)
-			objID = glCreateShader(type);
+		res->id = glCreateShader(type);
 
-		glShaderSource(objID, sources.size(), &sources[0], NULL);
-		glCompileShader(objID);
+		glShaderSource(res->id, sources.size(), &sources[0], NULL);
+		glCompileShader(res->id);
 
-		valid = glslIsValid(objID);
-		log   = glslGetLog(objID);
+		res->valid = glslIsValid(res->id);
+		res->log   = glslGetLog(res->id);
 
-		if (!IsValid()) {
-			LOG_L(L_WARNING, "[GLSL-SO::%s] shader-object name: %s, compile-log:\n%s\n", __FUNCTION__, srcFile.c_str(), log.c_str());
+		if (!res->valid) {
+			const std::string& name = srcFile.find("void main()") ? "unknown" : srcFile;
+			LOG_L(L_WARNING, "[GLSL-SO::%s] shader-object name: %s, compile-log:\n%s\n", __FUNCTION__, name.c_str(), res->log.c_str());
 			LOG_L(L_WARNING, "\n%s%s%s%s%s%s%s", sources[0], sources[1], sources[2], sources[3], sources[4], sources[5], sources[6]);
 		}
-	}
 
-	void GLSLShaderObject::Release() {
-		glDeleteShader(objID);
-		objID = 0;
+		return res;
 	}
 
 
+	/*****************************************************************/
 
-	IProgramObject::IProgramObject(const std::string& poName): name(poName), objID(0), curHash(0), valid(false), bound(false) {
+	IProgramObject::IProgramObject(const std::string& poName): name(poName), objID(0), curFlagsHash(0), valid(false), bound(false) {
 	}
 
 	void IProgramObject::Enable() {
@@ -230,33 +267,39 @@ namespace Shader {
 			delete so;
 		}
 
+		uniformStates.clear();
 		shaderObjs.clear();
+		textures.clear();
+		valid = false;
+		log = "";
 	}
 
 	bool IProgramObject::IsShaderAttached(const IShaderObject* so) const {
 		return (std::find(shaderObjs.begin(), shaderObjs.end(), so) != shaderObjs.end());
 	}
 
-	void IProgramObject::RecompileIfNeeded()
-	{
-		const unsigned int hash = GetHash();
+	bool IProgramObject::LoadFromLua(const std::string& filename) {
+		return Shader::LoadFromLua(this, filename);
+	}
 
-		if (hash == curHash)
+	void IProgramObject::RecompileIfNeeded(bool validate)
+	{
+		if (curFlagsHash != 0 && GetHash() == curFlagsHash)
 			return;
 
 		// NOTE: this does not preserve the #version pragma
 		const std::string definitionFlags = GetString();
-
 		for (IShaderObject*& so: shaderObjs) {
 			so->SetDefinitions(definitionFlags);
 		}
 
-		Reload(false);
-		PrintInfo();
+		Reload(curFlagsHash == 0, validate);
+		PrintDebugInfo();
 	}
 
-	void IProgramObject::PrintInfo()
+	void IProgramObject::PrintDebugInfo()
 	{
+	#if DEBUG
 		LOG_L(L_DEBUG, "Uniform States for program-object \"%s\":", name.c_str());
 		LOG_L(L_DEBUG, "Defs:\n %s", GetString().c_str());
 		LOG_L(L_DEBUG, "Uniforms:");
@@ -268,19 +311,48 @@ namespace Shader {
 				LOG_L(L_DEBUG, "\t%s: x=float:%f;int:%i y=%f z=%f used=%i", (p.second.GetName()).c_str(), p.second.GetFltValues()[0], p.second.GetIntValues()[0], p.second.GetFltValues()[1], p.second.GetFltValues()[2], int(curUsed));
 			}
 		}
+	#endif
 	}
 
 	UniformState* IProgramObject::GetNewUniformState(const std::string name)
 	{
-		UniformState* us = &uniformStates.emplace(hashString(name.c_str()), name).first->second;
+		const size_t hash = hashString(name.c_str());
+		const auto it = uniformStates.emplace(hash, name);
+
+		UniformState* us = &(it.first->second);
 		us->SetLocation(GetUniformLoc(name));
+
 	#if DEBUG
 		if (us->IsLocationValid())
 			us->SetType(GetUniformType(us->GetLocation()));
+
+		// make sure hash is unique
+		for (const auto us2: uniformStates)
+			assert(us2.first != hash || us2.second.GetName() == name);
 	#endif
+
 		return us;
 	}
 
+	void IProgramObject::AddTextureBinding(const int index, const std::string& luaTexName)
+	{
+		textures[index] = luaTexName;
+	}
+
+	void IProgramObject::BindTextures() const
+	{
+		LuaMatTexture luaTex;
+		for (auto& p: textures) {
+			if (LuaOpenGLUtils::ParseTextureImage(nullptr, luaTex, p.second)) {
+				glActiveTexture(GL_TEXTURE0 + p.first);
+				luaTex.Bind();
+			}
+		}
+		glActiveTexture(GL_TEXTURE0);
+	}
+
+
+	/*****************************************************************/
 
 	ARBProgramObject::ARBProgramObject(const std::string& poName): IProgramObject(poName) {
 		objID = -1; // not used for ARBProgramObject instances
@@ -295,7 +367,7 @@ namespace Shader {
 	}
 
 	void ARBProgramObject::Enable() {
-		RecompileIfNeeded();
+		RecompileIfNeeded(true);
 		for (const IShaderObject* so: shaderObjs) {
 			glEnable(so->GetType());
 			glBindProgramARB(so->GetType(), so->GetObjID());
@@ -311,6 +383,7 @@ namespace Shader {
 	}
 
 	void ARBProgramObject::Link() {
+		RecompileIfNeeded(false);
 		bool shaderObjectsValid = true;
 
 		for (const IShaderObject* so: shaderObjs) {
@@ -322,52 +395,60 @@ namespace Shader {
 	void ARBProgramObject::Release() {
 		IProgramObject::Release();
 	}
-	void ARBProgramObject::Reload(bool reloadFromDisk) {
-
+	void ARBProgramObject::Reload(bool reloadFromDisk, bool validate) {
+		for (IShaderObject* so: GetAttachedShaderObjs()) {
+			if (reloadFromDisk) so->ReloadFromDisk();
+			so->Compile();
+		}
+		if (validate) Validate();
 	}
 
 	int ARBProgramObject::GetUniformLoc(const std::string& name) {
 		return -1; //FIXME
 	}
 
-	#define glPEP4f  glProgramEnvParameter4fARB
-	#define glPEP4fv glProgramEnvParameter4fvARB
-	void ARBProgramObject::SetUniform1i(int idx, int   v0                              ) { glPEP4f(uniformTarget, idx, float(v0), float( 0), float( 0), float( 0)); }
-	void ARBProgramObject::SetUniform2i(int idx, int   v0, int   v1                    ) { glPEP4f(uniformTarget, idx, float(v0), float(v1), float( 0), float( 0)); }
-	void ARBProgramObject::SetUniform3i(int idx, int   v0, int   v1, int   v2          ) { glPEP4f(uniformTarget, idx, float(v0), float(v1), float(v2), float( 0)); }
-	void ARBProgramObject::SetUniform4i(int idx, int   v0, int   v1, int   v2, int   v3) { glPEP4f(uniformTarget, idx, float(v0), float(v1), float(v2), float(v3)); }
-	void ARBProgramObject::SetUniform1f(int idx, float v0                              ) { glPEP4f(uniformTarget, idx, v0, 0.0f, 0.0f, 0.0f); }
-	void ARBProgramObject::SetUniform2f(int idx, float v0, float v1                    ) { glPEP4f(uniformTarget, idx, v0,   v1, 0.0f, 0.0f); }
-	void ARBProgramObject::SetUniform3f(int idx, float v0, float v1, float v2          ) { glPEP4f(uniformTarget, idx, v0,   v1,   v2, 0.0f); }
-	void ARBProgramObject::SetUniform4f(int idx, float v0, float v1, float v2, float v3) { glPEP4f(uniformTarget, idx, v0,   v1,   v2,   v3); }
+	void ARBProgramObject::SetUniform1i(int idx, int   v0                              ) { glProgramEnvParameter4fARB(uniformTarget, idx, float(v0), float( 0), float( 0), float( 0)); }
+	void ARBProgramObject::SetUniform2i(int idx, int   v0, int   v1                    ) { glProgramEnvParameter4fARB(uniformTarget, idx, float(v0), float(v1), float( 0), float( 0)); }
+	void ARBProgramObject::SetUniform3i(int idx, int   v0, int   v1, int   v2          ) { glProgramEnvParameter4fARB(uniformTarget, idx, float(v0), float(v1), float(v2), float( 0)); }
+	void ARBProgramObject::SetUniform4i(int idx, int   v0, int   v1, int   v2, int   v3) { glProgramEnvParameter4fARB(uniformTarget, idx, float(v0), float(v1), float(v2), float(v3)); }
+	void ARBProgramObject::SetUniform1f(int idx, float v0                              ) { glProgramEnvParameter4fARB(uniformTarget, idx, v0, 0.0f, 0.0f, 0.0f); }
+	void ARBProgramObject::SetUniform2f(int idx, float v0, float v1                    ) { glProgramEnvParameter4fARB(uniformTarget, idx, v0,   v1, 0.0f, 0.0f); }
+	void ARBProgramObject::SetUniform3f(int idx, float v0, float v1, float v2          ) { glProgramEnvParameter4fARB(uniformTarget, idx, v0,   v1,   v2, 0.0f); }
+	void ARBProgramObject::SetUniform4f(int idx, float v0, float v1, float v2, float v3) { glProgramEnvParameter4fARB(uniformTarget, idx, v0,   v1,   v2,   v3); }
 
-	void ARBProgramObject::SetUniform2iv(int idx, const int*   v) { int   vv[4]; vv[0] = v[0]; vv[1] = v[1]; vv[2] =    0; vv[3] =    0; glPEP4fv(uniformTarget, idx, (float*) vv); }
-	void ARBProgramObject::SetUniform3iv(int idx, const int*   v) { int   vv[4]; vv[0] = v[0]; vv[1] = v[1]; vv[2] = v[2]; vv[3] =    0; glPEP4fv(uniformTarget, idx, (float*) vv); }
-	void ARBProgramObject::SetUniform4iv(int idx, const int*   v) { int   vv[4]; vv[0] = v[0]; vv[1] = v[1]; vv[2] = v[2]; vv[3] = v[3]; glPEP4fv(uniformTarget, idx, (float*) vv); }
-	void ARBProgramObject::SetUniform2fv(int idx, const float* v) { float vv[4]; vv[0] = v[0]; vv[1] = v[1]; vv[2] = 0.0f; vv[3] = 0.0f; glPEP4fv(uniformTarget, idx,          vv); }
-	void ARBProgramObject::SetUniform3fv(int idx, const float* v) { float vv[4]; vv[0] = v[0]; vv[1] = v[1]; vv[2] = v[2]; vv[3] = 0.0f; glPEP4fv(uniformTarget, idx,          vv); }
-	void ARBProgramObject::SetUniform4fv(int idx, const float* v) { float vv[4]; vv[0] = v[0]; vv[1] = v[1]; vv[2] = v[2]; vv[3] = v[3]; glPEP4fv(uniformTarget, idx,          vv); }
-	#undef glPEP4f
-	#undef glPEP4fv
-
-	void ARBProgramObject::AttachShaderObject(IShaderObject* so) {
-		if (so != NULL) {
-			IProgramObject::AttachShaderObject(so);
-		}
-	}
+	void ARBProgramObject::SetUniform2iv(int idx, const int*   v) { int   vv[4]; vv[0] = v[0]; vv[1] = v[1]; vv[2] =    0; vv[3] =    0; glProgramEnvParameter4fvARB(uniformTarget, idx, (float*) vv); }
+	void ARBProgramObject::SetUniform3iv(int idx, const int*   v) { int   vv[4]; vv[0] = v[0]; vv[1] = v[1]; vv[2] = v[2]; vv[3] =    0; glProgramEnvParameter4fvARB(uniformTarget, idx, (float*) vv); }
+	void ARBProgramObject::SetUniform4iv(int idx, const int*   v) { int   vv[4]; vv[0] = v[0]; vv[1] = v[1]; vv[2] = v[2]; vv[3] = v[3]; glProgramEnvParameter4fvARB(uniformTarget, idx, (float*) vv); }
+	void ARBProgramObject::SetUniform2fv(int idx, const float* v) { float vv[4]; vv[0] = v[0]; vv[1] = v[1]; vv[2] = 0.0f; vv[3] = 0.0f; glProgramEnvParameter4fvARB(uniformTarget, idx,          vv); }
+	void ARBProgramObject::SetUniform3fv(int idx, const float* v) { float vv[4]; vv[0] = v[0]; vv[1] = v[1]; vv[2] = v[2]; vv[3] = 0.0f; glProgramEnvParameter4fvARB(uniformTarget, idx,          vv); }
+	void ARBProgramObject::SetUniform4fv(int idx, const float* v) { float vv[4]; vv[0] = v[0]; vv[1] = v[1]; vv[2] = v[2]; vv[3] = v[3]; glProgramEnvParameter4fvARB(uniformTarget, idx,          vv); }
 
 
+	/*****************************************************************/
 
-	GLSLProgramObject::GLSLProgramObject(const std::string& poName): IProgramObject(poName) {
-		objID = 0;
+	GLSLProgramObject::GLSLProgramObject(const std::string& poName): IProgramObject(poName), curSrcHash(0) {
 		objID = glCreateProgram();
 	}
 
-	void GLSLProgramObject::Enable() { RecompileIfNeeded(); glUseProgram(objID); IProgramObject::Enable(); }
-	void GLSLProgramObject::Disable() { glUseProgram(0); IProgramObject::Disable(); }
+	GLSLProgramObject::~GLSLProgramObject() {
+		IProgramObject::Release();
+		glDeleteProgram(objID);
+		objID = 0;
+	}
+
+	void GLSLProgramObject::Enable() {
+		RecompileIfNeeded(true);
+		glUseProgram(objID);
+		IProgramObject::Enable();
+	}
+
+	void GLSLProgramObject::Disable() {
+		glUseProgram(0);
+		IProgramObject::Disable();
+	}
 
 	void GLSLProgramObject::Link() {
-		RecompileIfNeeded();
+		RecompileIfNeeded(false);
 		assert(glIsProgram(objID));
 
 		if (!glIsProgram(objID))
@@ -388,13 +469,13 @@ namespace Shader {
 
 		glValidateProgram(objID);
 		glGetProgramiv(objID, GL_VALIDATE_STATUS, &validated);
+		valid = bool(validated);
 
 		// append the validation-log
 		log += glslGetLog(objID);
 
-		valid = valid && bool(validated);
-
 	#ifdef DEBUG
+		// check if there are unset uniforms left
 		GLsizei numUniforms, maxUniformNameLength;
 		glGetProgramiv(objID, GL_ACTIVE_UNIFORMS, &numUniforms);
 		glGetProgramiv(objID, GL_ACTIVE_UNIFORM_MAX_LENGTH, &maxUniformNameLength);
@@ -417,8 +498,7 @@ namespace Shader {
 				continue;
 
 			const auto hash = hashString(&bufname[0]);
-			auto it = uniformStates.find(hash);
-			if (it != uniformStates.end())
+			if (uniformStates.find(hash) != uniformStates.end())
 				continue;
 
 			LOG_L(L_WARNING, "[GLSL-PO::%s] program-object name: %s, unset uniform: %s", __FUNCTION__, name.c_str(), &bufname[0]);
@@ -429,73 +509,100 @@ namespace Shader {
 
 	void GLSLProgramObject::Release() {
 		IProgramObject::Release();
-
 		glDeleteProgram(objID);
+		ClearHash();
+		curFlagsHash = 0;
 		objID = 0;
+		objID = glCreateProgram();
+		curSrcHash = 0;
 	}
 
-	void GLSLProgramObject::Reload(bool reloadFromDisk) {
-		const unsigned int oldHash = curHash;
-		curHash = GetHash();
+	void GLSLProgramObject::Reload(bool reloadFromDisk, bool validate) {
+		const auto   oldSrcHash = curSrcHash;
+		      bool   oldValid  = IsValid();
+		const GLuint oldProgID = objID;
 
+		// reload shader from disk?
+		reloadFromDisk = reloadFromDisk || !oldValid || (oldProgID == 0);
+		if (reloadFromDisk) {
+			bool sourceChanged = false;
+			for (IShaderObject*& so: GetAttachedShaderObjs()) {
+				sourceChanged |= so->ReloadFromDisk();
+			}
+		}
+
+		//
 		log = "";
 		valid = false;
 
-		if (GetAttachedShaderObjs().empty()) {
-			return;
+		// create shader source hash
+		curFlagsHash = GetHash();
+		curSrcHash = curFlagsHash;
+		for (const IShaderObject* so: GetAttachedShaderObjs()) {
+			curSrcHash ^= so->GetHash();
 		}
 
-		GLuint oldProgID = objID;
-
+		// clear all uniform locations
 		for (auto& us_pair: uniformStates) {
 			us_pair.second.SetLocation(GL_INVALID_INDEX);
 		}
 
-		bool deleteOldShader = false;
-		if (glslShadersCache.find(oldHash) == glslShadersCache.end()) {
-			glslShadersCache[oldHash] = oldProgID;
-		} else {
-			for (IShaderObject*& so: GetAttachedShaderObjs()) {
-				glDetachShader(oldProgID, so->GetObjID());
-				so->Release();
-			}
-			deleteOldShader = true;
+		// early-exit: empty program
+		// TODO delete existing program if exists?
+		if (GetAttachedShaderObjs().empty())
+			return;
+
+		// push old program to cache and pop new if available
+		const bool useShaderCache = oldValid && configHandler->GetBool("UseShaderCache");
+		bool deleteOldShader = true;
+		objID = 0;
+		if (useShaderCache) {
+			CShaderHandler::ShaderCache& shadersCache = shaderHandler->GetShaderCache();
+			deleteOldShader = !shadersCache.Push(oldSrcHash, oldProgID);
+			objID = shadersCache.Find(curSrcHash);
 		}
 
-		auto it = glslShadersCache.find(curHash);
-		if (it != glslShadersCache.end()) {
-			objID = it->second;
-			glslShadersCache.erase(it);
-		} else {
+		// recompile if not found in cache (id 0)
+		if (objID == 0) {
 			objID = glCreateProgram();
+
+			bool shadersValid = true;
 			for (IShaderObject*& so: GetAttachedShaderObjs()) {
-				so->Compile(reloadFromDisk); //FIXME check if changed or not (when it did, we can't use shader cache!)
-			}
-			for (IShaderObject*& so: GetAttachedShaderObjs()) {
-				if (so->IsValid()) {
-					glAttachShader(objID, so->GetObjID());
+				assert(dynamic_cast<GLSLShaderObject*>(so));
+				auto gso = static_cast<GLSLShaderObject*>(so);
+				auto obj = gso->CompileShaderObject();
+
+				if (obj->valid) {
+					glAttachShader(objID, obj->id);
+				} else {
+					shadersValid = false;
 				}
 			}
+			if (!shadersValid)
+				return;
+
 			Link();
+		} else {
+			valid = true;
 		}
 
-		GLSLCopyState(objID, oldProgID, &((IProgramObject*)(this))->uniformStates);
+		//
+		/*
+		if (validate) {
+			Validate(); //FIXME: fails on ATI, see https://springrts.com/mantis/view.php?id=4715
+		}
+		*/
 
+		// copy full program state from old to new program (uniforms etc.)
+		if (IsValid()) {
+			GLSLCopyState(objID, oldValid ? oldProgID : 0, &uniformStates);
+		}
+
+		// delete old program when not further used
 		if (deleteOldShader)
 			glDeleteProgram(oldProgID);
 	}
 
-	void GLSLProgramObject::AttachShaderObject(IShaderObject* so) {
-		if (so != NULL) {
-			assert(!IsShaderAttached(so));
-			IProgramObject::AttachShaderObject(so);
-
-			if (so->IsValid()) {
-				glAttachShader(objID, so->GetObjID());
-			}
-		}
-	}
-
 	int GLSLProgramObject::GetUniformType(const int loc) {
 		GLint size = 0;
 		GLenum type = 0;
@@ -552,27 +659,4 @@ namespace Shader {
 	void GLSLProgramObject::SetUniformMatrix2fv(int idx, bool transp, const float* v) { assert(IsBound()); auto it = uniformStates.find(uniformLocs[idx]); if (it != uniformStates.end() && it->second.Set2x2(v, transp)) glUniformMatrix2fv(it->second.GetLocation(), 1, transp, v); }
 	void GLSLProgramObject::SetUniformMatrix3fv(int idx, bool transp, const float* v) { assert(IsBound()); auto it = uniformStates.find(uniformLocs[idx]); if (it != uniformStates.end() && it->second.Set3x3(v, transp)) glUniformMatrix3fv(it->second.GetLocation(), 1, transp, v); }
 	void GLSLProgramObject::SetUniformMatrix4fv(int idx, bool transp, const float* v) { assert(IsBound()); auto it = uniformStates.find(uniformLocs[idx]); if (it != uniformStates.end() && it->second.Set4x4(v, transp)) glUniformMatrix4fv(it->second.GetLocation(), 1, transp, v); }
-	void GLSLProgramObject::SetUniformMatrixArray4fv(int idx, int count, bool transp, const float* v) { assert(IsBound()); /*glUniformMatrix4fv(it->second.GetLocation(), count, transp, v);*/ }
-
-	#ifdef glUniformMatrix2dv
-	void GLSLProgramObject::SetUniformMatrix2dv(int idx, bool transp, const double* v) { /*glUniformMatrix2dv(it->second.GetLocation(), 1, transp, v);*/ }
-	#else
-	void GLSLProgramObject::SetUniformMatrix2dv(int idx, bool transp, const double* v) {}
-	#endif
-	#ifdef glUniformMatrix3dv
-	void GLSLProgramObject::SetUniformMatrix3dv(int idx, bool transp, const double* v) { /*glUniformMatrix3dv(it->second.GetLocation(), 1, transp, v);*/ }
-	#else
-	void GLSLProgramObject::SetUniformMatrix3dv(int idx, bool transp, const double* v) {}
-	#endif
-	#ifdef glUniformMatrix4dv
-	void GLSLProgramObject::SetUniformMatrix4dv(int idx, bool transp, const double* v) { /*glUniformMatrix4dv(it->second.GetLocation(), 1, transp, v);*/ }
-	#else
-	void GLSLProgramObject::SetUniformMatrix4dv(int idx, bool transp, const double* v) {}
-	#endif
-
-	#ifdef glUniformMatrix4dv
-	void GLSLProgramObject::SetUniformMatrixArray4dv(int idx, int count, bool transp, const double* v) { /*glUniformMatrix4dv(it->second.GetLocation(), count, transp, v);*/ }
-	#else
-	void GLSLProgramObject::SetUniformMatrixArray4dv(int idx, int count, bool transp, const double* v) {}
-	#endif
 }
