@@ -12,6 +12,7 @@
 #include "Sim/Units/UnitHandler.h"
 #include "System/Matrix44f.h"
 #include "Sim/Misc/LosHandler.h"
+#include "Sim/Misc/Resource.h"
 
 #define TREE_RADIUS 20
 
@@ -53,10 +54,7 @@ public:
 	void UpdateFinalHeight(bool useGroundHeight);
 	void StartFire();
 	void EmitGeoSmoke();
-	float RemainingResource(float res) const;
-	float RemainingMetal() const;
-	float RemainingEnergy() const;
-	int ChunkNumber(float f) const;
+
 	void CalculateTransform();
 	void DependentDied(CObject *o);
 	void ChangeTeam(int newTeam);
@@ -70,6 +68,9 @@ public:
 	CMatrix44f GetTransformMatrix(const bool synced = false, const bool error = false) const { return transMatrix; }
 	const CMatrix44f& GetTransformMatrixRef() const { return transMatrix; }
 
+private:
+	static int ChunkNumber(float f);
+
 public:
 	int defID;
 
@@ -83,12 +84,14 @@ public:
 	bool isRepairingBeforeResurrect;
 	bool isAtFinalHeight;
 	bool inUpdateQue;
+	bool deleteMe;
+	
+	float finalHeight;
 
 	float resurrectProgress;
 	float reclaimLeft;
-	float finalHeight;
-
 	int lastReclaim;
+	SResourcePack resources;
 
 	/// which drawQuad we are part of
 	int drawQuad;
@@ -103,6 +106,7 @@ public:
 	/// the solid object that is on top of the geothermal
 	CSolidObject* solidOnTop;
 
+
 private:
 	void PostLoad();
 
