@@ -13,13 +13,24 @@
 CR_BIND(CCategoryHandler, )
 
 CR_REG_METADATA(CCategoryHandler, (
-		CR_MEMBER(categories),
-		CR_MEMBER(firstUnused),
-		CR_RESERVED(8)
-		))
+	CR_MEMBER(categories),
+	CR_MEMBER(firstUnused),
+	CR_RESERVED(8)
+))
 
-CCategoryHandler* CCategoryHandler::instance;
+CCategoryHandler* CCategoryHandler::instance = NULL;
 
+CCategoryHandler* CCategoryHandler::Instance() {
+	assert(instance != NULL);
+	return instance;
+}
+
+
+void CCategoryHandler::CreateInstance() {
+	if (instance == NULL) {
+		instance = new CCategoryHandler();
+	}
+}
 
 void CCategoryHandler::RemoveInstance()
 {
@@ -28,16 +39,6 @@ void CCategoryHandler::RemoveInstance()
 }
 
 
-CCategoryHandler::CCategoryHandler() : firstUnused(0)
-{
-}
-
-
-CCategoryHandler::~CCategoryHandler()
-{
-}
-
-
 unsigned int CCategoryHandler::GetCategory(std::string name)
 {
 	StringTrimInPlace(name);
@@ -81,6 +82,7 @@ unsigned int CCategoryHandler::GetCategories(std::string names)
 	// split on ' '
 	std::stringstream namesStream(names);
 	std::string name;
+
 	while (std::getline(namesStream, name, ' ')) {
 		if (!name.empty()) {
 			ret |= GetCategory(name);
@@ -95,11 +97,9 @@ std::vector<std::string> CCategoryHandler::GetCategoryNames(unsigned int bits) c
 {
 	std::vector<std::string> names;
 
-	std::map<std::string, unsigned int>::const_iterator it;
-
 	for (unsigned int bit = 1; bit != 0; bit = (bit << 1)) {
 		if ((bit & bits) != 0) {
-			for (it = categories.begin(); it != categories.end(); ++it) {
+			for (auto it = categories.cbegin(); it != categories.cend(); ++it) {
 				if (it->second == bit) {
 					names.push_back(it->first);
 				}
@@ -109,3 +109,4 @@ std::vector<std::string> CCategoryHandler::GetCategoryNames(unsigned int bits) c
 
 	return names;
 }
+
