@@ -3,11 +3,11 @@
 
 #include "CollisionHandler.h"
 #include "CollisionVolume.h"
+#include "Map/ReadMap.h"
 #include "Rendering/Models/3DModel.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Features/Feature.h"
 #include "Sim/Misc/GroundBlockingObjectMap.h"
-#include "Sim/Misc/GlobalSynced.h"
 #include "Sim/Misc/GlobalConstants.h"
 #include "System/FastMath.h"
 #include "System/Matrix44f.h"
@@ -122,9 +122,9 @@ bool CCollisionHandler::CollisionFootPrint(const CSolidObject* o, const float3&
 	// projectile might have tunneled across blocking squares to get to <p>)
 	// note: if we get here <v> is always a box
 	const float invSquareSize = 1.0f / SQUARE_SIZE;
-	const int squareIdx = int(p.x * invSquareSize) + int(p.z * invSquareSize) * gs->mapx;
+	const int squareIdx = int(p.x * invSquareSize) + int(p.z * invSquareSize) * mapDims.mapx;
 
-	if (squareIdx < 0 || squareIdx >= gs->mapSquares)
+	if (squareIdx < 0 || squareIdx >= mapDims.mapSquares)
 		return false;
 
 	const BlockingMapCell& cell = groundBlockingObjectMap->GetCell(squareIdx);
@@ -256,16 +256,10 @@ bool CCollisionHandler::IntersectPiecesHelper(
 	CMatrix44f unitMat = u->GetTransformMatrix(true);
 	CMatrix44f volMat;
 
-	CollisionQuery cqt;
-
-	if (cq == NULL)
-		cq = &cqt;
-
 	float minDistSq = std::numeric_limits<float>::max();
-	float curDistSq = std::numeric_limits<float>::max();
 
 	for (unsigned int n = 0; n < u->localModel->pieces.size(); n++) {
-		const LocalModelPiece* lmp = u->localModel->GetPiece(n);
+		      LocalModelPiece* lmp = u->localModel->GetPiece(n);
 		const CollisionVolume* lmpVol = lmp->GetCollisionVolume();
 
 		if (!lmp->scriptSetVisible || lmpVol->IgnoreHits())
@@ -274,24 +268,31 @@ bool CCollisionHandler::IntersectPiecesHelper(
 		volMat = unitMat * lmp->GetModelSpaceMatrix();
 		volMat.Translate(lmpVol->GetOffsets());
 
-		if (!CCollisionHandler::Intersect(lmpVol, volMat, p0, p1, cq))
-			continue;
-		// skip if neither an ingress nor an egress hit
-		if (cq->GetHitPos() == ZeroVector)
+		CollisionQuery cqn;
+		if (!CCollisionHandler::Intersect(lmpVol, volMat, p0, p1, &cqn))
 			continue;
 
-		cq->SetHitPiece(const_cast<LocalModelPiece*>(lmp));
+		// skip if neither an ingress nor an egress hit
+		if (!cqn.AnyHit())
+			continue;
 
 		// save the closest intersection (others are not needed)
-		if ((curDistSq = (cq->GetHitPos() - p0).SqLength()) >= minDistSq)
+		if (cqn.GetHitPos().SqDistance(p0) >= minDistSq)
 			continue;
 
-		minDistSq = curDistSq;
+		minDistSq = cqn.GetHitPos().SqDistance(p0);
+
+		if (cq != nullptr) {
+			*cq = cqn;
+			cq->SetHitPiece(lmp);
+		} else {
+			return true;
+		}
 	}
 
 	// true iff at least one piece was intersected
 	// (query must have been reset by calling code)
-	return (cq->GetHitPiece() != NULL);
+	return (cq != nullptr) ? (cq->GetHitPiece() != nullptr) : false;
 }
 
 
