@@ -27,6 +27,8 @@ public:
 
 	CGlobalSynced();  //!< Constructor
 	~CGlobalSynced(); //!< Destructor
+
+	void ResetState();
 	void LoadFromSetup(const CGameSetup*);
 
 	int    randInt();    //!< synced random int
@@ -48,6 +50,15 @@ public:
 	*/
 	int frameNum;
 
+	/**
+	* @brief temp num
+	*
+	* Used for getting temporary but unique numbers
+	* (increase after each use)
+	*/
+	int tempNum;
+
+
 	/**
 	* @brief speed factor
 	*
@@ -68,6 +79,7 @@ public:
 	*/
 	float wantedSpeedFactor;
 
+
 	/**
 	* @brief paused
 	*
@@ -75,70 +87,6 @@ public:
 	*/
 	bool paused;
 
-	/**
-	* @brief map x
-	*
-	* The map's number of squares in the x direction
-	* (note that the number of vertices is one more)
-	*/
-	int mapx;
-	int mapxm1; // mapx minus one
-	int mapxp1; // mapx plus one
-
-	/**
-	* @brief map y
-	*
-	* The map's number of squares in the y direction
-	*/
-	int mapy;
-	int mapym1; // mapy minus one
-	int mapyp1; // mapy plus one
-
-	/**
-	* @brief map squares
-	*
-	* Total number of squares on the map
-	*/
-	int mapSquares;
-
-	/**
-	* @brief half map x
-	*
-	* Contains half of the number of squares in the x direction
-	*/
-	int hmapx;
-
-	/**
-	* @brief half map y
-	*
-	* Contains half of the number of squares in the y direction
-	*/
-	int hmapy;
-
-	/**
-	* @brief map x power of 2
-	*
-	* Map's size in the x direction rounded
-	* up to the next power of 2
-	*/
-	int pwr2mapx;
-
-	/**
-	* @brief map y power of 2
-	*
-	* Map's size in the y direction rounded
-	* up to the next power of 2
-	*/
-	int pwr2mapy;
-
-	/**
-	* @brief temp num
-	*
-	* Used for getting temporary but unique numbers
-	* (increase after each use)
-	*/
-	int tempNum;
-
 	/**
 	* @brief god mode
 	*
@@ -183,6 +131,20 @@ public:
 	*/
 	bool useLuaGaia;
 
+private:
+	class SyncedRNG
+	{
+	public:
+		int operator()(unsigned N)
+		{
+			extern CGlobalSynced* gs;
+			return gs->randInt()%N;
+		};
+	};
+
+public:
+	static SyncedRNG rng;
+
 private:
 	/**
 	* @brief random seed
@@ -201,14 +163,4 @@ private:
 
 extern CGlobalSynced* gs;
 
-
-class SyncedRNG
-{
-public:
-	int operator()(unsigned N)
-	{
-		return gs->randInt()%N;
-	};
-};
-
 #endif // _GLOBAL_SYNCED_H
