@@ -7,11 +7,12 @@
 
 #include <vector>
 #include "System/type2.h"
+#include "System/myMath.h"
 
 /// map containing counts of how many units have Line Of Sight (LOS) to each square
 class CLosMap
 {
-	CR_DECLARE_STRUCT(CLosMap);
+	CR_DECLARE_STRUCT(CLosMap)
 
 public:
 	CLosMap() : size(0, 0), sendReadmapEvents(false) {}
@@ -28,8 +29,8 @@ public:
 	int operator[] (int square) const { return map[square]; }
 
 	int At(int x, int y) const {
-		x = std::max(0, std::min(size.x - 1, x));
-		y = std::max(0, std::min(size.y - 1, y));
+		x = Clamp(x, 0, size.x - 1);
+		y = Clamp(y, 0, size.y - 1);
 		return map[y * size.x + x];
 	}
 
@@ -46,7 +47,7 @@ protected:
 /// algorithm to calculate LOS squares using raycasting, taking terrain into account
 class CLosAlgorithm
 {
-	CR_DECLARE_STRUCT(CLosAlgorithm);
+	CR_DECLARE_STRUCT(CLosAlgorithm)
 
 public:
 	CLosAlgorithm(int2 size, float minMaxAng, float extraHeight, const float* heightmap)
