@@ -4,16 +4,15 @@
 #include "Sim/Misc/GlobalSynced.h"
 #include "Sim/Units/Scripts/UnitScript.h"
 
-CR_BIND(NanoPieceCache, );
+CR_BIND(NanoPieceCache, )
 
 CR_REG_METADATA(NanoPieceCache, (
 	CR_MEMBER(nanoPieces),
 	CR_MEMBER(lastNanoPieceCnt),
 	CR_MEMBER(curBuildPowerMask)
-));
+))
 
 int NanoPieceCache::GetNanoPiece(CUnitScript* ownerScript) {
-	assert(UNIT_SLOWUPDATE_RATE == 16);
 	curBuildPowerMask |= (1 << (UNIT_SLOWUPDATE_RATE - 1));
 
 	int nanoPiece = -1;
