@@ -3,42 +3,40 @@
 #ifndef QUAD_FIELD_H
 #define QUAD_FIELD_H
 
-#include <set>
 #include <vector>
-#include <list>
 #include <boost/noncopyable.hpp>
 
 #include "System/creg/creg_cond.h"
 #include "System/float3.h"
+#include "System/type2.h"
 
 class CUnit;
 class CFeature;
 class CProjectile;
 class CSolidObject;
 
+
 class CQuadField : boost::noncopyable
 {
 	CR_DECLARE_STRUCT(CQuadField)
 	CR_DECLARE_SUB(Quad)
 
 public:
-	static void Resize(unsigned int nqx, unsigned int nqz);
 
-	CQuadField(unsigned int nqx, unsigned int nqz);
+/*
+needed to support dynamic resizing (not used yet)
+      in large games the average loading factor (number of objects per quad)
+      can grow too large to maintain amortized constant performance so more
+      quads are needed
+*/
+//	static void Resize(int quad_size);
+
+	CQuadField(int2 mapDims, int quad_size);
 	~CQuadField();
 
-	std::vector<int> GetQuads(float3 pos, float radius) const;
-	std::vector<int> GetQuadsRectangle(const float3& pos1, const float3& pos2) const;
-
-	// optimized functions, somewhat less userfriendly
-	//
-	// when calling these, <begQuad> and <endQuad> are both expected
-	// to point to the *start* of an array of int's of size at least
-	// numQuadsX * numQuadsZ (eg. tempQuads) -- GetQuadsOnRay ensures
-	// this by itself, for GetQuads the callers take care of it
-	//
-	unsigned int GetQuads(float3 pos, float radius, int*& begQuad, int*& endQuad) const;
-	unsigned int GetQuadsOnRay(float3 start, float3 dir, float length, int*& begQuad, int*& endQuad);
+	std::vector<int> GetQuads(float3 pos, const float radius);
+	std::vector<int> GetQuadsRectangle(const float3 mins, const float3 maxs);
+	std::vector<int> GetQuadsOnRay(const float3 start, const float3 dir, const float length);
 
 	void GetUnitsAndFeaturesColVol(
 		const float3& pos,
@@ -65,17 +63,12 @@ public:
 	 * mins and maxs, which extends infinitely along the y-axis
 	 */
 	std::vector<CUnit*> GetUnitsExact(const float3& mins, const float3& maxs);
-
-	/**
-	 * Returns all features within @c radius of @c pos,
-	 * and takes the 3D model radius of each feature into account
-	 */
-	std::vector<CFeature*> GetFeaturesExact(const float3& pos, float radius);
 	/**
 	 * Returns all features within @c radius of @c pos,
+	 * takes the 3D model radius of each feature into account,
 	 * and performs the search within a sphere or cylinder depending on @c spherical
 	 */
-	std::vector<CFeature*> GetFeaturesExact(const float3& pos, float radius, bool spherical);
+	std::vector<CFeature*> GetFeaturesExact(const float3& pos, float radius, bool spherical = true);
 	/**
 	 * Returns all features within the rectangle defined by
 	 * mins and maxs, which extends infinitely along the y-axis
@@ -105,21 +98,22 @@ public:
 	struct Quad {
 		CR_DECLARE_STRUCT(Quad)
 		Quad();
-		std::list<CUnit*> units;
-		std::vector< std::list<CUnit*> > teamUnits;
-		std::list<CFeature*> features;
-		std::list<CProjectile*> projectiles;
+		std::vector<CUnit*> units;
+		std::vector< std::vector<CUnit*> > teamUnits;
+		std::vector<CFeature*> features;
+		std::vector<CProjectile*> projectiles;
 	};
 
-	const Quad& GetQuad(int i) const {
-		assert(i >= 0);
+	const Quad& GetQuad(unsigned i) const {
 		assert(i < baseQuads.size());
 		return baseQuads[i];
 	}
-	const Quad& GetQuadAt(int x, int z) const {
+	const Quad& GetQuadAt(unsigned x, unsigned z) const {
+		assert(unsigned(numQuadsX * z + x) < baseQuads.size());
 		return baseQuads[numQuadsX * z + x];
 	}
 
+
 	int GetNumQuadsX() const { return numQuadsX; }
 	int GetNumQuadsZ() const { return numQuadsZ; }
 
@@ -127,11 +121,22 @@ public:
 	int GetQuadSizeZ() const { return quadSizeZ; }
 
 	const static unsigned int BASE_QUAD_SIZE =  128;
-	const static unsigned int NUM_TEMP_QUADS = 1024;
+
+private:
+	// optimized functions, somewhat less userfriendly
+	//
+	// when calling these, <begQuad> and <endQuad> are both expected
+	// to point to the *start* of an array of int's of size at least
+	// numQuadsX * numQuadsZ (eg. tempQuads) -- GetQuadsOnRay ensures
+	// this by itself, for GetQuads the callers take care of it
+	//
+	void GetQuads(float3 pos, float radius, std::vector<int>* quads) const;
+
+	int2 WorldPosToQuadField(const float3 p) const;
+	int WorldPosToQuadFieldIdx(const float3 p) const;
 
 private:
 	std::vector<Quad> baseQuads;
-	std::vector<int> tempQuads;
 
 	int numQuadsX;
 	int numQuadsZ;
