@@ -19,8 +19,8 @@ CR_BIND(MoveDefHandler, (NULL))
 CR_REG_METADATA(MoveDef, (
 	CR_MEMBER(name),
 
-	CR_ENUM_MEMBER(speedModClass),
-	CR_ENUM_MEMBER(terrainClass),
+	CR_MEMBER(speedModClass),
+	CR_MEMBER(terrainClass),
 
 	CR_MEMBER(xsize),
 	CR_MEMBER(xsizeh),
@@ -314,11 +314,12 @@ MoveDef::MoveDef(const LuaTable& moveDefTable, int moveDefID) {
 	assert((zsize & 1) == 1);
 }
 
+
 bool MoveDef::TestMoveSquare(
 	const CSolidObject* collider,
 	const int xTestMoveSqr,
 	const int zTestMoveSqr,
-	const float3& testMoveDir,
+	const float3 testMoveDir,
 	bool testTerrain,
 	bool testObjects,
 	bool centerOnly,
@@ -326,36 +327,41 @@ bool MoveDef::TestMoveSquare(
 	int* maxBlockBit
 ) const {
 	bool ret = true;
+	assert(testTerrain || testObjects);
+	float                minSpeedMod_ = std::numeric_limits<float>::max();;
+	CMoveMath::BlockType maxBlockBit_ = CMoveMath::BLOCK_NONE;
 
-	if (testTerrain || testObjects) {
-		const int jMin = -zsizeh * (1 - centerOnly), jMax = zsizeh * (1 - centerOnly);
-		const int iMin = -xsizeh * (1 - centerOnly), iMax = xsizeh * (1 - centerOnly);
+	const int zMin = -zsizeh * (1 - centerOnly), zMax = zsizeh * (1 - centerOnly);
+	const int xMin = -xsizeh * (1 - centerOnly), xMax = xsizeh * (1 - centerOnly);
 
-		for (int j = jMin; (j <= jMax) && ret; j++) {
-			for (int i = iMin; (i <= iMax) && ret; i++) {
-				// GetPosSpeedMod only checks *one* square of terrain
-				// (heightmap/slopemap/typemap), not the blocking-map
-				const float speedMod = (testMoveDir != ZeroVector)?
-					CMoveMath::GetPosSpeedMod(*this, xTestMoveSqr + i, zTestMoveSqr + j, testMoveDir):
-					CMoveMath::GetPosSpeedMod(*this, xTestMoveSqr + i, zTestMoveSqr + j);
-				const CMoveMath::BlockType blockBits = CMoveMath::SquareIsBlocked(*this, xTestMoveSqr + i, zTestMoveSqr + j, collider);
-
-				if (testTerrain) { ret &= (speedMod > 0.0f); }
-				if (testObjects) { ret &= ((blockBits & CMoveMath::BLOCK_STRUCTURE) == 0); }
-
-				if (minSpeedMod != NULL) { *minSpeedMod = std::min(*minSpeedMod,    (speedMod )); }
-				if (maxBlockBit != NULL) { *maxBlockBit = std::max(*maxBlockBit, int(blockBits)); }
-			}
+	for (int z = zMin; (z <= zMax) && ret; z++) {
+		for (int x = xMin; (x <= xMax) && ret; x++) {
+			const float speedMod = CMoveMath::GetPosSpeedMod(*this, xTestMoveSqr + x, zTestMoveSqr + z, testMoveDir);
+			minSpeedMod_ = std::min(minSpeedMod_, speedMod);
+			if (testTerrain) { ret &= (speedMod > 0.0f); }
 		}
 	}
 
+	// GetPosSpeedMod only checks *one* square of terrain
+	// (heightmap/slopemap/typemap), not the blocking-map
+	for (int z = zMin; (z <= zMax) && ret; z++) {
+		for (int x = xMin; (x <= xMax) && ret; x++) {
+			const CMoveMath::BlockType blockBits = CMoveMath::SquareIsBlocked(*this, xTestMoveSqr + x, zTestMoveSqr + z, collider);
+			maxBlockBit_ |= blockBits;
+			if (testObjects) { ret &= !(blockBits & CMoveMath::BLOCK_STRUCTURE); }
+		}
+	}
+
+	if (minSpeedMod != nullptr) *minSpeedMod  = std::min(*minSpeedMod, minSpeedMod_);
+	if (maxBlockBit != nullptr) *maxBlockBit |= int(maxBlockBit_);
 	return ret;
 }
 
+
 bool MoveDef::TestMoveSquare(
 	const CSolidObject* collider,
-	const float3& testMovePos,
-	const float3& testMoveDir,
+	const float3 testMovePos,
+	const float3 testMoveDir,
 	bool testTerrain,
 	bool testObjects,
 	bool centerOnly,
