@@ -14,7 +14,7 @@ class MoveDefHandler;
 class CSolidObject;
 class LuaTable;
 
-#pragma pack(push, 1)
+
 struct MoveDef {
 	CR_DECLARE_STRUCT(MoveDef)
 
@@ -25,7 +25,7 @@ struct MoveDef {
 		const CSolidObject* collider,
 		const int xTestMoveSqr,
 		const int zTestMoveSqr,
-		const float3& testMoveDir = ZeroVector,
+		const float3 testMoveDir,
 		bool testTerrain = true,
 		bool testObjects = true,
 		bool centerOnly = false,
@@ -34,8 +34,8 @@ struct MoveDef {
 	) const;
 	bool TestMoveSquare(
 		const CSolidObject* collider,
-		const float3& testMovePos,
-		const float3& testMoveDir = ZeroVector,
+		const float3 testMovePos,
+		const float3 testMoveDir,
 		bool testTerrain = true,
 		bool testObjects = true,
 		bool centerOnly = false,
@@ -130,7 +130,7 @@ struct MoveDef {
 	bool heatMapping;
 	bool flowMapping;
 };
-#pragma pack(pop)
+
 
 
 class LuaParser;
