@@ -12,12 +12,13 @@ class CUnit;
 
 class AMoveType : public CObject
 {
-	CR_DECLARE(AMoveType);
+	CR_DECLARE(AMoveType)
 
 public:
 	AMoveType(CUnit* owner);
 	virtual ~AMoveType() {}
 
+	virtual void StartMovingRaw(const float3 moveGoalPos, float moveGoalRadius) {}
 	virtual void StartMoving(float3 pos, float goalRadius) = 0;
 	virtual void StartMoving(float3 pos, float goalRadius, float speed) = 0;
 	virtual void KeepPointingTo(float3 pos, float distance, bool aggressive) = 0;
@@ -41,6 +42,7 @@ public:
 	//     MoveType classes expects maxSpeed to be != 0
 	virtual void SetMaxSpeed(float speed) { maxSpeed = std::max(0.001f, speed); }
 	virtual void SetWantedMaxSpeed(float speed) { maxWantedSpeed = speed; }
+	virtual void SetManeuverLeash(float leashLength) { maneuverLeash = leashLength; }
 
 	virtual bool Update() = 0;
 	virtual void SlowUpdate();
@@ -62,6 +64,9 @@ public:
 	float GetMaxSpeedDef() const { return maxSpeedDef; }
 	float GetMaxWantedSpeed() const { return maxWantedSpeed; }
 	float GetRepairBelowHealth() const { return repairBelowHealth; }
+	float GetManeuverLeash() const { return maneuverLeash; }
+
+	float CalcStaticTurnRadius() const;
 
 public:
 	CUnit* owner;
@@ -86,6 +91,7 @@ protected:
 	float maxWantedSpeed;      // maximum speed (temporarily) set by a CMD_SET_WANTED_MAX_SPEED modifier command
 
 	float repairBelowHealth;
+	float maneuverLeash;       // maximum distance away a target can be and still be chased
 };
 
 #endif // MOVETYPE_H
