@@ -6,10 +6,10 @@
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitDef.h"
 
-CR_BIND_DERIVED(CStaticMoveType, AMoveType, (NULL));
+CR_BIND_DERIVED(CStaticMoveType, AMoveType, (NULL))
 CR_REG_METADATA(CStaticMoveType, (
 	CR_RESERVED(63)
-));
+))
 
 void CStaticMoveType::SlowUpdate()
 {
