@@ -31,7 +31,6 @@ public:
 	void UpdateManeuver();
 	void UpdateAttack();
 	bool UpdateFlying(float wantedHeight, float engine);
-	void UpdateLanded();
 	void UpdateLanding();
 	void UpdateAirPhysics(
 		float rudder,
@@ -77,6 +76,8 @@ public:
 	float maxAileron;
 	float maxElevator;
 	float maxRudder;
+	// fighters abort dive toward target if within this distance and climb back to normal altitude
+	float attackSafetyDistance;
 
 	/// used while landing
 	float crashAileron;
