@@ -7,9 +7,16 @@
 #include "Sim/MoveTypes/MoveDefHandler.h"
 #include "Sim/Misc/GlobalSynced.h"
 
-CPathFinderDef::CPathFinderDef(const float3& goalCenter, float goalRadius, float sqGoalDistance):
-goal(goalCenter),
-sqGoalRadius(goalRadius * goalRadius)
+
+CPathFinderDef::CPathFinderDef(const float3& goalCenter, float goalRadius, float sqGoalDistance)
+: goal(goalCenter)
+, sqGoalRadius(goalRadius * goalRadius)
+, constraintDisabled(false)
+, testMobile(true)
+, needPath(true)
+, exactPath(true)
+, dirIndependent(false)
+, synced(true)
 {
 	goalSquareX = goalCenter.x / SQUARE_SIZE;
 	goalSquareZ = goalCenter.z / SQUARE_SIZE;
@@ -35,8 +42,8 @@ float CPathFinderDef::Heuristic(unsigned int xSquare, unsigned int zSquare) cons
 
 // returns if the goal is inaccessable: this is
 // true if the goal area is "small" and blocked
-bool CPathFinderDef::GoalIsBlocked(const MoveDef& moveDef, const CMoveMath::BlockType& blockMask, const CSolidObject* owner) const {
-	const float r0 = SQUARE_SIZE * SQUARE_SIZE * 4.0f; // (SQUARE_SIZE*2)^2
+bool CPathFinderDef::IsGoalBlocked(const MoveDef& moveDef, const CMoveMath::BlockType& blockMask, const CSolidObject* owner) const {
+	const float r0 = SQUARE_SIZE * SQUARE_SIZE * 4.0f; // same as (SQUARE_SIZE*2)^2
 	const float r1 = ((moveDef.xsize * SQUARE_SIZE) >> 1) * ((moveDef.zsize * SQUARE_SIZE) >> 1) * 1.5f;
 
 	if (sqGoalRadius >= r0 && sqGoalRadius > r1)
@@ -60,16 +67,14 @@ int2 CPathFinderDef::GoalSquareOffset(unsigned int blockSize) const {
 
 
 
-CRangedGoalWithCircularConstraint::CRangedGoalWithCircularConstraint(
+CCircularSearchConstraint::CCircularSearchConstraint(
 	const float3& start,
 	const float3& goal,
 	float goalRadius,
 	float searchSize,
-	unsigned int extraSize):
-	CPathFinderDef(goal, goalRadius, start.SqDistance2D(goal))
+	unsigned int extraSize
+): CPathFinderDef(goal, goalRadius, start.SqDistance2D(goal))
 {
-	disabled = false;
-
 	// calculate the center and radius of the constrained area
 	const unsigned int startX = start.x / SQUARE_SIZE;
 	const unsigned int startZ = start.z / SQUARE_SIZE;
@@ -87,13 +92,31 @@ CRangedGoalWithCircularConstraint::CRangedGoalWithCircularConstraint(
 	searchRadiusSq += extraSize;
 }
 
-// tests if a square is inside is the circular constrained area
-// defined by the start and goal positions (disabled: this only
-// saves CPU under certain conditions and destroys admissibility)
-bool CRangedGoalWithCircularConstraint::WithinConstraints(unsigned int xSquare, unsigned int zSquare) const
-{
-	const int dx = halfWayX - xSquare;
-	const int dz = halfWayZ - zSquare;
 
-	return (disabled || ((dx * dx + dz * dz) <= searchRadiusSq));
+
+CRectangularSearchConstraint::CRectangularSearchConstraint(
+	const float3 startPos,
+	const float3 goalPos,
+	unsigned int blockSize
+): CPathFinderDef(goalPos, 0.0f, startPos.SqDistance2D(goalPos))
+{
+	unsigned int startBlockX = startPos.x / SQUARE_SIZE;
+	unsigned int startBlockZ = startPos.z / SQUARE_SIZE;
+	unsigned int  goalBlockX =  goalPos.x / SQUARE_SIZE;
+	unsigned int  goalBlockZ =  goalPos.z / SQUARE_SIZE;
+	startBlockX -= startBlockX % blockSize;
+	startBlockZ -= startBlockZ % blockSize;
+	 goalBlockX -=  goalBlockX % blockSize;
+	 goalBlockZ -=  goalBlockZ % blockSize;
+
+	startBlockRect.x1 = startBlockX;
+	startBlockRect.z1 = startBlockZ;
+	startBlockRect.x2 = startBlockX + blockSize;
+	startBlockRect.z2 = startBlockZ + blockSize;
+
+	goalBlockRect.x1 = goalBlockX;
+	goalBlockRect.z1 = goalBlockZ;
+	goalBlockRect.x2 = goalBlockX + blockSize;
+	goalBlockRect.z2 = goalBlockZ + blockSize;
 }
+
