@@ -7,27 +7,29 @@
 #include "Sim/MoveTypes/MoveDefHandler.h"
 #include "Sim/Objects/SolidObject.h"
 
+// not extern'ed, so static
+static PathHeatMap* gPathHeatMap = NULL;
+
 PathHeatMap* PathHeatMap::GetInstance() {
-	static PathHeatMap* phm = NULL;
+	if (gPathHeatMap == NULL)
+		gPathHeatMap = new PathHeatMap(PATH_HEATMAP_XSCALE, PATH_HEATMAP_ZSCALE);
 
-	if (phm == NULL) {
-		phm = new PathHeatMap(PATH_HEATMAP_XSCALE, PATH_HEATMAP_ZSCALE);
-	}
-
-	return phm;
+	return gPathHeatMap;
 }
 
 void PathHeatMap::FreeInstance(PathHeatMap* phm) {
+	assert(phm == gPathHeatMap);
 	delete phm;
+	gPathHeatMap = NULL;
 }
 
 
 
 PathHeatMap::PathHeatMap(unsigned int scalex, unsigned int scalez): enabled(true) {
-	xscale = std::max(1, std::min(gs->hmapx, int(scalex)));
-	zscale = std::max(1, std::min(gs->hmapy, int(scalez)));
-	xsize  = gs->hmapx / xscale;
-	zsize  = gs->hmapy / zscale;
+	xscale = std::max(1, std::min(mapDims.hmapx, int(scalex)));
+	zscale = std::max(1, std::min(mapDims.hmapy, int(scalez)));
+	xsize  = mapDims.hmapx / xscale;
+	zsize  = mapDims.hmapy / zscale;
 
 	heatMap.resize(xsize * zsize, HeatCell());
 	heatMapOffset = 0;
@@ -40,7 +42,7 @@ PathHeatMap::~PathHeatMap() {
 unsigned int PathHeatMap::GetHeatMapIndex(unsigned int hmx, unsigned int hmz) const {
 	assert(!heatMap.empty());
 
-	//! x & y are given in gs->mapi coords (:= gs->hmapi * 2)
+	//! x & y are given in mapDims.mapi coords (:= mapDims.hmapi * 2)
 	hmx >>= xscale;
 	hmz >>= zscale;
 
