@@ -56,14 +56,10 @@ boost::int64_t CPathManager::Finalize() {
 		medResPE = new CPathEstimator(maxResPF, MEDRES_PE_BLOCKSIZE, "pe",  mapInfo->map.name);
 		lowResPE = new CPathEstimator(medResPE, LOWRES_PE_BLOCKSIZE, "pe2", mapInfo->map.name);
 
-		#ifdef SYNCDEBUG
-		// clients may have a non-writable cache directory (which causes
-		// the estimator path-file checksum to remain zero), so we can't
-		// update the sync-checker with this in normal builds
-		// NOTE: better to just checksum the in-memory data and broadcast
-		// that instead of relying on the zip-file CRC?
+		// make cached path data checksum part of synced state
+		// so when one client got a corrupted/incorrect cache
+		// it desyncs from the starts and not minutes later
 		{ SyncedUint tmp(GetPathCheckSum()); }
-		#endif
 	}
 
 	const spring_time dt = spring_gettime() - t0;
@@ -230,49 +226,31 @@ IPath::SearchResult CPathManager::ArrangePath(
 }
 
 
-/*
-Help-function.
-Turns a start->goal-request into a well-defined request.
-*/
-unsigned int CPathManager::RequestPath(
-	CSolidObject* caller,
-	const MoveDef* moveDef,
-	const float3& startPos,
-	const float3& goalPos,
-	float goalRadius,
-	bool synced
-) {
-	float3 sp(startPos); sp.ClampInBounds();
-	float3 gp(goalPos); gp.ClampInBounds();
-
-	// Create an estimator definition.
-	CCircularSearchConstraint* pfDef = new CCircularSearchConstraint(sp, gp, goalRadius, 3.0f, 2000);
-
-	// Make request.
-	return (RequestPath(moveDef, sp, gp, pfDef, caller, synced));
-}
-
-
 /*
 Request a new multipath, store the result and return a handle-id to it.
 */
 unsigned int CPathManager::RequestPath(
-	const MoveDef* moveDef,
-	const float3& startPos,
-	const float3& goalPos,
-	CPathFinderDef* pfDef,
 	CSolidObject* caller,
+	const MoveDef* moveDef,
+	float3 startPos,
+	float3 goalPos,
+	float goalRadius,
 	bool synced
 ) {
-	SCOPED_TIMER("PathManager::RequestPath");
-
 	if (!IsFinalized())
 		return 0;
 
+	SCOPED_TIMER("PathManager::RequestPath");
+	startPos.ClampInBounds();
+	goalPos.ClampInBounds();
+
+	// Create an estimator definition.
+	goalRadius = std::max<float>(goalRadius, PATH_NODE_SPACING * SQUARE_SIZE); //FIXME do on a per PE & PF level?
+	CCircularSearchConstraint* pfDef = new CCircularSearchConstraint(startPos, goalPos, goalRadius, 3.0f, 2000);
 	assert(moveDef == moveDefHandler->GetMoveDefByPathType(moveDef->pathType));
 
 	// Creates a new multipath.
-	MultiPath* newPath = new MultiPath(startPos, pfDef, moveDef);
+	MultiPath* newPath = new MultiPath(startPos, pfDef, moveDef); // deletes pfDef in dtor
 	newPath->finalGoal = goalPos;
 	newPath->caller = caller;
 	pfDef->synced = synced;
