@@ -45,11 +45,11 @@ public:
 	unsigned int RequestPath(
 		CSolidObject* caller,
 		const MoveDef* moveDef,
-		const float3& startPos,
-		const float3& goalPos,
+		float3 startPos,
+		float3 goalPos,
 		float goalRadius,
 		bool synced
-	);
+	) override;
 
 	/**
 	 * Returns waypoints of the max-resolution path segments.
@@ -82,15 +82,6 @@ public:
 	int2 GetNumQueuedUpdates() const;
 
 private:
-	unsigned int RequestPath(
-		const MoveDef* moveDef,
-		const float3& startPos,
-		const float3& goalPos,
-		CPathFinderDef* peDef,
-		CSolidObject* caller,
-		bool synced = true
-	);
-
 	struct MultiPath {
 		MultiPath(const float3& pos, const CPathFinderDef* def, const MoveDef* moveDef)
 			: searchResult(IPath::Error)
@@ -119,6 +110,16 @@ private:
 		CSolidObject* caller;
 	};
 
+private:
+	IPath::SearchResult ArrangePath(
+		MultiPath* newPath,
+		const MoveDef* moveDef,
+		const float3& startPos,
+		const float3& goalPos,
+		CPathFinderDef* peDef,
+		CSolidObject* caller
+	) const;
+
 	inline MultiPath* GetMultiPath(int pathID) const;
 	unsigned int Store(MultiPath* path);
 	static void FinalizePath(MultiPath* path, const float3 startPos, const float3 goalPos, const bool cantGetCloser);
