@@ -8,19 +8,24 @@
 IPathManager* pathManager = NULL;
 
 IPathManager* IPathManager::GetInstance(unsigned int type) {
-	static IPathManager* pm = NULL;
-
-	if (pm == NULL) {
+	if (pathManager == NULL) {
 		const char* fmtStr = "[IPathManager::GetInstance] using %s path-manager";
 		const char* typeStr = "";
 
 		switch (type) {
-			case PFS_TYPE_DEFAULT: { typeStr = "DEFAULT"; pm = new       CPathManager(); } break;
-			case PFS_TYPE_QTPFS:   { typeStr = "QTPFS";   pm = new QTPFS::PathManager(); } break;
+			case PFS_TYPE_DEFAULT: { typeStr = "DEFAULT"; pathManager = new       CPathManager(); } break;
+			case PFS_TYPE_QTPFS:   { typeStr = "QTPFS";   pathManager = new QTPFS::PathManager(); } break;
 		}
 
 		LOG(fmtStr, typeStr);
 	}
 
-	return pm;
+	return pathManager;
 }
+
+void IPathManager::FreeInstance(IPathManager* pm) {
+	assert(pm == pathManager);
+	delete pm;
+	pathManager = NULL;
+}
+
