@@ -80,7 +80,8 @@ void CFireProjectile::Update()
 {
 	ttl--;
 	if (ttl > 0) {
-		if (projectileHandler->particleSaturation < 0.8f || (projectileHandler->particleSaturation < 1 && (gs->frameNum & 1))) {
+		const float partSat = (gs->frameNum & 1) ? 1.0f : 0.8f;
+		if (projectileHandler->GetParticleSaturation() < partSat) {
 			// unsynced code
 			SubParticle sub;
 			sub.age = 0;
@@ -106,14 +107,15 @@ void CFireProjectile::Update()
 			const std::vector<CFeature*>& features = quadField->GetFeaturesExact(emitPos + wind.GetCurrentWind() * 0.7f, emitRadius * 2);
 			const std::vector<CUnit*>& units = quadField->GetUnitsExact(emitPos + wind.GetCurrentWind() * 0.7f, emitRadius * 2);
 
-			for (std::vector<CFeature*>::const_iterator fi = features.begin(); fi != features.end(); ++fi) {
+			for (CFeature* f: features) {
 				if (gs->randFloat() > 0.8f) {
-					(*fi)->StartFire();
+					f->StartFire();
 				}
 			}
 
-			for (std::vector<CUnit*>::const_iterator ui = units.begin(); ui != units.end(); ++ui) {
-				(*ui)->DoDamage(DamageArray(30), ZeroVector, NULL, -CSolidObject::DAMAGE_EXTSOURCE_FIRE, -1);
+			const DamageArray fireDmg(30);
+			for (CUnit* u: units) {
+				u->DoDamage(fireDmg, ZeroVector, NULL, -CSolidObject::DAMAGE_EXTSOURCE_FIRE, -1);
 			}
 		}
 	}
@@ -137,9 +139,7 @@ void CFireProjectile::Update()
 		pi->posDif*=0.9f;
 	}
 
-	if (subParticles.empty() && (ttl <= 0)) {
-		deleteMe = true;
-	}
+	deleteMe |= ttl <= -particleTime;
 }
 
 void CFireProjectile::Draw()
@@ -158,8 +158,8 @@ void CFireProjectile::Draw()
 
 		float sinRot = fastmath::sin(rot);
 		float cosRot = fastmath::cos(rot);
-		float3 dir1 = (camera->right*cosRot + camera->up*sinRot) * size;
-		float3 dir2 = (camera->right*sinRot - camera->up*cosRot) * size;
+		float3 dir1 = (camera->GetRight()*cosRot + camera->GetUp()*sinRot) * size;
+		float3 dir2 = (camera->GetRight()*sinRot - camera->GetUp()*cosRot) * size;
 
 		float3 interPos=pi->pos;
 
@@ -180,8 +180,8 @@ void CFireProjectile::Draw()
 
 		float sinRot = fastmath::sin(rot);
 		float cosRot = fastmath::cos(rot);
-		float3 dir1 = (camera->right*cosRot + camera->up*sinRot) * size;
-		float3 dir2 = (camera->right*sinRot - camera->up*cosRot) * size;
+		float3 dir1 = (camera->GetRight()*cosRot + camera->GetUp()*sinRot) * size;
+		float3 dir2 = (camera->GetRight()*sinRot - camera->GetUp()*cosRot) * size;
 
 		float3 interPos = pi->pos;
 
@@ -215,3 +215,9 @@ void CFireProjectile::Draw()
 	}
 }
 
+
+int CFireProjectile::GetProjectilesCount() const
+{
+	return subParticles2.size() + subParticles.size() * 2;
+}
+
