@@ -14,9 +14,12 @@ public:
 	CFlareProjectile(const float3& pos, const float3& speed, CUnit* owner, int activateFrame);
 	~CFlareProjectile();
 
-	void Update();
-	void Draw();
+	void Update() override;
+	void Draw() override;
 
+	virtual int GetProjectilesCount() const override;
+
+public:
 	int activateFrame;
 	int deathFrame;
 
