@@ -44,7 +44,6 @@ CR_REG_METADATA(CPieceProjectile,(
 	CR_MEMBER(spinAngle),
 	CR_MEMBER(oldSmokePos),
 	CR_MEMBER(oldSmokeDir),
-	CR_MEMBER(alphaThreshold),
 	// CR_MEMBER(target),
 	CR_MEMBER(drawTrail),
 
@@ -70,7 +69,6 @@ CPieceProjectile::CPieceProjectile(
 
 	spinSpeed(0.0f),
 	spinAngle(0.0f),
-	alphaThreshold(0.1f),
 
 	oldSmokePos(pos),
 	oldSmokeDir(FwdVector),
@@ -85,7 +83,6 @@ CPieceProjectile::CPieceProjectile(
 		}
 
 		model = owner->model;
-		alphaThreshold = owner->alphaThreshold;
 		explFlags |= (PF_NoCEGTrail * (cegID == -1u));
 	}
 
@@ -135,22 +132,17 @@ CPieceProjectile::CPieceProjectile(
 #endif
 
 	projectileHandler->AddProjectile(this);
+	assert(!detached);
 }
 
-void CPieceProjectile::Detach()
+
+CPieceProjectile::~CPieceProjectile()
 {
-	// SYNCED
 	if (curCallback) {
 		// this is unsynced, but it prevents some callback crash on exit
 		curCallback->drawCallbacker = NULL;
 	}
 
-	CProjectile::Detach();
-}
-
-CPieceProjectile::~CPieceProjectile()
-{
-	// UNSYNCED
 	for (unsigned int a = 0; a < NUM_TRAIL_PARTS; ++a) {
 		delete fireTrailPoints[a];
 	}
@@ -434,10 +426,10 @@ void CPieceProjectile::Draw()
 				const float3 pos1 = CalcBeizer(float(a) / (numParts), pos, dirpos1, dirpos2, oldSmokePos);
 
 				#define st projectileDrawer->smoketex[0]
-				va->AddVertexQTC(pos1 + ( camera->up+camera->right) * size, st->xstart, st->ystart, col);
-				va->AddVertexQTC(pos1 + ( camera->up-camera->right) * size, st->xend,   st->ystart, col);
-				va->AddVertexQTC(pos1 + (-camera->up-camera->right) * size, st->xend,   st->ystart, col);
-				va->AddVertexQTC(pos1 + (-camera->up+camera->right) * size, st->xstart, st->ystart, col);
+				va->AddVertexQTC(pos1 + ( camera->GetUp()+camera->GetRight()) * size, st->xstart, st->ystart, col);
+				va->AddVertexQTC(pos1 + ( camera->GetUp()-camera->GetRight()) * size, st->xend,   st->ystart, col);
+				va->AddVertexQTC(pos1 + (-camera->GetUp()-camera->GetRight()) * size, st->xend,   st->ystart, col);
+				va->AddVertexQTC(pos1 + (-camera->GetUp()+camera->GetRight()) * size, st->xstart, st->ystart, col);
 				#undef st
 			}
 		}
@@ -475,11 +467,17 @@ void CPieceProjectile::DrawCallback()
 			col[3] = (unsigned char) (alpha * 50);
 
 			#define eft projectileDrawer->explofadetex
-			va->AddVertexQTC(interPos - camera->right * drawsize-camera->up * drawsize, eft->xstart, eft->ystart, col);
-			va->AddVertexQTC(interPos + camera->right * drawsize-camera->up * drawsize, eft->xend,   eft->ystart, col);
-			va->AddVertexQTC(interPos + camera->right * drawsize+camera->up * drawsize, eft->xend,   eft->yend,   col);
-			va->AddVertexQTC(interPos - camera->right * drawsize+camera->up * drawsize, eft->xstart, eft->yend,   col);
+			va->AddVertexQTC(interPos - camera->GetRight() * drawsize-camera->GetUp() * drawsize, eft->xstart, eft->ystart, col);
+			va->AddVertexQTC(interPos + camera->GetRight() * drawsize-camera->GetUp() * drawsize, eft->xend,   eft->ystart, col);
+			va->AddVertexQTC(interPos + camera->GetRight() * drawsize+camera->GetUp() * drawsize, eft->xend,   eft->yend,   col);
+			va->AddVertexQTC(interPos - camera->GetRight() * drawsize+camera->GetUp() * drawsize, eft->xstart, eft->yend,   col);
 			#undef eft
 		}
 	}
 }
+
+
+int CPieceProjectile::GetProjectilesCount() const
+{
+	return NUM_TRAIL_PARTS;
+}
