@@ -6,6 +6,7 @@
 #include "Rendering/GL/VertexArray.h"
 #include "Sim/Projectiles/ProjectileHandler.h"
 #include "Sim/Misc/QuadField.h"
+#include "Sim/Misc/TeamHandler.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitHandler.h"
 #include "System/Matrix44f.h"
@@ -23,6 +24,7 @@ CR_REG_METADATA(CProjectile,
 	CR_MEMBER(checkCol),
 	CR_MEMBER(ignoreWater),
 	CR_MEMBER(deleteMe),
+	CR_MEMBER(callEvent),
 
 	CR_MEMBER(castShadow),
 	CR_MEMBER(drawSorted),
@@ -37,16 +39,15 @@ CR_REG_METADATA(CProjectile,
 
 	CR_MEMBER(ownerID),
 	CR_MEMBER(teamID),
+	CR_MEMBER(allyteamID),
 	CR_MEMBER(cegID),
 
 	CR_MEMBER(projectileType),
 	CR_MEMBER(collisionFlags),
 
-	CR_MEMBER(qfCellData)
+	CR_MEMBER(quads)
 ))
 
-CR_BIND(CProjectile::QuadFieldCellData, )
-
 
 
 //////////////////////////////////////////////////////////////////////
@@ -66,14 +67,16 @@ CProjectile::CProjectile()
 	, checkCol(true)
 	, ignoreWater(false)
 	, deleteMe(false)
-
+	, callEvent(true)
 	, castShadow(false)
 	, drawSorted(true)
 
 	, mygravity(mapInfo? mapInfo->map.gravity: 0.0f)
+	, sortDist(0.0f)
 
 	, ownerID(-1u)
 	, teamID(-1u)
+	, allyteamID(-1)
 	, cegID(-1u)
 
 	, projectileType(-1u)
@@ -100,7 +103,7 @@ CProjectile::CProjectile(
 	, checkCol(true)
 	, ignoreWater(false)
 	, deleteMe(false)
-
+	, callEvent(true)
 	, castShadow(false)
 	, drawSorted(true)
 
@@ -109,6 +112,7 @@ CProjectile::CProjectile(
 
 	, ownerID(-1u)
 	, teamID(-1u)
+	, allyteamID(-1)
 	, cegID(-1u)
 
 	, projectileType(-1u)
@@ -118,17 +122,15 @@ CProjectile::CProjectile(
 	Init(owner, ZeroVector);
 }
 
-void CProjectile::Detach() {
-	// SYNCED
+
+CProjectile::~CProjectile()
+{
 	if (synced) {
 		quadField->RemoveProjectile(this);
+#ifdef TRACE_SYNC
+		tracefile << "Projectile died id: " << id << ", pos: <" << pos.x << ", " << pos.y << ", " << pos.z << ">\n";
+#endif	
 	}
-	CExpGenSpawnable::Detach();
-}
-
-CProjectile::~CProjectile() {
-	// UNSYNCED
-	assert(!synced || detached);
 }
 
 void CProjectile::Init(const CUnit* owner, const float3& offset)
@@ -137,6 +139,7 @@ void CProjectile::Init(const CUnit* owner, const float3& offset)
 		// must be set before the AddProjectile call
 		ownerID = owner->id;
 		teamID = owner->team;
+		allyteamID =  teamHandler->IsValidTeam(teamID)? teamHandler->AllyTeam(teamID): -1;
 	}
 	if (!hitscan) {
 		SetPosition(pos + offset);
@@ -151,7 +154,7 @@ void CProjectile::Init(const CUnit* owner, const float3& offset)
 		projectileHandler->AddProjectile(this);
 	}
 	if (synced && !weapon) {
-		quadField->AddProjectile(this);
+		quadField->AddProjectile(this);	
 	}
 }
 
@@ -188,20 +191,12 @@ void CProjectile::DrawOnMinimap(CVertexArray& lines, CVertexArray& points)
 	points.AddVertexQC(pos, color4::whiteA);
 }
 
-int CProjectile::DrawArray()
+void CProjectile::DrawArray()
 {
 	va->DrawArrayTC(GL_QUADS);
-
-	// draw-index gets divided by 24 because each element is
-	// 12 + 4 + 4 + 4 = 24 bytes in size (pos + u + v + color)
-	// for each type of "projectile"
-	const int idx = (va->drawIndex() / 24);
-
 	va = GetVertexArray();
 	va->Initialize();
 	inArray = false;
-
-	return idx;
 }
 
 CUnit* CProjectile::owner() const {
