@@ -1,17 +1,9 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "ProjectileFunctors.h"
 #include "Projectile.h"
 #include "Unsynced/FlyingPiece.h"
 
-void ProjectileDetacher::Detach(CProjectile* p) {
-	p->Detach();
-}
-
-int ProjectileIndexer::Index(const CProjectile* p) {
-	return p->id;
-}
-
-
 
 bool ProjectileDistanceComparator::operator() (const CProjectile* arg1, const CProjectile* arg2) const {
 	if (arg1->GetSortDist() != arg2->GetSortDist()) // strict ordering required
