@@ -24,8 +24,10 @@ public:
 	);
 	virtual ~CDirtProjectile();
 
-	virtual void Draw();
-	virtual void Update();
+	virtual void Draw() override;
+	virtual void Update() override;
+
+	virtual int GetProjectilesCount() const override;
 
 private:
 	float alpha;
