@@ -20,9 +20,12 @@ public:
 	);
 	~CGenericParticleProjectile();
 
-	virtual void Draw();
-	virtual void Update();
+	virtual void Draw() override;
+	virtual void Update() override;
 
+	virtual int GetProjectilesCount() const override;
+
+public:
 	float3 gravity;
 
 	AtlasedTexture* texture;
