@@ -22,8 +22,10 @@ public:
 	);
 	virtual ~CHeatCloudProjectile();
 
-	virtual void Draw();
-	virtual void Update();
+	virtual void Draw() override;
+	virtual void Update() override;
+
+	virtual int GetProjectilesCount() const override;
 
 private:
 	float heat;
