@@ -9,13 +9,14 @@ class CMuzzleFlame : public CProjectile
 {
 	CR_DECLARE(CMuzzleFlame)
 
-	void creg_Serialize(creg::ISerializer& s);
 public:
 	CMuzzleFlame(const float3& pos, const float3& speed, const float3& dir, float size);
 	~CMuzzleFlame();
 
-	void Draw();
-	void Update();
+	void Draw() override;
+	void Update() override;
+
+	virtual int GetProjectilesCount() const override;
 
 private:
 	float size;
@@ -23,7 +24,6 @@ private:
 	int numFlame;
 	int numSmoke;
 
-//	float3* randSmokeDir;
 	std::vector<float3> randSmokeDir;
 };
 
