@@ -3,9 +3,11 @@
 #ifndef SHIELD_PROJECTILE_H
 #define SHIELD_PROJECTILE_H
 
+#include <list>
 #include "Sim/Projectiles/Projectile.h"
 #include "System/float3.h"
 #include "System/type2.h"
+#include "System/Color.h"
 
 class CUnit;
 class CPlasmaRepulser;
@@ -15,22 +17,21 @@ struct AtlasedTexture;
 class ShieldSegmentProjectile;
 
 
-class ShieldProjectile: public CProjectile {
+class ShieldProjectile: public CProjectile
+{
 	CR_DECLARE(ShieldProjectile)
 public:
 	ShieldProjectile(CPlasmaRepulser*);
-	~ShieldProjectile();
 
-	void Update();
+	void Update() override;
+	virtual int GetProjectilesCount() const override;
+
+public:
+	void PreDelete();
 	bool AllowDrawing();
-
-	void PreDelete() {
-		deleteMe = true;
-		shield = NULL;
-	}
-
-	inline CPlasmaRepulser* GetShield() const { return shield; }
+	CPlasmaRepulser* GetShield() const { return shield; }
 	const AtlasedTexture* GetShieldTexture() const { return shieldTexture; }
+	float3 GetShieldDrawPos() const;
 
 private:
 	CPlasmaRepulser* shield;
@@ -40,12 +41,13 @@ private:
 	bool allowDrawing;
 
 	// NOTE: these are also registered in ProjectileHandler
-	std::list<ShieldSegmentProjectile*> shieldSegments;
+	std::list<ShieldSegmentProjectile*> shieldSegments; //FIXME deque
 };
 
 
 
-class ShieldSegmentProjectile: public CProjectile {
+class ShieldSegmentProjectile: public CProjectile
+{
 	CR_DECLARE(ShieldSegmentProjectile)
 public:
 	ShieldSegmentProjectile(
@@ -57,29 +59,25 @@ public:
 	);
 	~ShieldSegmentProjectile();
 
-	void Draw();
-	void Update();
-	void PreDelete() {
-		deleteMe = true;
-		shieldProjectile = NULL;
-	}
+	void Draw() override;
+	void Update() override;
+	void PreDelete();
+
+	virtual int GetProjectilesCount() const override;
 
 private:
+	bool UsingPerlinNoise() const;
 	static const float3* GetSegmentVertices(const int xpart, const int ypart);
 	static const float2* GetSegmentTexCoords(const AtlasedTexture* texture, const int xpart, const int ypart);
 
 private:
 	ShieldProjectile* shieldProjectile;
-
-	float3 segmentPos;
-	float3 segmentColor;
-
 	const float3* vertices;
 	const float2* texCoors;
 
+	SColor segmentColor;
+	float3 segmentPos;
 	float segmentSize;
-	float segmentAlpha;
-	bool usePerlinTex;
 };
 
 #endif
