@@ -24,9 +24,11 @@ public:
 		float color
 	);
 
-	void Update();
-	void Draw();
-	void Init(const CUnit* owner, const float3& offset);
+	void Update() override;
+	void Draw() override;
+	void Init(const CUnit* owner, const float3& offset) override;
+
+	virtual int GetProjectilesCount() const override;
 
 private:
 	float color;
