@@ -19,17 +19,19 @@ public:
 		const float3& dir2,
 		bool firstSegment,
 		bool lastSegment,
-		float size = 1,
-		int time = 80,
-		float color = 0.7f,
-		bool drawTrail = true,
-		CProjectile* drawCallback = 0,
-		AtlasedTexture* texture = 0
+		float size,
+		int time,
+		float color,
+		bool drawTrail,
+		CProjectile* drawCallback,
+		AtlasedTexture* texture
 	);
 	virtual ~CSmokeTrailProjectile();
 
-	void Update();
-	void Draw();
+	void Update() override;
+	void Draw() override;
+
+	virtual int GetProjectilesCount() const override;
 
 private:
 	float3 pos1;
