@@ -20,8 +20,10 @@ public:
 		float fadeupTime
 	);
 
-	void Update();
-	void Draw();
+	void Update() override;
+	void Draw() override;
+
+	virtual int GetProjectilesCount() const override;
 
 private:
 	float alpha;
