@@ -7,13 +7,15 @@
 
 class CBeamLaserProjectile: public CWeaponProjectile
 {
-	CR_DECLARE(CBeamLaserProjectile);
+	CR_DECLARE(CBeamLaserProjectile)
 public:
 	CBeamLaserProjectile(const ProjectileParams& params);
 
-	void Update();
-	void Draw();
-	void DrawOnMinimap(CVertexArray& lines, CVertexArray& points);
+	void Update() override;
+	void Draw() override;
+	void DrawOnMinimap(CVertexArray& lines, CVertexArray& points) override;
+
+	virtual int GetProjectilesCount() const override;
 
 private:
 	unsigned char coreColStart[4];
