@@ -11,11 +11,13 @@ class CEmgProjectile : public CWeaponProjectile
 public:
 	CEmgProjectile(const ProjectileParams& params);
 
-	void Update();
-	void Draw();
+	void Update() override;
+	void Draw() override;
+
+	virtual int GetProjectilesCount() const override;
 
 	int ShieldRepulse(CPlasmaRepulser* shield, float3 shieldPos,
-			float shieldForce, float shieldMaxSpeed);
+			float shieldForce, float shieldMaxSpeed) override;
 
 private:
 	float intensity;
