@@ -7,16 +7,18 @@
 
 class CFlameProjectile : public CWeaponProjectile
 {
-	CR_DECLARE(CFlameProjectile);
+	CR_DECLARE(CFlameProjectile)
 public:
 	CFlameProjectile(const ProjectileParams& params);
 
-	void Update();
-	void Draw();
-	void Collision();
+	void Update() override;
+	void Draw() override;
+	void Collision() override;
+
+	virtual int GetProjectilesCount() const override;
 
 	int ShieldRepulse(CPlasmaRepulser* shield, float3 shieldPos,
-			float shieldForce, float shieldMaxSpeed);
+			float shieldForce, float shieldMaxSpeed) override;
 
 private:
 	float curTime;
