@@ -12,9 +12,11 @@ class CLargeBeamLaserProjectile : public CWeaponProjectile
 public:
 	CLargeBeamLaserProjectile(const ProjectileParams& params);
 
-	void Update();
-	void Draw();
-	void DrawOnMinimap(CVertexArray& lines, CVertexArray& points);
+	void Update() override;
+	void Draw() override;
+	void DrawOnMinimap(CVertexArray& lines, CVertexArray& points) override;
+
+	virtual int GetProjectilesCount() const override;
 
 private:
 	unsigned char coreColStart[4];
