@@ -2,6 +2,7 @@
 
 #define LUA_SYNCED_ONLY
 
+#include <sstream>
 #include "LuaUnitScript.h"
 
 #include "CobInstance.h"
@@ -217,21 +218,22 @@ CLuaUnitScript::~CLuaUnitScript()
 
 void CLuaUnitScript::HandleFreed(CLuaHandle* handle)
 {
-	std::list<CUnit*>::iterator ui;
-	for (ui = unitHandler->activeUnits.begin(); ui != unitHandler->activeUnits.end(); ++ui) {
+	for (auto ui = unitHandler->activeUnits.begin(); ui != unitHandler->activeUnits.end(); ++ui) {
 		CLuaUnitScript* script = dynamic_cast<CLuaUnitScript*>((*ui)->script);
 
 		// kill only the Lua scripts running in this handle
-		if (script != NULL && script->handle == handle) {
+		if (script == NULL)
+			continue;
+		if (script->handle != handle)
+			continue;
 
-			// we don't have anything better ...
-			(*ui)->script = &CNullUnitScript::value;
+		// we don't have anything better ...
+		(*ui)->script = &CNullUnitScript::value;
 
-			// signal the destructor it shouldn't unref refs
-			script->L = NULL;
+		// signal the destructor it shouldn't unref refs
+		script->L = NULL;
 
-			delete script;
-		}
+		delete script;
 	}
 }
 
@@ -910,6 +912,12 @@ static void PushEntry(lua_State* L, const char* name, lua_CFunction fun)
 
 bool CLuaUnitScript::PushEntries(lua_State* L)
 {
+	{
+		// reset these in case we were reloaded
+		activeUnit = NULL;
+		activeScript = NULL;
+	}
+
 	lua_pushstring(L, "UnitScript");
 	lua_newtable(L);
 
@@ -1036,8 +1044,7 @@ int CLuaUnitScript::CreateScript(lua_State* L)
 	unit->script = newScript;
 
 	// flush some caches (which store availability of certain script functions)
-	for (std::vector<CWeapon*>::iterator wi = unit->weapons.begin(); wi != unit->weapons.end(); ++wi) {
-		CWeapon* w = *wi;
+	for (CWeapon* w: unit->weapons) {
 		w->SetWeaponNum(w->weaponNum);
 	}
 
@@ -1140,9 +1147,9 @@ int CLuaUnitScript::GetUnitValue(lua_State* L, CUnitScript* script, int arg)
 int CLuaUnitScript::GetUnitCOBValue(lua_State* L)
 {
 	CUnit* unit = ParseUnit(L, __FUNCTION__, 1);
-	if (unit == NULL) {
+
+	if (unit == NULL)
 		return 0;
-	}
 
 	return GetUnitValue(L, unit->script, 2);
 }
@@ -1150,9 +1157,8 @@ int CLuaUnitScript::GetUnitCOBValue(lua_State* L)
 
 int CLuaUnitScript::GetUnitValue(lua_State* L)
 {
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
 
 	return GetUnitValue(L, activeScript, 1);
 }
@@ -1181,9 +1187,9 @@ int CLuaUnitScript::SetUnitValue(lua_State* L, CUnitScript* script, int arg)
 int CLuaUnitScript::SetUnitCOBValue(lua_State* L)
 {
 	CUnit* unit = ParseUnit(L, __FUNCTION__, 1);
-	if (unit == NULL) {
+
+	if (unit == NULL)
 		return 0;
-	}
 
 	return SetUnitValue(L, unit->script, 2);
 }
@@ -1191,9 +1197,8 @@ int CLuaUnitScript::SetUnitCOBValue(lua_State* L)
 
 int CLuaUnitScript::SetUnitValue(lua_State* L)
 {
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
 
 	return SetUnitValue(L, activeScript, 1);
 }
@@ -1202,9 +1207,8 @@ int CLuaUnitScript::SetUnitValue(lua_State* L)
 int CLuaUnitScript::SetPieceVisibility(lua_State* L)
 {
 	// void SetVisibility(int piece, bool visible);
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
 
 	// note: for Lua unit scripts it would be confusing if the unit's
 	// unit->script->pieces differs from the unit->localModel->pieces.
@@ -1219,9 +1223,8 @@ int CLuaUnitScript::SetPieceVisibility(lua_State* L)
 int CLuaUnitScript::EmitSfx(lua_State* L)
 {
 	// void EmitSfx(int type, int piece);
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
 
 	// note: the arguments are reversed compared to the C++ (and COB?) function
 	const int piece = luaL_checkint(L, 1) - 1;
@@ -1235,14 +1238,14 @@ int CLuaUnitScript::EmitSfx(lua_State* L)
 int CLuaUnitScript::AttachUnit(lua_State* L)
 {
 	// void AttachUnit(int piece, int unit);
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
+
 	const int piece = luaL_checkint(L, 1) - 1;
 	const CUnit* transportee = ParseUnit(L, __FUNCTION__, 2);
-	if (transportee == NULL) {
+	if (transportee == NULL)
 		return 0;
-	}
+
 	activeScript->AttachUnit(piece, transportee->id);
 	return 0;
 }
@@ -1251,13 +1254,13 @@ int CLuaUnitScript::AttachUnit(lua_State* L)
 int CLuaUnitScript::DropUnit(lua_State* L)
 {
 	// void DropUnit(int unit);
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
+
 	const CUnit* transportee = ParseUnit(L, __FUNCTION__, 1);
-	if (transportee == NULL) {
+	if (transportee == NULL)
 		return 0;
-	}
+
 	activeScript->DropUnit(transportee->id);
 	return 0;
 }
@@ -1266,9 +1269,9 @@ int CLuaUnitScript::DropUnit(lua_State* L)
 int CLuaUnitScript::Explode(lua_State* L)
 {
 	// void Explode(int piece, int flags);
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
+
 	const int piece = luaL_checkint(L, 1) - 1;
 	const int flags = luaL_checkint(L, 2);
 	activeScript->Explode(piece, flags);
@@ -1279,9 +1282,9 @@ int CLuaUnitScript::Explode(lua_State* L)
 int CLuaUnitScript::ShowFlare(lua_State* L)
 {
 	// void ShowFlare(int piece);
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
+
 	const int piece = luaL_checkint(L, 1) - 1;
 	activeScript->ShowFlare(piece);
 	return 0;
@@ -1291,9 +1294,9 @@ int CLuaUnitScript::ShowFlare(lua_State* L)
 int CLuaUnitScript::Spin(lua_State* L)
 {
 	// void Spin(int piece, int axis, int speed, int accel);
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
+
 	const int piece = luaL_checkint(L, 1) - 1;
 	const int axis = ParseAxis(L, __FUNCTION__, 2);
 	const float speed = luaL_checkfloat(L, 3);
@@ -1307,9 +1310,9 @@ int CLuaUnitScript::Spin(lua_State* L)
 int CLuaUnitScript::StopSpin(lua_State* L)
 {
 	// void StopSpin(int piece, int axis, int decel);
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
+
 	const int piece = luaL_checkint(L, 1) - 1;
 	const int axis = ParseAxis(L, __FUNCTION__, 2);
 	const float decel = luaL_optfloat(L, 3, 0.0f); // decel == 0 -> stop immediately
@@ -1323,9 +1326,9 @@ int CLuaUnitScript::Turn(lua_State* L)
 {
 	// void Turn(int piece, int axis, int speed, int destination);
 	// void TurnNow(int piece, int axis, int destination);
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
+
 	const int piece = luaL_checkint(L, 1) - 1;
 	const int axis = ParseAxis(L, __FUNCTION__, 2);
 	const float dest  = luaL_checkfloat(L, 3);
@@ -1357,9 +1360,9 @@ int CLuaUnitScript::Move(lua_State* L)
 {
 	// void Move(int piece, int axis, int speed, int destination);
 	// void MoveNow(int piece, int axis, int destination);
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
+
 	const int piece = luaL_checkint(L, 1) - 1;
 	const int axis = ParseAxis(L, __FUNCTION__, 2);
 	const float dest  = luaL_checkfloat(L, 3);
@@ -1389,9 +1392,9 @@ int CLuaUnitScript::Move(lua_State* L)
 
 int CLuaUnitScript::IsInAnimation(lua_State* L, const char* caller, AnimType type)
 {
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
+
 	const int piece = luaL_checkint(L, 1) - 1;
 	const int axis  = ParseAxis(L, caller, 2);
 
@@ -1420,13 +1423,15 @@ int CLuaUnitScript::IsInSpin(lua_State* L)
 
 int CLuaUnitScript::WaitForAnimation(lua_State* L, const char* caller, AnimType type)
 {
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
+
 	CLuaUnitScript* script = dynamic_cast<CLuaUnitScript*>(activeScript);
+
 	if (script == NULL) {
 		luaL_error(L, "%s(): not a Lua unit script", caller);
 	}
+
 	const int piece = luaL_checkint(L, 1) - 1;
 	const int axis  = ParseAxis(L, caller, 2);
 
@@ -1449,13 +1454,15 @@ int CLuaUnitScript::WaitForMove(lua_State* L)
 
 int CLuaUnitScript::SetDeathScriptFinished(lua_State* L)
 {
-	if (activeUnit == NULL || activeScript == NULL) {
+	if (activeUnit == NULL || activeScript == NULL)
 		return 0;
-	}
+
 	CLuaUnitScript* script = dynamic_cast<CLuaUnitScript*>(activeScript);
+
 	if (script == NULL || !script->inKilled) {
 		luaL_error(L, "%s(): not a Lua unit script or 'Killed' not called", __FUNCTION__);
 	}
+
 	activeUnit->deathScriptFinished = true;
 	activeUnit->delayedWreckLevel = luaL_optint(L, 1, -1);
 	return 0;
@@ -1465,9 +1472,9 @@ int CLuaUnitScript::SetDeathScriptFinished(lua_State* L)
 
 int CLuaUnitScript::GetPieceTranslation(lua_State* L)
 {
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
+
 	LocalModelPiece* piece = ParseLocalModelPiece(L, activeScript, __FUNCTION__);
 	return ToLua(L, piece->GetPosition() - piece->original->offset);
 }
@@ -1475,9 +1482,9 @@ int CLuaUnitScript::GetPieceTranslation(lua_State* L)
 
 int CLuaUnitScript::GetPieceRotation(lua_State* L)
 {
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
+
 	LocalModelPiece* piece = ParseLocalModelPiece(L, activeScript, __FUNCTION__);
 	return ToLua(L, piece->GetRotation());
 }
@@ -1485,14 +1492,16 @@ int CLuaUnitScript::GetPieceRotation(lua_State* L)
 
 int CLuaUnitScript::GetPiecePosDir(lua_State* L)
 {
-	if (activeScript == NULL) {
+	if (activeScript == NULL)
 		return 0;
-	}
+
 	LocalModelPiece* piece = ParseLocalModelPiece(L, activeScript, __FUNCTION__);
 	float3 pos, dir;
+
 	if (!piece->GetEmitDirPos(pos, dir)) {
 		return 0;
 	}
+
 	ToLua(L, pos); ToLua(L, dir);
 	return 6;
 }
@@ -1502,8 +1511,10 @@ int CLuaUnitScript::GetPiecePosDir(lua_State* L)
 
 int CLuaUnitScript::GetActiveUnitID(lua_State* L)
 {
-	if (activeScript == NULL) { return 0; }
-	if (activeUnit == NULL) { return 0; }
+	if (activeScript == NULL)
+		return 0;
+	if (activeUnit == NULL)
+		return 0;
 
 	lua_pushnumber(L, activeUnit->id);
 	return 1;
