@@ -3,7 +3,6 @@
 #ifndef UNIT_H
 #define UNIT_H
 
-#include <map>
 #include <list>
 #include <vector>
 #include <string>
@@ -11,9 +10,12 @@
 #include "Lua/LuaRulesParams.h"
 #include "Lua/LuaUnitMaterial.h"
 #include "Sim/Objects/SolidObject.h"
+#include "Sim/Misc/Resource.h"
+#include "Sim/Weapons/WeaponTarget.h"
 #include "System/Matrix44f.h"
 #include "System/type2.h"
 
+
 class CPlayer;
 class CCommandAI;
 class CGroup;
@@ -64,6 +66,9 @@ enum ScriptCloakBits { // FIXME -- not implemented
 };
 
 
+
+
+
 class CUnit : public CSolidObject
 {
 public:
@@ -72,6 +77,8 @@ public:
 	CUnit();
 	virtual ~CUnit();
 
+	static void InitStatic();
+
 	virtual void PreInit(const UnitLoadParams& params);
 	virtual void PostInit(const CUnit* builder);
 
@@ -87,6 +94,7 @@ public:
 
 	bool AttackUnit(CUnit* unit, bool isUserTarget, bool wantManualFire, bool fpsMode = false);
 	bool AttackGround(const float3& pos, bool isUserTarget, bool wantManualFire, bool fpsMode = false);
+	void DropCurrentAttackTarget();
 
 	int GetBlockingMapID() const { return id; }
 
@@ -122,19 +130,25 @@ public:
 
 	void DependentDied(CObject* o);
 
-	bool SetGroup(CGroup* group, bool fromFactory = false);
-
 	bool AllowedReclaim(CUnit* builder) const;
+
+	void SetMetalStorage(float newStorage);
+	void SetEnergyStorage(float newStorage);
+
 	bool UseMetal(float metal);
 	void AddMetal(float metal, bool useIncomeMultiplier = true);
 	bool UseEnergy(float energy);
 	void AddEnergy(float energy, bool useIncomeMultiplier = true);
 	bool AddHarvestedMetal(float metal);
 
+	void SetStorage(const SResourcePack& newstorage);
+	bool HaveResources(const SResourcePack& res) const;
+	bool UseResources(const SResourcePack& res);
+	void AddResources(const SResourcePack& res, bool useIncomeMultiplier = true);
+	bool IssueResourceOrder(SResourceOrder* order);
+
 	/// push the new wind to the script
 	void UpdateWind(float x, float z, float strength);
-	void SetMetalStorage(float newStorage);
-	void SetEnergyStorage(float newStorage);
 
 	void AddExperience(float exp);
 
@@ -156,6 +170,8 @@ public:
 	bool IsCloaked() const { return isCloaked; }
 	bool IsIdle() const;
 
+	bool CanUpdateWeapons() const;
+
 	void SetStunned(bool stun);
 	bool IsStunned() const { return stunned; }
 
@@ -184,6 +200,7 @@ public:
 
 	void TempHoldFire(int cmdID);
 	void ReleaseTempHoldFire() { dontFire = false; }
+	bool HaveTarget() const;
 
 	/// start this unit in free fall from parent unit
 	void Drop(const float3& parentPos, const float3& parentDir, CUnit* parent);
@@ -195,6 +212,9 @@ protected:
 	void UpdateLosStatus(int allyTeam);
 	float GetFlankingDamageBonus(const float3& attackDir);
 
+public: // unsynced methods
+	bool SetGroup(CGroup* newGroup, bool fromFactory = false);
+
 public:
 	static void  SetExpMultiplier(float value) { expMultiplier = value; }
 	static void  SetExpPowerScale(float value) { expPowerScale = value; }
@@ -219,11 +239,29 @@ public:
 public:
 	const UnitDef* unitDef;
 
+	std::vector<CWeapon*> weapons;
+
+	/// Our shield weapon, NULL if we have none
+	CWeapon* shieldWeapon;
+	/// Our weapon with stockpiled ammo, NULL if we have none
+	CWeapon* stockpileWeapon;
+
 	CUnit* soloBuilder;
 	/// last attacker
 	CUnit* lastAttacker;
+
+	/// piece that was last hit by a projectile
+	LocalModelPiece* lastAttackedPiece;
+
+	/// frame in which lastAttackedPiece was hit
+	int lastAttackedPieceFrame;
+	/// last frame unit was attacked by other unit
+	int lastAttackFrame;
+	/// last time this unit fired a weapon
+	int lastFireWeapon;
+
 	/// current attackee
-	CUnit* attackTarget;
+	SWeaponTarget curTarget;
 
 	/// transport that the unit is currently in
 	CTransportUnit* transporter;
@@ -232,45 +270,25 @@ public:
 	AMoveType* prevMoveType;
 
 	CCommandAI* commandAI;
-	/// if the unit is part of an group (hotkey group)
-	CGroup* group;
-
-	/// Our shield weapon, NULL if we have none
-	CWeapon* shieldWeapon;
-	/// Our weapon with stockpiled ammo, NULL if we have none
-	CWeapon* stockpileWeapon;
 
 	LocalModel* localModel;
 	CUnitScript* script;
 
-	/// piece that was last hit by a projectile
-	LocalModelPiece* lastAttackedPiece;
-
 	/// which squares the unit can currently observe
 	LosInstance* los;
 
-	/// player who is currently FPS'ing this unit
-	CPlayer* fpsControlPlayer;
-
-	UnitTrackStruct* myTrack;
-	icon::CIconData* myIcon;
-
-
-	std::vector<CWeapon*> weapons;
-	/// quads the unit is part of
-	std::vector<int> quads;
-	std::vector<int> radarSquares;
-
 	/// indicate the los/radar status the allyteam has on this unit
 	std::vector<unsigned short> losStatus;
 
-	/// length-per-pixel (UNSYNCED)
-	std::vector<float> lodLengths;
+	/// player who is currently FPS'ing this unit
+	CPlayer* fpsControlPlayer;
+
+	/// quads the unit is part of
+	std::vector<int> quads;
+	std::vector<int> radarSquares;
 
 	std::list<CMissileProjectile*> incomingMissiles; //FIXME make std::set?
 
-
-	float3 attackPos;
 	float3 deathSpeed;
 	float3 lastMuzzleFlameDir;
 
@@ -281,7 +299,6 @@ public:
 	float3 posErrorVector;
 	float3 posErrorDelta;
 
-
 	int unitDefID;
 	int featureDefID; // FeatureDef id of the wreck we spawn on death
 
@@ -296,17 +313,12 @@ public:
 	LuaRulesParams::Params  modParams;
 	LuaRulesParams::HashMap modParamsMap; ///< name map for mod parameters
 
-	/// if the updir is straight up or align to the ground vector
-	bool upright;
-
-	/// total distance the unit has moved
-	float travel;
-	/// 0.0f disables travel accumulation
-	float travelPeriod;
-
 	/// indicate the relative power of the unit, used for experience calulations etc
 	float power;
 
+	/// 0.0-1.0
+	float buildProgress;
+
 	float maxHealth;
 	/// if health-this is negative the unit is stunned
 	float paralyzeDamage;
@@ -322,7 +334,14 @@ public:
 	 * FIRESTATE_FIREATWILL
 	 */
 	bool neutral;
+	/// is in built (:= nanoframe)
 	bool beingBuilt;
+	/// if the updir is straight up or align to the ground vector
+	bool upright;
+	/// whether the ground below this unit has been terraformed
+	bool groundLevelled;
+	/// how much terraforming is left to do
+	float terraformLeft;
 
 	/// if we arent built on for a while start decaying
 	int lastNanoAdd;
@@ -333,15 +352,7 @@ public:
 
 	/// id of transport that the unit is about to be picked up by
 	int loadingTransportId;
-	/// 0.0-1.0
-	float buildProgress;
-	/// whether the ground below this unit has been terraformed
-	bool groundLevelled;
-	/// how much terraforming is left to do
-	float terraformLeft;
-	/// set los to this when finished building
-	int realLosRadius;
-	int realAirLosRadius;
+
 
 	/// used by constructing units
 	bool inBuildStance;
@@ -365,10 +376,6 @@ public:
 	float reloadSpeed;
 	float maxRange;
 
-	/// true if at least one weapon has targetType != Target_None
-	bool haveTarget;
-	bool haveManualFireRequest;
-
 	/// used to determine muzzle flare size
 	float lastMuzzleFlameSize;
 
@@ -378,6 +385,10 @@ public:
 
 	int mapSquare;
 
+	/// set los to this when finished building
+	int realLosRadius;
+	int realAirLosRadius;
+
 	int losRadius;
 	int airLosRadius;
 	int lastLosUpdate;
@@ -391,68 +402,51 @@ public:
 	int sonarJamRadius;
 	int seismicRadius;
 	float seismicSignature;
-	bool hasRadarCapacity;
 	int2 oldRadarPos;
 	bool hasRadarPos;
 	bool stealth;
 	bool sonarStealth;
+	bool hasRadarCapacity;
 
-	// only when the unit is active
-	float condUseMetal;
-	float condUseEnergy;
-	float condMakeMetal;
-	float condMakeEnergy;
-	// always applied
-	float uncondUseMetal;
-	float uncondUseEnergy;
-	float uncondMakeMetal;
-	float uncondMakeEnergy;
+	/// only when the unit is active
+	SResourcePack resourcesCondUse;
+	SResourcePack resourcesCondMake;
 
-	/// cost per 16 frames
-	float metalUse;
-	/// cost per 16 frames
-	float energyUse;
-	/// metal income generated by unit
-	float metalMake;
-	/// energy income generated by unit
-	float energyMake;
+	/// always applied
+	SResourcePack resourcesUncondUse;
+	SResourcePack resourcesUncondMake;
+
+	/// costs per UNIT_SLOWUPDATE_RATE frames
+	SResourcePack resourcesUse;
+
+	/// incomes per UNIT_SLOWUPDATE_RATE frames
+	SResourcePack resourcesMake;
+
+	/// variables used for calculating unit resource usage
+	SResourcePack resourcesUseI;
+	SResourcePack resourcesMakeI;
+	SResourcePack resourcesUseOld;
+	SResourcePack resourcesMakeOld;
 
-	// variables used for calculating unit resource usage
-	float metalUseI;
-	float energyUseI;
-	float metalMakeI;
-	float energyMakeI;
-	float metalUseold;
-	float energyUseold;
-	float metalMakeold;
-	float energyMakeold;
 	/// energy added each halftick
-	float energyTickMake;
+	float energyTickMake; //FIXME???
 
 	/// how much metal the unit currently extracts from the ground
 	float metalExtract;
 
-	float metalCost;
-	float energyCost;
-	float buildTime;
+	/// the amount of storage the unit contributes to the team
+	SResourcePack storage;
 
-	float metalStorage;
-	float energyStorage;
 	/// per unit metal storage (gets filled on reclaim and needs then to be unloaded at some storage building -> 2nd part is lua's job)
-	float harvestStorage;
+	SResourcePack harvestStorage;
+	SResourcePack harvested;
 
-	/// frame in which lastAttackedPiece was hit
-	int lastAttackedPieceFrame;
-	/// last frame unit was attacked by other unit
-	int lastAttackFrame;
-	/// last time this unit fired a weapon
-	int lastFireWeapon;
+	SResourcePack cost;
+	float buildTime;
 
 	/// decaying value of how much damage the unit has taken recently (for severity of death)
 	float recentDamage;
 
-	bool userAttackGround;
-
 	int fireState;
 	int moveState;
 
@@ -464,6 +458,11 @@ public:
 	/// for units being dropped from transports (parachute drops)
 	float fallSpeed;
 
+	/// total distance the unit has moved
+	float travel;
+	/// 0.0f disables travel accumulation
+	float travelPeriod;
+
 	/**
 	 * 0 = no flanking bonus
 	 * 1 = global coords, mobile
@@ -485,8 +484,10 @@ public:
 	/// multiply all damage the unit take with this
 	float curArmorMultiple;
 
-	int	nextPosErrorUpdate;
+	int nextPosErrorUpdate;
 
+	///true if the unit is currently cloaked (has enough energy etc)
+	bool isCloaked;
 	/// true if the unit currently wants to be cloaked
 	bool wantCloak;
 	/// true if a script currently wants the unit to be cloaked
@@ -495,8 +496,6 @@ public:
 	int cloakTimeout;
 	/// the earliest frame the unit can cloak again
 	int curCloakTimeout;
-	///true if the unit is currently cloaked (has enough energy etc)
-	bool isCloaked;
 	float decloakDistance;
 
 	int lastTerrainType;
@@ -507,8 +506,6 @@ public:
 
 	float currentFuel;
 
-	/// minimum alpha value for a texel to be drawn
-	float alphaThreshold;
 	/// the damage value passed to CEGs spawned by this unit's script
 	int cegDamage;
 
@@ -522,27 +519,33 @@ public:
 	bool isIcon;
 	float iconRadius;
 
-	unsigned int lodCount;
-	unsigned int currentLOD;
-
-	LuaUnitMaterial luaMats[LUAMAT_TYPE_COUNT];
-
 	int lastDrawFrame;
 	unsigned int lastUnitUpdate;
 
 	std::string tooltip;
 
+	CGroup* group;
+
+	UnitTrackStruct* myTrack;
+	icon::CIconData* myIcon;
+
+	/// LOD length-per-pixel
+	unsigned int lodCount;
+	unsigned int currentLOD;
+	std::vector<float> lodLengths;
+	LuaUnitMaterial luaMats[LUAMAT_TYPE_COUNT];
+
 private:
 	/// if we are stunned by a weapon or for other reason, access via IsStunned/SetStunned(bool)
 	bool stunned;
 
+	static float empDeclineRate;
 	static float expMultiplier;
 	static float expPowerScale;
 	static float expHealthScale;
 	static float expReloadScale;
 	static float expGrade;
 
-	static float empDecline;
 	static bool spawnFeature;
 };
 
