@@ -54,7 +54,7 @@ CR_REG_METADATA(CBuilder, (
 	CR_MEMBER(tx1), CR_MEMBER(tx2), CR_MEMBER(tz1), CR_MEMBER(tz2),
 	CR_MEMBER(terraformCenter),
 	CR_MEMBER(terraformRadius),
-	CR_ENUM_MEMBER(terraformType),
+	CR_MEMBER(terraformType),
 	CR_MEMBER(nanoPieceCache),
 	CR_POSTLOAD(PostLoad)
 ))
@@ -184,7 +184,7 @@ void CBuilder::Update()
 
 						for (int z = tz1; z <= tz2; z++) {
 							for (int x = tx1; x <= tx2; x++) {
-								int idx = z * gs->mapxp1 + x;
+								int idx = z * mapDims.mapxp1 + x;
 								float ch = heightmap[idx];
 
 								readMap->AddHeight(idx, (curBuild->pos.y - ch) * terraformScale);
@@ -214,7 +214,7 @@ void CBuilder::Update()
 
 					for (int z = tz1; z <= tz2; z++) {
 						for (int x = tx1; x <= tx2; x++) {
-							int idx = z * gs->mapxp1 + x;
+							int idx = z * mapDims.mapxp1 + x;
 							float ch = heightmap[idx];
 							float oh = readMap->GetOriginalHeightMapSynced()[idx];
 
@@ -237,20 +237,20 @@ void CBuilder::Update()
 				// smooth the borders x
 				for (int x = 1; x <= 3; x++) {
 					if (tx1 - 3 >= 0) {
-						const float ch3 = heightmap[z * gs->mapxp1 + tx1    ];
-						const float ch  = heightmap[z * gs->mapxp1 + tx1 - x];
-						const float ch2 = heightmap[z * gs->mapxp1 + tx1 - 3];
+						const float ch3 = heightmap[z * mapDims.mapxp1 + tx1    ];
+						const float ch  = heightmap[z * mapDims.mapxp1 + tx1 - x];
+						const float ch2 = heightmap[z * mapDims.mapxp1 + tx1 - 3];
 						const float amount = ((ch3 * (3 - x) + ch2 * x) / 3 - ch) * terraformScale;
 
-						readMap->AddHeight(z * gs->mapxp1 + tx1 - x, amount);
+						readMap->AddHeight(z * mapDims.mapxp1 + tx1 - x, amount);
 					}
-					if (tx2 + 3 < gs->mapx) {
-						const float ch3 = heightmap[z * gs->mapxp1 + tx2    ];
-						const float ch  = heightmap[z * gs->mapxp1 + tx2 + x];
-						const float ch2 = heightmap[z * gs->mapxp1 + tx2 + 3];
+					if (tx2 + 3 < mapDims.mapx) {
+						const float ch3 = heightmap[z * mapDims.mapxp1 + tx2    ];
+						const float ch  = heightmap[z * mapDims.mapxp1 + tx2 + x];
+						const float ch2 = heightmap[z * mapDims.mapxp1 + tx2 + 3];
 						const float amount = ((ch3 * (3 - x) + ch2 * x) / 3 - ch) * terraformScale;
 
-						readMap->AddHeight(z * gs->mapxp1 + tx2 + x, amount);
+						readMap->AddHeight(z * mapDims.mapxp1 + tx2 + x, amount);
 					}
 				}
 			}
@@ -258,20 +258,20 @@ void CBuilder::Update()
 				// smooth the borders z
 				for (int x = tx1; x <= tx2; x++) {
 					if (tz1 - 3 >= 0) {
-						const float ch3 = heightmap[(tz1    ) * gs->mapxp1 + x];
-						const float ch  = heightmap[(tz1 - z) * gs->mapxp1 + x];
-						const float ch2 = heightmap[(tz1 - 3) * gs->mapxp1 + x];
+						const float ch3 = heightmap[(tz1    ) * mapDims.mapxp1 + x];
+						const float ch  = heightmap[(tz1 - z) * mapDims.mapxp1 + x];
+						const float ch2 = heightmap[(tz1 - 3) * mapDims.mapxp1 + x];
 						const float adjust = ((ch3 * (3 - z) + ch2 * z) / 3 - ch) * terraformScale;
 
-						readMap->AddHeight((tz1 - z) * gs->mapxp1 + x, adjust);
+						readMap->AddHeight((tz1 - z) * mapDims.mapxp1 + x, adjust);
 					}
-					if (tz2 + 3 < gs->mapy) {
-						const float ch3 = heightmap[(tz2    ) * gs->mapxp1 + x];
-						const float ch  = heightmap[(tz2 + z) * gs->mapxp1 + x];
-						const float ch2 = heightmap[(tz2 + 3) * gs->mapxp1 + x];
+					if (tz2 + 3 < mapDims.mapy) {
+						const float ch3 = heightmap[(tz2    ) * mapDims.mapxp1 + x];
+						const float ch  = heightmap[(tz2 + z) * mapDims.mapxp1 + x];
+						const float ch2 = heightmap[(tz2 + 3) * mapDims.mapxp1 + x];
 						const float adjust = ((ch3 * (3 - z) + ch2 * z) / 3 - ch) * terraformScale;
 
-						readMap->AddHeight((tz2 + z) * gs->mapxp1 + x, adjust);
+						readMap->AddHeight((tz2 + z) * mapDims.mapxp1 + x, adjust);
 					}
 				}
 			}
@@ -365,7 +365,7 @@ void CBuilder::Update()
 					if ((modInfo.reclaimMethod != 1) && (curResurrect->reclaimLeft < 1)) {
 						// This corpse has been reclaimed a little, need to restore
 						// the resources before we can let the player resurrect it.
-						curResurrect->AddBuildPower(this, repairSpeed);
+						curResurrect->AddBuildPower(this, resurrectSpeed);
 					} else {
 						// Corpse has been restored, begin resurrection
 						const float step = resurrectSpeed / ud->buildTime;
@@ -452,7 +452,7 @@ void CBuilder::Update()
 					const float captureProgressTemp = std::min(curCapture->captureProgress + captureProgressStep, 1.0f);
 
 					const float captureFraction = captureProgressTemp - curCapture->captureProgress;
-					const float energyUseScaled = curCapture->energyCost * captureFraction * modInfo.captureEnergyCostFactor;
+					const float energyUseScaled = curCapture->cost.energy * captureFraction * modInfo.captureEnergyCostFactor;
 
 					const bool captureAllowed = (eventHandler.AllowUnitBuildStep(this, curCapture, captureProgressStep));
 					const bool canExecCapture = (captureAllowed && UseEnergy(energyUseScaled));
@@ -510,9 +510,9 @@ void CBuilder::SetRepairTarget(CUnit* target)
 	if (!target->groundLevelled) {
 		//resume levelling the ground
 		tx1 = (int)max((float)0,(target->pos.x - (target->unitDef->xsize*0.5f*SQUARE_SIZE))/SQUARE_SIZE);
-		tx2 = min(gs->mapx,tx1+target->unitDef->xsize);
+		tx2 = min(mapDims.mapx,tx1+target->unitDef->xsize);
 		tz1 = (int)max((float)0,(target->pos.z - (target->unitDef->zsize*0.5f*SQUARE_SIZE))/SQUARE_SIZE);
-		tz2 = min(gs->mapy,tz1+target->unitDef->zsize);
+		tz2 = min(mapDims.mapy,tz1+target->unitDef->zsize);
 		terraformCenter = target->pos;
 		terraformRadius = (tx1 - tx2) * SQUARE_SIZE;
 		terraformType=Terraform_Building;
@@ -591,9 +591,9 @@ void CBuilder::StartRestore(float3 centerPos, float radius)
 	terraformRadius=radius;
 
 	tx1 = (int)max((float)0,(centerPos.x-radius)/SQUARE_SIZE);
-	tx2 = (int)min((float)gs->mapx,(centerPos.x+radius)/SQUARE_SIZE);
+	tx2 = (int)min((float)mapDims.mapx,(centerPos.x+radius)/SQUARE_SIZE);
 	tz1 = (int)max((float)0,(centerPos.z-radius)/SQUARE_SIZE);
-	tz2 = (int)min((float)gs->mapy,(centerPos.z+radius)/SQUARE_SIZE);
+	tz2 = (int)min((float)mapDims.mapy,(centerPos.z+radius)/SQUARE_SIZE);
 
 	float tcost = 0.0f;
 	const float* curHeightMap = readMap->GetCornerHeightMapSynced();
@@ -601,7 +601,7 @@ void CBuilder::StartRestore(float3 centerPos, float radius)
 
 	for (int z = tz1; z <= tz2; z++) {
 		for (int x = tx1; x <= tx2; x++) {
-			float delta = orgHeightMap[z * gs->mapxp1 + x] - curHeightMap[z * gs->mapxp1 + x];
+			float delta = orgHeightMap[z * mapDims.mapxp1 + x] - curHeightMap[z * mapDims.mapxp1 + x];
 			tcost += math::fabs(delta);
 		}
 	}
@@ -709,9 +709,9 @@ bool CBuilder::StartBuild(BuildInfo& buildInfo, CFeature*& feature, bool& waitSt
 		buildee->groundLevelled = true;
 	} else {
 		tx1 = (int)max(    0.0f, (buildee->pos.x - (buildeeDef->xsize * 0.5f * SQUARE_SIZE)) / SQUARE_SIZE);
-		tx2 =      min(gs->mapx,             tx1 +  buildeeDef->xsize                                     );
+		tx2 =      min(mapDims.mapx,             tx1 +  buildeeDef->xsize                                     );
 		tz1 = (int)max(    0.0f, (buildee->pos.z - (buildeeDef->zsize * 0.5f * SQUARE_SIZE)) / SQUARE_SIZE);
-		tz2 =      min(gs->mapy,             tz1 +  buildeeDef->zsize                                     );
+		tz2 =      min(mapDims.mapy,             tz1 +  buildeeDef->zsize                                     );
 
 		buildee->terraformLeft = CalculateBuildTerraformCost(buildInfo);
 		buildee->groundLevelled = false;
@@ -753,7 +753,7 @@ float CBuilder::CalculateBuildTerraformCost(BuildInfo& buildInfo)
 
 	for (int z = tz1; z <= tz2; z++) {
 		for (int x = tx1; x <= tx2; x++) {
-			const int idx = z * gs->mapxp1 + x;
+			const int idx = z * mapDims.mapxp1 + x;
 			float delta = buildPos.y - curHeightMap[idx];
 			float cost;
 			if (delta > 0) {
