@@ -11,7 +11,7 @@ struct SolidObjectGroundDecal;
 class CBuilding : public CUnit
 {
 public:
-	CR_DECLARE(CBuilding);
+	CR_DECLARE(CBuilding)
 
 	CBuilding();
 	virtual ~CBuilding() {}
