@@ -11,19 +11,15 @@ class CBombDropper: public CWeapon
 public:
 	CBombDropper(CUnit* owner, const WeaponDef* def, bool useTorps);
 
-	void Init();
-	void Update();
-	void SlowUpdate();
+	float GetPredictedImpactTime(float3 p) const override final;
 
 private:
-	bool CanFire(bool ignoreAngleGood, bool ignoreTargetType, bool ignoreRequestedDir) const;
+	bool CanFire(bool ignoreAngleGood, bool ignoreTargetType, bool ignoreRequestedDir) const override final;
 
-	bool TestTarget(const float3& pos, bool userTarget, const CUnit* unit) const;
-	bool TestRange(const float3& pos, bool userTarget, const CUnit* unit) const;
-	bool HaveFreeLineOfFire(const float3& pos, bool userTarget, const CUnit* unit) const;
-	void FireImpl(bool scriptCall);
-
-	float GetPredictedImpactTime(const float3& impactPos) const;
+	bool TestTarget(const float3 pos, const SWeaponTarget& trg) const override final;
+	bool TestRange(const float3 pos, const SWeaponTarget& trg) const override final;
+	bool HaveFreeLineOfFire(const float3 pos, const SWeaponTarget& trg) const override final;
+	void FireImpl(const bool scriptCall) override final;
 
 private:
 	/// if we should drop torpedoes
