@@ -11,10 +11,8 @@ class CEmgCannon: public CWeapon
 public:
 	CEmgCannon(CUnit* owner, const WeaponDef* def);
 
-	void Update();
-
 private:
-	void FireImpl(bool scriptCall);
+	void FireImpl(const bool scriptCall) override final;
 };
 
 #endif // _EMG_CANNON_H
