@@ -11,13 +11,11 @@ class CFlameThrower: public CWeapon
 public:
 	CFlameThrower(CUnit* owner, const WeaponDef* def);
 
-	void Update();
-
 	float3 color;
 	float3 color2;
 
 private:
-	void FireImpl(bool scriptCall);
+	void FireImpl(const bool scriptCall) override final;
 };
 
 #endif // _FLAME_THROWER_H
