@@ -27,23 +27,14 @@ CRifle::CRifle(CUnit* owner, const WeaponDef* def): CWeapon(owner, def)
 {
 }
 
-
-void CRifle::Update()
+float CRifle::GetPredictedImpactTime(float3 p) const
 {
-	if (targetType != Target_None) {
-		weaponPos = owner->GetObjectSpacePos(relWeaponPos);
-		weaponMuzzlePos = owner->GetObjectSpacePos(relWeaponMuzzlePos);
-
-		wantedDir = (targetPos - weaponPos).Normalize();
-	}
-
-	CWeapon::Update();
+	return 0;
 }
 
-
-void CRifle::FireImpl(bool scriptCall)
+void CRifle::FireImpl(const bool scriptCall)
 {
-	float3 dir = (targetPos - weaponMuzzlePos).Normalize();
+	float3 dir = (currentTargetPos - weaponMuzzlePos).SafeNormalize();
 	dir +=
 		(gs->randVector() * SprayAngleExperience() + SalvoErrorExperience());
 	dir.Normalize();
