@@ -24,9 +24,7 @@ public:
 	const WeaponDef* GetWeaponDef(std::string weaponname) const;
 	const WeaponDef* GetWeaponDefByID(int weaponDefId) const;
 
-	static DamageArray DynamicDamages(const DamageArray& damages, const float3 startPos,
-					const float3 curPos, const float range, const float exp,
-					const float damageMin, const bool inverted);
+	static DamageArray DynamicDamages(const WeaponDef* weaponDef, const float3 startPos, const float3 curPos);
 
 public:
 	std::vector<WeaponDef> weaponDefs;
