@@ -6,6 +6,7 @@
 #include "CregLoadSaveHandler.h"
 #include "LuaLoadSaveHandler.h"
 #include "System/Config/ConfigHandler.h"
+#include "System/FileSystem/FileSystem.h"
 
 
 CONFIG(bool, UseCREGSaveLoad).defaultValue(false);
@@ -25,15 +26,11 @@ ILoadSaveHandler::~ILoadSaveHandler()
 }
 
 
-std::string ILoadSaveHandler::FindSaveFile(const std::string& name)
+std::string ILoadSaveHandler::FindSaveFile(const std::string& file)
 {
-	std::string name2 = name;
-#ifdef _WIN32
-	if (name2.find(":\\")==std::string::npos)
-		name2 = "Saves\\" + name2;
-#else
-	if (name2.find("/")==std::string::npos)
-		name2 = "Saves/" + name2;
-#endif
-	return name2;
+	if (!FileSystem::IsAbsolutePath(file)) {
+		return FileSystem::EnsurePathSepAtEnd("Saves") + file;
+	}
+
+	return file;
 }
