@@ -10,6 +10,7 @@ const char* log_util_levelToString(int level) {
 	switch (level) {
 		case LOG_LEVEL_DEBUG:   return "Debug";
 		case LOG_LEVEL_INFO:    return "Info";
+		case LOG_LEVEL_NOTICE:  return "Notice";
 		case LOG_LEVEL_WARNING: return "Warning";
 		case LOG_LEVEL_ERROR:   return "Error";
 		case LOG_LEVEL_FATAL:   return "Fatal";
@@ -22,6 +23,7 @@ char log_util_levelToChar(int level) {
 	switch (level) {
 		case LOG_LEVEL_DEBUG:   return 'D';
 		case LOG_LEVEL_INFO:    return 'I';
+		case LOG_LEVEL_NOTICE:  return 'N';
 		case LOG_LEVEL_WARNING: return 'W';
 		case LOG_LEVEL_ERROR:   return 'E';
 		case LOG_LEVEL_FATAL:   return 'F';
@@ -29,6 +31,17 @@ char log_util_levelToChar(int level) {
 	}
 }
 
+int log_util_getNearestLevel(int level) {
+
+	if (level >= LOG_LEVEL_FATAL)   return LOG_LEVEL_FATAL;
+	if (level >= LOG_LEVEL_ERROR)   return LOG_LEVEL_ERROR;
+	if (level >= LOG_LEVEL_WARNING) return LOG_LEVEL_WARNING;
+	if (level >= LOG_LEVEL_NOTICE)  return LOG_LEVEL_NOTICE;
+	if (level >= LOG_LEVEL_INFO)    return LOG_LEVEL_INFO;
+	if (level >= LOG_LEVEL_DEBUG)   return LOG_LEVEL_DEBUG;
+	return DEFAULT_LOG_LEVEL;
+}
+
 const char* log_util_prepareSection(const char* section) {
 
 	// make sure we always have a string for printing
