@@ -20,7 +20,7 @@ CMatrix44f::CMatrix44f(const CMatrix44f& mat)
 }
 
 
-CMatrix44f::CMatrix44f(const float3& pos, const float3& x, const float3& y, const float3& z)
+CMatrix44f::CMatrix44f(const float3 pos, const float3 x, const float3 y, const float3 z)
 {
 	// column-major!
 	m[0] = x.x;   m[4] = y.x;   m[ 8] = z.x;   m[12] = pos.x;
@@ -37,7 +37,7 @@ CMatrix44f::CMatrix44f(const float rotX, const float rotY, const float rotZ)
 	RotateZ(rotZ);
 }
 
-CMatrix44f::CMatrix44f(const float3& p)
+CMatrix44f::CMatrix44f(const float3 p)
 {
 	LoadIdentity();
 	SetPos(p);
@@ -196,7 +196,7 @@ CMatrix44f& CMatrix44f::RotateZ(float rad)
 }
 
 
-CMatrix44f& CMatrix44f::Rotate(float rad, const float3& axis)
+CMatrix44f& CMatrix44f::Rotate(float rad, const float3 axis)
 {
 	const float sr = math::sin(rad);
 	const float cr = math::cos(rad);
@@ -258,7 +258,7 @@ CMatrix44f& CMatrix44f::Translate(const float x, const float y, const float z)
 }
 
 
-void CMatrix44f::SetPos(const float3& pos)
+void CMatrix44f::SetPos(const float3 pos)
 {
 	const float x = pos.x;
 	const float y = pos.y;
@@ -353,7 +353,7 @@ CMatrix44f& CMatrix44f::operator<<= (const CMatrix44f& m2)
 }
 
 __FORCE_ALIGN_STACK__
-float3 CMatrix44f::operator* (const float3& v) const
+float3 CMatrix44f::operator* (const float3 v) const
 {
 	//SCOPED_TIMER("CMatrix44f::Mul");
 
@@ -373,7 +373,7 @@ float3 CMatrix44f::operator* (const float3& v) const
 }
 
 __FORCE_ALIGN_STACK__
-float4 CMatrix44f::operator* (const float4& v) const
+float4 CMatrix44f::operator* (const float4 v) const
 {
 	__m128 out;
 	out =                 _mm_mul_ps(_mm_loadu_ps(&md[0][0]), _mm_load1_ps(&v.x));
@@ -386,7 +386,7 @@ float4 CMatrix44f::operator* (const float4& v) const
 }
 
 
-void CMatrix44f::SetUpVector(const float3& up)
+void CMatrix44f::SetUpVector(const float3 up)
 {
 	float3 zdir(m[8], m[9], m[10]);
 	float3 xdir(zdir.cross(up));
@@ -427,10 +427,9 @@ CMatrix44f& CMatrix44f::Transpose()
 CMatrix44f& CMatrix44f::InvertAffineInPlace()
 {
 	//! transpose the rotation
-	float tmp;
-	tmp = m[1]; m[1] = m[4]; m[4] = tmp;
-	tmp = m[2]; m[2] = m[8]; m[8] = tmp;
-	tmp = m[6]; m[6] = m[9]; m[9] = tmp;
+	std::swap(m[1], m[4]);
+	std::swap(m[2], m[8]);
+	std::swap(m[6], m[9]);
 
 	//! get the inverse translation
 	const float3 t(-m[12], -m[13], -m[14]);
@@ -460,14 +459,14 @@ static inline T CalculateCofactor(const T m[4][4], const int ei, const int ej)
 		case 0: { ai = 1; bi = 2; ci = 3; break; }
 		case 1: { ai = 0; bi = 2; ci = 3; break; }
 		case 2: { ai = 0; bi = 1; ci = 3; break; }
-		case 3: { ai = 0; bi = 1; ci = 2; break; }
+		default: { assert(ei < 4); ai = 0; bi = 1; ci = 2; break; }
 	}
 	size_t aj, bj, cj;
 	switch (ej) {
 		case 0: { aj = 1; bj = 2; cj = 3; break; }
 		case 1: { aj = 0; bj = 2; cj = 3; break; }
 		case 2: { aj = 0; bj = 1; cj = 3; break; }
-		case 3: { aj = 0; bj = 1; cj = 2; break; }
+		default: { assert(ej < 4); aj = 0; bj = 1; cj = 2; break; }
 	}
 
 	const T val =
@@ -507,8 +506,8 @@ bool CMatrix44f::InvertInPlace()
 	}
 
 	const float scale = 1.0f / det;
-	for (int i = 0; i < 4; i++) {
-		for (int j = 0; j < 4; j++) {
+	for (int j = 0; j < 4; j++) {
+		for (int i = 0; i < 4; i++) {
 			//! (adjoint / determinant)
 			//! (note the transposition in 'cofac')
 			md[i][j] = cofac[j][i] * scale;
