@@ -8,7 +8,7 @@
 #include "System/creg/creg_cond.h"
 
 struct SRectangle {
-	CR_DECLARE_STRUCT(SRectangle);
+	CR_DECLARE_STRUCT(SRectangle)
 
 	SRectangle()
 		: x1(0)
@@ -27,6 +27,13 @@ struct SRectangle {
 	int GetHeight() const { return z2 - z1; }
 	int GetArea() const { return (GetWidth() * GetHeight()); }
 
+	bool Inside(const int2 pos) const {
+		// note: *min inclusive, *max exclusive
+		const bool xb = (pos.x >= x1 && pos.x < x2);
+		const bool yb = (pos.y >= y1 && pos.y < y2);
+		return (xb && yb);
+	}
+
 	void ClampPos(int2* pos) const {
 		pos->x = Clamp(pos->x, x1, x2);
 		pos->y = Clamp(pos->y, y1, y2);
