@@ -13,7 +13,7 @@
 template<class T>
 class safe_vector : public std::vector<T>
 {
-	CR_DECLARE_STRUCT(safe_vector);
+	CR_DECLARE_STRUCT(safe_vector)
 
 public:
 	typedef typename std::vector<T>::size_type size_type;
