@@ -92,14 +92,14 @@ namespace creg
 				}
 			}
 		}
-		std::string GetName() { return "map<" + keyType->GetName() + ", " + mappedType->GetName() + ">"; }
-		size_t GetSize() { return sizeof(T); }
+		std::string GetName() const { return "map<" + keyType->GetName() + ", " + mappedType->GetName() + ">"; }
+		size_t GetSize() const { return sizeof(T); }
 	};
 
 	// Map type
 	template<typename TKey, typename TValue>
 	struct DeduceType<std::map<TKey, TValue> > {
-		boost::shared_ptr<IType> Get() {
+		static boost::shared_ptr<IType> Get() {
 			DeduceType<TValue> valuetype;
 			DeduceType<TKey> keytype;
 			return boost::shared_ptr<IType>(new MapType<std::map<TKey, TValue> >(keytype.Get(), valuetype.Get()));
@@ -108,7 +108,7 @@ namespace creg
 	// Multimap
 	template<typename TKey, typename TValue>
 	struct DeduceType<std::multimap<TKey, TValue> > {
-		boost::shared_ptr<IType> Get() {
+		static boost::shared_ptr<IType> Get() {
 			DeduceType<TValue> valuetype;
 			DeduceType<TKey> keytype;
 			return boost::shared_ptr<IType>(new MapType<std::multimap<TKey, TValue> >(keytype.Get(), valuetype.Get()));
@@ -117,7 +117,7 @@ namespace creg
 	// Hash map
 	template<typename TKey, typename TValue>
 	struct DeduceType<SPRING_HASH_MAP<TKey, TValue> > {
-		boost::shared_ptr<IType> Get() {
+		static boost::shared_ptr<IType> Get() {
 			DeduceType<TValue> valuetype;
 			DeduceType<TKey> keytype;
 			return boost::shared_ptr<IType>(new MapType<SPRING_HASH_MAP<TKey, TValue> >(keytype.Get(), valuetype.Get()));
@@ -139,15 +139,15 @@ namespace creg
 			firstType->Serialize(s, &p.first);
 			secondType->Serialize(s, &p.second);
 		}
-		std::string GetName() { return "pair<" + firstType->GetName() + "," + secondType->GetName() + ">"; }
-		size_t GetSize() { return sizeof(T); }
+		std::string GetName() const { return "pair<" + firstType->GetName() + "," + secondType->GetName() + ">"; }
+		size_t GetSize() const { return sizeof(T); }
 	};
 
 	// std::pair
 	template<typename TFirst, typename TSecond>
 	struct DeduceType<std::pair<TFirst, TSecond> >
 	{
-		boost::shared_ptr<IType> Get() {
+		static boost::shared_ptr<IType> Get() {
 			DeduceType<TFirst> first;
 			DeduceType<TSecond> second;
 			return boost::shared_ptr<IType>(new PairType<std::pair<TFirst, TSecond> >(first.Get(), second.Get()));
