@@ -7,6 +7,7 @@
 
 #include <vector>
 #include <string>
+#include <type_traits> // std::is_polymorphic
 #include <boost/shared_ptr.hpp>
 
 #include "ISerializer.h"
@@ -24,20 +25,11 @@ namespace creg {
 	// Fundamental/basic types
 	enum BasicTypeID
 	{
-		crInt,		crUInt,
-		crShort,	crUShort,
-		crChar,		crUChar,
-		crInt64,	crUInt64,
+		crInt,
 		crFloat,
-		crDouble,
-		crBool,
 #if defined(SYNCDEBUG) || defined(SYNCCHECK)
-		crSyncedSint,   crSyncedUint,
-		crSyncedSshort, crSyncedUshort,
-		crSyncedSchar,  crSyncedUchar,
-		crSyncedFloat,
-		crSyncedDouble,
-		crSyncedBool,
+		crSyncedInt,
+		crSyncedFloat
 #endif
 	};
 
@@ -48,13 +40,12 @@ namespace creg {
 		virtual ~IType();
 
 		virtual void Serialize(ISerializer* s, void* instance) = 0;
-		virtual std::string GetName() = 0;
-		virtual size_t GetSize() = 0;
+		virtual std::string GetName() const = 0;
+		virtual size_t GetSize() const = 0;
 
-		static boost::shared_ptr<IType> CreateBasicType(BasicTypeID t);
+		static boost::shared_ptr<IType> CreateBasicType(BasicTypeID t, size_t size);
 		static boost::shared_ptr<IType> CreateStringType();
 		static boost::shared_ptr<IType> CreateObjInstanceType(Class* objectType);
-		static boost::shared_ptr<IType> CreateEnumeratedType(size_t size);
 	};
 
 	class IMemberRegistrator
@@ -70,8 +61,6 @@ namespace creg {
 		CF_Synced = 8,
 	};
 
-	struct _DummyStruct {};
-
 	/** Class member flags to use with CR_MEMBER_SETFLAG */
 	enum ClassMemberFlag {
 		CM_NoSerialize = 1, /// Make the serializers skip the member
@@ -85,7 +74,7 @@ namespace creg {
 	{
 	public:
 		ClassBinder(const char* className, unsigned int cf, ClassBinder* base,
-				IMemberRegistrator** mreg, int instanceSize, int instanceAlignment, bool hasVTable,
+				IMemberRegistrator** mreg, int instanceSize, int instanceAlignment, bool hasVTable, bool isCregStruct,
 				void (*constructorProc)(void* instance),
 				void (*destructorProc)(void* instance));
 
@@ -97,6 +86,7 @@ namespace creg {
 		int size; // size of an instance in bytes
 		int alignment;
 		bool hasVTable;
+		bool isCregStruct;
 
 		void (*constructor)(void* instance);
 		/**
@@ -150,7 +140,7 @@ namespace creg {
 		};
 
 		Class();
-		~Class();
+		virtual ~Class();
 
 		/// Returns true if this class is equal to or derived from other
 		bool IsSubclassOf(Class* other) const;
@@ -171,6 +161,11 @@ namespace creg {
 
 		bool IsAbstract() const { return (binder->flags & CF_Abstract) != 0; }
 
+		virtual void CallSerializeProc(void* object, ISerializer* s) = 0;
+		virtual void CallPostLoadProc(void* object) = 0;
+		virtual bool HasSerialize() = 0;
+		virtual bool HasPostLoad() = 0;
+
 		/// Returns all concrete classes that implement this class
 		std::vector<Class*> GetImplementations();
 
@@ -182,12 +177,44 @@ namespace creg {
 		int size;
 		int alignment;
 		Class* base;
-		void (_DummyStruct::*serializeProc)(ISerializer& s);
-		void (_DummyStruct::*postLoadProc)();
 
 		friend class ClassBinder;
 	};
 
+	template<class T>
+	class ClassStrong : public Class
+	{
+	public:
+		ClassStrong() :
+			serializeProc(NULL),
+			postLoadProc(NULL) { }
+
+		virtual ~ClassStrong() { }
+
+		virtual void CallSerializeProc(void* object, ISerializer* s)
+		{
+			(static_cast<T*>(object)->*serializeProc)(s);
+		}
+
+		virtual void CallPostLoadProc(void* object)
+		{
+			(static_cast<T*>(object)->*postLoadProc)();
+		}
+
+		virtual bool HasSerialize()
+		{
+			return !!serializeProc;
+		}
+
+		virtual bool HasPostLoad()
+		{
+			return !!postLoadProc;
+		}
+
+		void (T::*serializeProc)(ISerializer* s);
+		void (T::*postLoadProc)();
+	};
+
 // -------------------------------------------------------------------
 // Container Type templates
 // -------------------------------------------------------------------
@@ -223,8 +250,8 @@ namespace creg {
 				}
 			}
 		}
-		std::string GetName() { return elemType->GetName() + "[]"; }
-		size_t GetSize() { return sizeof(T); }
+		std::string GetName() const { return elemType->GetName() + "[]"; }
+		size_t GetSize() const { return sizeof(T); }
 	};
 
 	class StaticArrayBaseType : public IType
@@ -237,8 +264,8 @@ namespace creg {
 			: elemType(et), size(Size), elemSize(ElemSize) {}
 		~StaticArrayBaseType() {}
 
-		std::string GetName();
-		size_t GetSize() { return size * elemSize; }
+		std::string GetName() const;
+		size_t GetSize() const { return size * elemSize; }
 	};
 
 	template<typename T, int Size>
@@ -286,8 +313,8 @@ namespace creg {
 				}
 			}
 		}
-		std::string GetName() { return elemType->GetName() + "[]"; }
-		size_t GetSize() { return sizeof(T); }
+		std::string GetName() const { return elemType->GetName() + "[]"; }
+		size_t GetSize() const { return sizeof(T); }
 	};
 
 	class EmptyType : public IType
@@ -304,11 +331,11 @@ namespace creg {
 				s->Serialize(&c,1);
 			}
 		}
-		std::string GetName()
+		std::string GetName() const
 		{
 			return "void";
 		}
-		size_t GetSize() { return 0; /* size*/ } //FIXME used by CR_RESERVED(), ignored by now
+		size_t GetSize() const { return 0; /* size*/ } //FIXME used by CR_RESERVED(), ignored by now
 	};
 
 	class IgnoredType : public IType
@@ -325,11 +352,11 @@ namespace creg {
 				s->Serialize(&c,1);
 			}
 		}
-		std::string GetName()
+		std::string GetName() const
 		{
 			return "ignored";
 		}
-		size_t GetSize() { return size; }
+		size_t GetSize() const { return size; }
 	};
 }
 
@@ -355,7 +382,8 @@ namespace creg {
 	friend struct TCls##MemberRegistrator;			\
 	inline static creg::Class* StaticClass() { return binder.class_; } \
 	virtual creg::Class* GetClass() const; \
-	static const bool hasVTable = true;
+	static bool creg_hasVTable; \
+	static const bool creg_isStruct = false;
 
 /** @def CR_DECLARE_STRUCT
  * Use this to declare a structure
@@ -372,7 +400,8 @@ namespace creg {
 	friend struct TStr##MemberRegistrator;			\
 	inline static creg::Class* StaticClass() { return binder.class_; } \
 	creg::Class* GetClass() const; \
-	static const bool hasVTable = false;
+	static bool creg_hasVTable; \
+	static const bool creg_isStruct = true;
 
 /** @def CR_DECLARE_SUB
  * Use this to declare a sub class. This should be put in the class definition
@@ -389,11 +418,12 @@ namespace creg {
  * @param ctor_args constructor arguments
  */
 #define CR_BIND_DERIVED(TCls, TBase, ctor_args) \
+	bool TCls::creg_hasVTable = std::is_polymorphic<TCls>::value; \
 	creg::IMemberRegistrator* TCls::memberRegistrator=0;	\
 	creg::Class* TCls::GetClass() const { return binder.class_; } \
 	void TCls::_ConstructInstance(void* d) { new(d) MyType ctor_args; } \
 	void TCls::_DestructInstance(void* d) { ((MyType*)d)->~MyType(); } \
-	creg::ClassBinder TCls::binder(#TCls, 0, &TBase::binder, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::hasVTable, TCls::_ConstructInstance, TCls::_DestructInstance);
+	creg::ClassBinder TCls::binder(#TCls, 0, &TBase::binder, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, TCls::_ConstructInstance, TCls::_DestructInstance);
 
 /** @def CR_BIND_DERIVED_SUB
  * Bind a derived class inside another class to creg
@@ -404,11 +434,12 @@ namespace creg {
  * @param ctor_args constructor arguments
  */
 #define CR_BIND_DERIVED_SUB(TSuper, TCls, TBase, ctor_args) \
+	bool TSuper::TCls::creg_hasVTable = std::is_polymorphic<TSuper::TCls>::value; \
 	creg::IMemberRegistrator* TSuper::TCls::memberRegistrator=0;	 \
 	creg::Class* TSuper::TCls::GetClass() const { return binder.class_; }  \
 	void TSuper::TCls::_ConstructInstance(void* d) { new(d) TCls ctor_args; }  \
 	void TSuper::TCls::_DestructInstance(void* d) { ((TCls*)d)->~TCls(); }  \
-	creg::ClassBinder TSuper::TCls::binder(#TSuper "::" #TCls, 0, &TBase::binder, &TSuper::TCls::memberRegistrator, sizeof(TSuper::TCls), alignof(TCls), TCls::hasVTable, TSuper::TCls::_ConstructInstance, TSuper::TCls::_DestructInstance);
+	creg::ClassBinder TSuper::TCls::binder(#TSuper "::" #TCls, 0, &TBase::binder, &TSuper::TCls::memberRegistrator, sizeof(TSuper::TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, TSuper::TCls::_ConstructInstance, TSuper::TCls::_DestructInstance);
 
 /** @def CR_BIND
  * Bind a class not derived from CObject
@@ -417,27 +448,30 @@ namespace creg {
  * @param ctor_args constructor arguments
  */
 #define CR_BIND(TCls, ctor_args) \
+	bool TCls::creg_hasVTable = std::is_polymorphic<TCls>::value; \
 	creg::IMemberRegistrator* TCls::memberRegistrator=0;	\
 	creg::Class* TCls::GetClass() const { return binder.class_; } \
 	void TCls::_ConstructInstance(void* d) { new(d) MyType ctor_args; } \
 	void TCls::_DestructInstance(void* d) { ((MyType*)d)->~MyType(); } \
-	creg::ClassBinder TCls::binder(#TCls, 0, 0, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::hasVTable, TCls::_ConstructInstance, TCls::_DestructInstance);
+	creg::ClassBinder TCls::binder(#TCls, 0, 0, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, TCls::_ConstructInstance, TCls::_DestructInstance);
 
 #ifdef __clang__
 	// LLVM/Clang expects a different order
 	#define CR_BIND_TEMPLATE(TCls, ctor_args) \
+		template<> bool TCls::creg_hasVTable = std::is_polymorphic<TCls>::value; \
 		template<> creg::IMemberRegistrator* TCls::memberRegistrator=0; \
 		template<> void TCls::_ConstructInstance(void* d) { new(d) MyType ctor_args; } \
 		template<> void TCls::_DestructInstance(void* d) { ((MyType*)d)->~MyType(); } \
-		template<> creg::ClassBinder TCls::binder(#TCls, 0, 0, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::hasVTable, TCls::_ConstructInstance, TCls::_DestructInstance); \
+		template<> creg::ClassBinder TCls::binder(#TCls, 0, 0, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, TCls::_ConstructInstance, TCls::_DestructInstance); \
 		template<> creg::Class* TCls::GetClass() const { return binder.class_; }
 #else
 	#define CR_BIND_TEMPLATE(TCls, ctor_args) \
+		template<> bool TCls::creg_hasVTable = std::is_polymorphic<TCls>::value; \
 		template<> creg::IMemberRegistrator* TCls::memberRegistrator=0; \
 		template<> creg::Class* TCls::GetClass() const { return binder.class_; } \
 		template<> void TCls::_ConstructInstance(void* d) { new(d) MyType ctor_args; } \
 		template<> void TCls::_DestructInstance(void* d) { ((MyType*)d)->~MyType(); } \
-		template<> creg::ClassBinder TCls::binder(#TCls, 0, 0, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::hasVTable, TCls::_ConstructInstance, TCls::_DestructInstance);
+		template<> creg::ClassBinder TCls::binder(#TCls, 0, 0, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, TCls::_ConstructInstance, TCls::_DestructInstance);
 #endif
 
 /** @def CR_BIND_DERIVED_INTERFACE
@@ -447,9 +481,10 @@ namespace creg {
  * @param TBase base class of TCls
  */
 #define CR_BIND_DERIVED_INTERFACE(TCls, TBase)	\
+	bool TCls::creg_hasVTable = std::is_polymorphic<TCls>::value; \
 	creg::IMemberRegistrator* TCls::memberRegistrator=0;	\
 	creg::Class* TCls::GetClass() const { return binder.class_; } \
-	creg::ClassBinder TCls::binder(#TCls, (unsigned int)creg::CF_Abstract, &TBase::binder, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::hasVTable, 0, 0);
+	creg::ClassBinder TCls::binder(#TCls, (unsigned int)creg::CF_Abstract, &TBase::binder, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, 0, 0);
 
 /** @def CR_BIND_INTERFACE
  * Bind an abstract class
@@ -459,9 +494,10 @@ namespace creg {
  * @param TCls abstract class to bind
  */
 #define CR_BIND_INTERFACE(TCls)	\
+	bool TCls::creg_hasVTable = std::is_polymorphic<TCls>::value; \
 	creg::IMemberRegistrator* TCls::memberRegistrator=0;	\
 	creg::Class* TCls::GetClass() const { return binder.class_; } \
-	creg::ClassBinder TCls::binder(#TCls, (unsigned int)creg::CF_Abstract, 0, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::hasVTable, 0, 0);
+	creg::ClassBinder TCls::binder(#TCls, (unsigned int)creg::CF_Abstract, 0, &TCls::memberRegistrator, sizeof(TCls), alignof(TCls), TCls::creg_hasVTable, TCls::creg_isStruct, 0, 0);
 
 /** @def CR_REG_METADATA
  * Binds the class metadata to the class itself
@@ -471,7 +507,6 @@ namespace creg {
  * should consist of a series of single expression of metadata macros\n
  * for example: (CR_MEMBER(a),CR_POSTLOAD(PostLoadCallback))
  * @see CR_MEMBER
- * @see CR_ENUM_MEMBER
  * @see CR_SERIALIZER
  * @see CR_POSTLOAD
  * @see CR_MEMBER_SETFLAG
@@ -482,7 +517,8 @@ namespace creg {
 	TClass##MemberRegistrator() {				\
 		Type::memberRegistrator=this;				\
 	}												\
-	void RegisterMembers(creg::Class* class_) {		\
+	void RegisterMembers(creg::Class* class_base_) {		\
+		creg::ClassStrong<TClass>* class_ = static_cast<creg::ClassStrong<TClass>*>(class_base_); \
 		Type* null=nullptr;						\
 		(void)null; /*suppress compiler warning if this isn't used*/	\
 		Members; }									\
@@ -498,7 +534,8 @@ namespace creg {
 	TSubClass##MemberRegistrator() {				\
 		Type::memberRegistrator=this;				\
 	}												\
-	void RegisterMembers(creg::Class* class_) { \
+	void RegisterMembers(creg::Class* class_base_) { \
+		creg::ClassStrong<TSubClass>* class_ = static_cast<creg::ClassStrong<TSubClass>*>(class_base_); \
 		Type* null=nullptr; \
 		(void)null; \
 		Members; } \
@@ -518,17 +555,11 @@ namespace creg {
  * - a std::string
  * - an array
  * - a pointer/reference to a creg registered struct or class instance
- * For enumerated type members, @see CR_ENUM_MEMBER
+ * - an enum
  */
 #define CR_MEMBER(Member) \
 	class_->AddMember( #Member, creg::GetType(null->Member), offsetof_creg(Type, Member), alignof(decltype(Type::Member)))
 
-/** @def CR_ENUM_MEMBER
- * Registers a class/struct member variable with an enumerated type
- */
-#define CR_ENUM_MEMBER(Member) \
-	class_->AddMember( #Member, creg::IType::CreateEnumeratedType(sizeof(Type::Member)), offsetof_creg(Type, Member), alignof(decltype(Type::Member)))
-
 /** @def CR_IGNORED
  * Registers a member variable that isn't saved/loaded
  */
@@ -579,7 +610,7 @@ namespace creg {
 
 /** @def CR_MEMBER_SETFLAG
  * Set a flag for a class/struct member
- * This should come after the CR_MEMBER or CR_ENUM_MEMBER for the member
+ * This should come after the CR_MEMBER for the member
  * @param Member the class member variable
  * @param Flag the class member flag @see ClassMemberFlag
  * @see ClassMemberFlag
@@ -604,7 +635,7 @@ namespace creg {
  *   class
  */
 #define CR_SERIALIZER(SerializeFunc) \
-	(class_->serializeProc = (void(creg::_DummyStruct::*)(creg::ISerializer&))&Type::SerializeFunc)
+	(class_->serializeProc = &Type::SerializeFunc)
 
 /** @def CR_POSTLOAD
  * Registers a custom post-loading method for the class/struct
@@ -613,7 +644,7 @@ namespace creg {
  * There can only be one postload method per class/struct
  */
 #define CR_POSTLOAD(PostLoadFunc) \
-	(class_->postLoadProc = (void(creg::_DummyStruct::*)())&Type::PostLoadFunc)
+	(class_->postLoadProc = &Type::PostLoadFunc)
 }
 
 #endif // _CREG_H
