@@ -23,7 +23,6 @@
 #define CR_DECLARE_STRUCT(TStr)
 #define CR_DECLARE_SUB(cl)
 #define CR_BIND_DERIVED(TCls, TBase, ctor_args)
-#define CR_BIND_DERIVED(TCls, TBase, ctor_args)
 #define CR_BIND(TCls, ctor_args)
 #define CR_BIND_DERIVED_INTERFACE(TCls, TBase)
 #define CR_BIND_INTERFACE(TCls)
@@ -31,7 +30,8 @@
 #define CR_REG_METADATA(TClass, Members)
 #define CR_REG_METADATA_SUB(TSuperClass, TSubClass, Members)
 #define CR_MEMBER(Member)
-#define CR_ENUM_MEMBER(Member)
+#define CR_IGNORED(Member)
+#define CR_MEMBER_UN(Member)
 #define CR_RESERVED(Size)
 #define CR_SETFLAG(Flag)
 #define CR_MEMBER_SETFLAG(Member, Flag)
