@@ -25,10 +25,6 @@ class float3
 {
 public:
 	CR_DECLARE_STRUCT(float3)
-/*	void* operator new(size_t size) { return mempool.Alloc(size); }
-	void* operator new(size_t n, void* p) { return p; } // cp visual
-	void operator delete(void* p, size_t size) { mempool.Free(p, size); }
-*/
 
 #if (__cplusplus > 199711L) || defined(__GXX_EXPERIMENTAL_CXX0X__)
 	static constexpr float CMP_EPS = 1e-4f;
@@ -647,7 +643,7 @@ public:
 	/**
 	 * @brief max x pos
 	 *
-	 * Static value containing the maximum x position (:= gs->mapx-1)
+	 * Static value containing the maximum x position (:= mapDims.mapx-1)
 	 * @note maxxpos is set after loading the map.
 	 */
 	static float maxxpos;
@@ -655,7 +651,7 @@ public:
 	/**
 	 * @brief max z pos
 	 *
-	 * Static value containing the maximum z position (:= gs->mapy-1)
+	 * Static value containing the maximum z position (:= mapDims.mapy-1)
 	 * @note maxzpos is set after loading the map.
 	 */
 	static float maxzpos;
@@ -663,7 +659,7 @@ public:
 	/**
 	 * @brief Check against FaceHeightmap bounds
 	 *
-	 * Check if this vector is in bounds [0 .. gs->mapxy-1]
+	 * Check if this vector is in bounds [0 .. mapDims.mapxy-1]
 	 * @note THIS IS THE WRONG SPACE! _ALL_ WORLD SPACE POSITIONS SHOULD BE IN VertexHeightmap RESOLUTION!
 	 * @see #IsInMap
 	 */
@@ -671,7 +667,7 @@ public:
 	/**
 	 * @brief Check against FaceHeightmap bounds
 	 *
-	 * Check if this vector is in map [0 .. gs->mapxy]
+	 * Check if this vector is in map [0 .. mapDims.mapxy]
 	 * @note USE THIS!
 	 */
 	bool IsInMap() const;
@@ -679,7 +675,7 @@ public:
 	/**
 	 * @brief Clamps to FaceHeightmap
 	 *
-	 * Clamps to the `face heightmap` resolution [0 .. gs->mapxy-1]
+	 * Clamps to the `face heightmap` resolution [0 .. mapDims.mapxy-1] * SQUARE_SIZE
 	 * @note THIS IS THE WRONG SPACE! _ALL_ WORLD SPACE POSITIONS SHOULD BE IN VertexHeightmap RESOLUTION!
 	 * @deprecated  use ClampInMap instead, but see the note!
 	 * @see #ClampInMap
@@ -689,7 +685,7 @@ public:
 	/**
 	 * @brief Clamps to VertexHeightmap
 	 *
-	 * Clamps to the `vertex heightmap`/`opengl space` resolution [0 .. gs->mapxy]
+	 * Clamps to the `vertex heightmap`/`opengl space` resolution [0 .. mapDims.mapxy] * SQUARE_SIZE
 	 * @note USE THIS!
 	 */
 	void ClampInMap();
