@@ -8,3 +8,9 @@ CR_REG_METADATA(int2, (CR_MEMBER(x), CR_MEMBER(y)))
 
 CR_BIND_TEMPLATE(float2, )
 CR_REG_METADATA(float2, (CR_MEMBER(x), CR_MEMBER(y)))
+
+CR_BIND_TEMPLATE(short2, )
+CR_REG_METADATA(short2, (CR_MEMBER(x), CR_MEMBER(y)))
+
+CR_BIND_TEMPLATE(ushort2, )
+CR_REG_METADATA(ushort2, (CR_MEMBER(x), CR_MEMBER(y)))
