@@ -1,12 +1,13 @@
 /* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include <map>
-#include <boost/thread/recursive_mutex.hpp>
 #include <boost/thread.hpp>
 
 #include "LuaInclude.h"
+#include "Game/GameVersion.h"
 #include "Lua/LuaHandle.h"
 #include "System/Platform/Threading.h"
+#include "System/Threading/SpringMutex.h"
 #include "System/Log/ILog.h"
 #if (!defined(DEDICATED) && !defined(UNITSYNC) && !defined(BUILDING_AI))
 	#include "System/Misc/SpringTime.h"
@@ -16,24 +17,30 @@
 ///////////////////////////////////////////////////////////////////////////
 // Custom Lua Mutexes
 
-static std::map<lua_State*, boost::recursive_mutex*> mutexes;
+static std::map<lua_State*, spring::recursive_mutex*> mutexes;
 static std::map<lua_State*, bool> coroutines;
 
-static boost::recursive_mutex* GetLuaMutex(lua_State* L)
+static spring::recursive_mutex* GetLuaMutex(lua_State* L)
 {
 	assert(!mutexes[L]);
-	return new boost::recursive_mutex();
+	return new spring::recursive_mutex();
 }
 
 
 
 void LuaCreateMutex(lua_State* L)
 {
+	#if (ENABLE_USERSTATE_LOCKS == 0)
+	// if LoadingMT=1, everything runs in the game-load thread (on startup)
+	assert(Threading::IsMainThread() || Threading::IsGameLoadThread() || SpringVersion::IsUnitsync());
+	return;
+	#endif
+
 	luaContextData* lcd = GetLuaContextData(L);
 	if (!lcd) return; // CLuaParser
 	assert(lcd);
 
-	boost::recursive_mutex* mutex = GetLuaMutex(L);
+	spring::recursive_mutex* mutex = GetLuaMutex(L);
 	lcd->luamutex = mutex;
 	mutexes[L] = mutex;
 }
@@ -41,6 +48,11 @@ void LuaCreateMutex(lua_State* L)
 
 void LuaDestroyMutex(lua_State* L)
 {
+	#if (ENABLE_USERSTATE_LOCKS == 0)
+	assert(Threading::IsMainThread() || Threading::IsGameLoadThread() || SpringVersion::IsUnitsync());
+	return;
+	#endif
+
 	if (!GetLuaContextData(L)) return; // CLuaParser
 	assert(GetLuaContextData(L));
 
@@ -50,7 +62,7 @@ void LuaDestroyMutex(lua_State* L)
 	} else {
 		lua_unlock(L);
 		assert(mutexes.find(L) != mutexes.end());
-		boost::recursive_mutex* mutex = GetLuaContextData(L)->luamutex;
+		spring::recursive_mutex* mutex = GetLuaContextData(L)->luamutex;
 		assert(mutex);
 		delete mutex;
 		mutexes.erase(L);
@@ -61,6 +73,11 @@ void LuaDestroyMutex(lua_State* L)
 
 void LuaLinkMutex(lua_State* L_parent, lua_State* L_child)
 {
+	#if (ENABLE_USERSTATE_LOCKS == 0)
+	assert(Threading::IsMainThread() || Threading::IsGameLoadThread() || SpringVersion::IsUnitsync());
+	return;
+	#endif
+
 	luaContextData* plcd = GetLuaContextData(L_parent);
 	assert(plcd);
 
@@ -76,9 +93,14 @@ void LuaLinkMutex(lua_State* L_parent, lua_State* L_child)
 
 void LuaMutexLock(lua_State* L)
 {
+	#if (ENABLE_USERSTATE_LOCKS == 0)
+	assert(Threading::IsMainThread() || Threading::IsGameLoadThread() || SpringVersion::IsUnitsync());
+	return;
+	#endif
+
 	if (!GetLuaContextData(L)) return; // CLuaParser
 
-	boost::recursive_mutex* mutex = GetLuaContextData(L)->luamutex;
+	spring::recursive_mutex* mutex = GetLuaContextData(L)->luamutex;
 
 	if (mutex->try_lock())
 		return;
@@ -91,15 +113,25 @@ void LuaMutexLock(lua_State* L)
 
 void LuaMutexUnlock(lua_State* L)
 {
+	#if (ENABLE_USERSTATE_LOCKS == 0)
+	assert(Threading::IsMainThread() || Threading::IsGameLoadThread() || SpringVersion::IsUnitsync());
+	return;
+	#endif
+
 	if (!GetLuaContextData(L)) return; // CLuaParser
 
-	boost::recursive_mutex* mutex = GetLuaContextData(L)->luamutex;
+	spring::recursive_mutex* mutex = GetLuaContextData(L)->luamutex;
 	mutex->unlock();
 }
 
 
 void LuaMutexYield(lua_State* L)
 {
+	#if (ENABLE_USERSTATE_LOCKS == 0)
+	assert(Threading::IsMainThread() || Threading::IsGameLoadThread() || SpringVersion::IsUnitsync());
+	return;
+	#endif
+
 	assert(GetLuaContextData(L));
 	/*mutexes[L]->unlock();
 	if (!mutexes[L]->try_lock()) {
