@@ -3,6 +3,8 @@
 #ifndef SPRING_LUA_USER_H
 #define SPRING_LUA_USER_H
 
+#include "lua.h"
+
 extern void LuaCreateMutex(lua_State* L);
 extern void LuaDestroyMutex(lua_State* L);
 extern void LuaLinkMutex(lua_State* L_parent, lua_State* L_child);
@@ -10,6 +12,7 @@ extern void LuaMutexLock(lua_State* L);
 extern void LuaMutexUnlock(lua_State* L);
 extern void LuaMutexYield(lua_State* L);
 
+extern const char* spring_lua_getName(lua_State* L);
 
 struct SLuaInfo {
 	unsigned int allocedBytes;
